<?php

namespace Avos\SQLite;

//
// calc the filename of the sqlite database file and create if needed
//

function getDatabase($path)
{
	global $_AVOS;
	$perfile = 100;
	$perfolder = 1000;

	if (preg_match('!(\d+)!', $path, $c)) {

		$webcamsid = $c[1];
		$top = $_AVOS['config']['site']['avosDir'];

		#$fileidx = (((int)($c[1] / $perdb) + 1 ) * $perdb);
		#$subdir = $fileidx / $perdb;
		#$fullpath = $top . 'db/' . $subdir . '/';

		$file = (int)($webcamsid / $perfile) % $perfolder;
		$folder = (int)($webcamsid / $perfile / $perfolder);
		$fullpath = $top . "db/$folder/";
		$dbname = $fullpath . $file . '.db';

		//
		// if not cli and file doesn't exist return false
		//

		if (PHP_SAPI != 'cli' && !file_exists($dbname)) {
			return false;
		}

		//
		// create subdir for database if needed
		//

		if (!file_exists($fullpath)) {
			if (mkdir($fullpath, 0755) == false) {
				return false;
			}
		}

		//
		// check schema and recreate if empty
		//

		$renew = false;

		$db = new \SQLite3($dbname);
		$db->busyTimeout(5000);
		$db->exec('PRAGMA journal_mode = wal;');

		$dbs = $db->query("PRAGMA table_info(webcams);");
		$res = $dbs->fetchArray(SQLITE3_ASSOC);
		if (empty($res)) {
			$renew = true;
		}

		//
		// create database
		//

		if ($renew || !file_exists($dbname)) {

			//
			// create the schema if a new database
			//

			#$db = new \SQLite3($dbname);
			#$db->busyTimeout(5000);
			#$db->exec('PRAGMA journal_mode = wal;');
			$db->exec('
				CREATE TABLE webcams (
					path TEXT NOT NULL PRIMARY KEY,
					data BLOB
				)'
			);

			chmod($fullpath, 0777);
			chmod($dbname, 0777);

			return $db;
		}
		else {

			//
			// database exists so just return a db handle
			//

			$db = new \SQLite3($dbname);
			$db->busyTimeout(5000);
			$db->exec('PRAGMA journal_mode = wal;');
			return $db;
		}
	}
	else {
		return false;
	}
}

//
// set data
//

function set($path, $data)
{
	$db = getDatabase($path);

	//
	// should only return false if $path is malformed
	//

	if ($db == false) {
		return false;
	}

	//
	// INSERT or UPDATE the entry
	//

	$json = gzdeflate(json_encode($data));
	$chk = $db->querySingle("SELECT path FROM webcams WHERE path='$path'");

	if ($chk === NULL) {
		#$db->exec("INSERT INTO webcams (path, data) VALUES ('$path', '$json')");
		$dbs = $db->prepare("INSERT INTO webcams (path, data) VALUES (:path, :json)");
	}
	else {
		#$db->exec("UPDATE webcams SET data='$json' WHERE path='$path'");
		$dbs = $db->prepare("UPDATE webcams SET data=:json WHERE path=:path");
	}

	$dbs->bindValue(':path', $path, SQLITE3_TEXT);
	$dbs->bindValue(':json', $json, SQLITE3_BLOB);
	$res = $dbs->execute();
	$res->finalize();
	$db->close();
	unset($db);

	return true;
}

//
// get data
//

function get($path)
{
	$db = getDatabase($path);

	//
	// should only return false if $path is malformed
	//

	if ($db == false) {
		return false;
	}

	// could cache prepared statements like this to reuse
	$dbs = $db->prepare("SELECT data FROM webcams WHERE path=:path");
	$dbs->bindValue(':path', $path, SQLITE3_TEXT);
	$res = $dbs->execute();
	$data = $res->fetchArray(SQLITE3_ASSOC);
	$res->finalize();
	$db->close();
	unset($db);

	//
	// check we found the data
	//

	if (!isset($data['data'])) {
		return array();
	}

	$data = json_decode(gzinflate($data['data']), true);

	return $data;
}

//
// search for keys
//

function search($path)
{
	$db = getDatabase($path);

	//
	// should only return false if $path is malformed
	//

	if ($db == false) {
		return false;
	}

	//
	// allow searching with * as wildcard like Redis so convert to % for sqlite
	//

	$path = str_replace('*', '%', $path);

	//
	// fetch all matching keys
	//

	$res = $db->query("SELECT path FROM webcams WHERE path LIKE '$path'");
	$ret = array();

	while ($row = $res->fetchArray()) {
		$ret[] = $row['path'];
	}

	$db->close();
	unset($db);

	return $ret;
}

//
// delete a single item
//


function del($path)
{
	$db = getDatabase($path);

	//
	// should only return false if $path is malformed
	//

	if ($db == false) {
		return false;
	}

	$dbs = $db->prepare("DELETE FROM webcams WHERE path=:path");
	$dbs->bindValue(':path', $path, SQLITE3_TEXT);
	$res = $dbs->execute();
	$res->finalize();
	$db->close();
	unset($db);

	return true;
}
