<?
namespace Avos\Cron;

//
// online status:
//
// o: offline
// a: away
// f: freechat
// g: group
// p: private
//

$_cron_start_time = 0;
$_module = '';

function debugModule($module)
{
	global $_module;
	$_module = $module;
}

function debug($str)
{
	global $_module;

	echo date('ymd H:i:s') . ' ';
	if (!empty($_module)) echo "[$_module] ";
	else echo "* ";

    echo "$str\n";
    flush();
}

function online_status_update($site)
{
	global $_AVOS;

	$time = time();
	$now = (int)(($time - EPOCHSTART) / 60);
	$omodels = array();

	$added = 0;
	$changed = 0;
	$unarchived = 0;

	$models = call_user_func('\\Sites\\' . $site . '\\online_status');

	// dont update status if the feed returns an error

	if ($models === false) {
		\Avos\Cron\debug("error from feed: not modifying models status");
		return;
	}

	$site_short = call_user_func('\\Sites\\' . $site . '\\get_shortname');
	$model_info_cb = '\\Sites\\' . $site . '\\get_model';

	if (!function_exists($model_info_cb)) {
		$model_info_cb = false;
	}

	\Avos\Cron\debug("processing " . count($models) . " online models");

	$genderRank = array();
	$onlineCount = 0;

	$yymm = date('ym', $time);
	
	foreach ($models as $i => $m) {
	
		if (DEBUG) \Avos\Cron\debug("$m[site] $m[nickname] $m[status]");
		if ($m['status'] != 'o') $onlineCount++;

		//
		// data fixup
		//

		foreach (array('subject') as $f) {
			if (!isset($m[$f])) {
				$m[$f] = '';
			}
		}

		foreach (array('viewers', 'followers', 'age', 'hd') as $f) {
			if (!isset($m[$f])) {
				$m[$f] = 0;
			}
		}

		$m['age'] = (int)$m['age'];
		#if ($m['age'] == 0) $m['age'] = 99;
        if ($m['age'] == 0 && strpos($m['tags'], 'teen') !== false) $m['age'] = 19;

		$m['subject'] = str_replace('\\', ' ', $m['subject']);
		$m['subject'] = substr($m['subject'], 0, 255);
		$m['subject'] = \Avos\Utils\convert_ascii($m['subject']);
        $m['name'] = str_replace('\\', ' ', $m['name']);

		if (!empty($m['location'])) {
			$m['location'] = substr(iconv('UTF-8', 'ASCII//TRANSLIT', $m['location']), 0, 32);
		}
		else {
			$m['location'] = '';
		}

		if (!empty($m['languages'])) {
			$m['languages'] = substr($m['languages'], 0, 128);
		}
		else {
			$m['languages'] = '';
		}

        if (!isset($m['body'])) $m['body'] = '';

        if (!empty($m['birthday'])) {
		$m['birthday'] = strtotime($m['birthday']);
		if ($m['birthday'] < 0) {
			$m['birthday'] = 0;
		}
        }
        else {
            $m['birthday'] = 0;
        }

        if ($m['viewers'] < 0) {
            // bongacams
            $m['viewers'] = 0;
        }

		//
		// we need tags to start and end with a comma
		//

		if (!empty($m['tags'])) {
			if (substr($m['tags'], -1) != ',') {
				$m['tags'] .= ',';
			}

			if ($m['tags'][0] != ',') {
				$m['tags'] = ',' . $m['tags'];
			}
		}

		//
		// site and site/gender ranks
		//

		$m['rank'] = $i + 1;

		if (isset($genderRank[ $m['gender'] ])) {
			$genderRank[ $m['gender'] ] ++;
		}
		else {
			$genderRank[ $m['gender'] ] = 1;
		}

		$m['genderRank'] = $genderRank[ $m['gender'] ];

		if ($site_short == 'sm') {
			$m['rank'] = 0;
			$m['genderRank'] = 0;
		}
	
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// fetch existing model id and add if needed
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
		$chk = \Avos\Database\queryList('SELECT id, updatedon FROM webcams WHERE nickname=:nickname AND site=:site', $m);
		$updatedon = 0;

		//
		// if doesnt exist next check the webcamsArchive table
		//

		if (count($chk) == 0) {
			$id = \Avos\Cron\unarchive_model($m);

			// skip if the model has been hidden
			if ($id == -1) {
				continue;
			}

			if ($id !== false) {
				$chk = array( $id );
				$unarchived++;
			}
		}
			
		//
		// insert the model if we dont have an entry
		//
	
		if (count($chk) > 0) {
			$id = $chk[0];
			#$updatedon = $chk[1];
			if (DEBUG) \Avos\Cron\debug(" exists, id=$id");
		}
		else {
			#echo "adding ($m[nickname])\n";

			// get more info if needed
			if ($model_info_cb) {
				$extra = call_user_func($model_info_cb, $m);

				foreach ($extra as $k => $v) {
					$m[$k] = $v;
				}
			}

			$id = \Avos\Database\query('
			INSERT INTO webcams (addedOn, updatedOn, nickname, name, site, status, statusTime, subject, age, imageUrl, viewers, followers, gender, hd, rank, genderRank, tags, birthday, location, languages, body, earnedForGoal, neededForGoal, broadcastVR)
			VALUES (UNIX_TIMESTAMP(), UNIX_TIMESTAMP(), :nickname, :name, :site, :status, 0, :subject, :age, :imageUrl, :viewers, :followers, :gender, :hd, :rank, :genderRank, :tags, :birthday, :location, :languages, :body, :earnedForGoal, :neededForGoal, :broadcastVR)		
				', $m);
	
			if (DEBUG) echo "  added, id=$id\n";
			$added++;

			// to decide whether to store rank/viewers/etc info
			if ($m['status'] == 'f') {
				\Avos\Redis\setModel($id, 'ft', $now);
			}
		}

		#if (preg_match('!worshipmis!', $m['nickname'])) { echo "BUGGERED = "; print_r($m); exit; }

		//
		// for quick hash lookup via webcamsId to see who is online
		//
	
		$omodels[$id] = 1;
	
		//
		// merge database id with the model info
		//
	
		$m['epochTime'] = $time;
		$m['webcamsId'] = $id;
		$models[$i]['webcamsId'] = $id;
		$id = 0;

		#if ($m['webcamsId'] == 89135) \Avos\Cron\debug("model $m[webcamsId] = $m[status]"); #FISH

		//
		// set rank info - streamate returns models in alphabetic order
		//

		$freechatStart = \Avos\Redis\getModel($m['webcamsId'], 'ft');
		$freechatMins = $now - $freechatStart;
		$freechatDelay = $_AVOS['config']['webcams']['delayRankSaving'];

		#echo "$m[webcamsId] now=$now freechatStart=$freechatStart freechatMins=$freechatMins\n";

		if ($site_short != 'sm' && $m['status'] == 'f' && $freechatMins > $freechatDelay) {
			$lastRankTime = \Avos\Redis\getModel($m['webcamsId'], 'lastRankTime');
			\Avos\Redis\setModel($m['webcamsId'], 'rank', $m['rank']);
			\Avos\Redis\setModel($m['webcamsId'], 'genderRank', $m['genderRank']);

			if (empty($lastRankTime) || $lastRankTime + $_AVOS['config']['webcams']['refreshRankInterval'] < $now) {
				 \Avos\Redis\setModel($m['webcamsId'], 'lastRankTime', $now);
				 \Avos\Redis\setModel($m['webcamsId'], "r/$yymm", 
				 	array(
						'e' => $now, 
						'r' => $m['rank'], 
						'g' => $m['genderRank'],
						'v' => $m['viewers'],
						'f' => $m['followers']
						)
					, true);
			}
		}

		//
		// store any extra info in redis if we have it
		//

		foreach (array('externalId', 'streamUrl', 'photos', 'blocked') as $f) {
			if (!empty($m[$f])) {
                \Avos\Redis\setModel($m['webcamsId'], $f, $m[$f]);
                #\Avos\Cron\debug("$m[site]/$m[nickname]/$f: " . $m[$f]);
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// fetch last history record
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
		$hid = 0;
		$chk = \Avos\Database\queryAssoc('SELECT * FROM history WHERE webcamsId=:webcamsId AND statusTime=0 ORDER BY id DESC LIMIT 1', $m);
	
		if (count($chk) > 0) {
	
			$history = $chk[0];
	
			//
			// check if status has changed
			//
	
			if ($history['status'] != $m['status']) {
	
				$history['statusTime'] = $time - $history['epochTime'];
				if (DEBUG) echo "  CHG: model $m[webcamsId] has changed from " . $history['status'] . " (" . $history['statusTime'] . "s) to $m[status]\n";
				$changed++;

				//
				// no clue why - but limit stattsTime
				//

				if ($history['statusTime'] > 86400) $history['statusTime'] = 86400;
	
				\Avos\Database\query('
					UPDATE history SET statusTime=:statusTime WHERE id=:id
					', $history);

				// to decide whether to store rank/viewers/etc info
				if ($m['status'] == 'f') {
					\Avos\Redis\setModel($m['webcamsId'], 'ft', $now);
				}
			}
			else {
				$hid = $history['id'];
				if (DEBUG) echo "  found existing status of $m[status]\n";
			}
		}
	
		//
		// add new history element if new model or status has changed and update the model status
		//

        // only update body if we have data as for cb needs to come from scraping
        $body = '';
        if (!empty($m['body'])) {
            $body = ", body=:body";
        }

		if ($hid == 0) {

			#if ($m['webcamsId'] == 0) { print_r($m); exit; }

			\Avos\Database\query('
				INSERT INTO history (webcamsId, epochTime, status)
				VALUES (:webcamsId, :epochTime, :status)
				', $m);

			\Avos\Database\query("
				UPDATE webcams 
				SET 
					status=:status, imageUrl=:imageUrl, updatedOn=UNIX_TIMESTAMP(), gender=:gender, age=:age,
                    viewers=:viewers, followers=:followers, rank=:rank, genderRank=:genderRank,
                    subject=:subject, tags=:tags, location=:location, languages=:languages ,earnedForGoal = :earnedForGoal,neededForGoal = :neededForGoal,broadcastVR = :broadcastVR $body
				WHERE id=:webcamsId
				", $m);
	
			if (DEBUG) echo "  added new status of $m[status]\n";
		}
		elseif ($updatedon + $_AVOS['config']['webcams']['refreshViewersInterval'] * 60 < $time) {

			//
			// if no new status just update followers, etc
			//

			if (DEBUG) echo "$m[site] $m[nickname] setting rank to $m[rank]\n";

			\Avos\Database\query("
				UPDATE webcams 
				SET 
					viewers=:viewers, followers=:followers,
					rank=:rank, genderRank=:genderRank, gender=:gender, age=:age,
					updatedon=$time, imageUrl=:imageUrl
				WHERE id=:webcamsId
				", $m);
		}
	}

	#\Avos\Cron\debug("found $onlineCount models online");
	\Avos\Cron\debug("added $added models, unarchived $unarchived and $changed have changed online status");

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// set statusTime for any models no longer online
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	$chk = \Avos\Database\queryAssoc('
				SELECT h.* 
				FROM history h 
				LEFT JOIN webcams w ON h.webcamsId=w.id 
				WHERE h.statusTime=0 AND w.site=:site
			', 
			array('site' => $site_short)
		);

	$count = 0;

	foreach ($chk as $m) {
		if (!isset($omodels[$m['webcamsId']])) {
	
			if (DEBUG) echo "CHG: model id $m[webcamsId] logged off\n";
			$count++;
	
			//
			// update model entry
			//
	
			$m['status'] = 'o';
			\Avos\Database\query('UPDATE webcams SET status=:status, updatedOn=UNIX_TIMESTAMP(), rank=0 WHERE id=:webcamsId', $m);
	
			//
			// update history entry with statusTime
			//
	
			$m['statusTime'] = $time - $m['epochTime'];
			\Avos\Database\query('UPDATE history SET statusTime=:statusTime WHERE id=:id', $m);

			//
			// move to archive?
			//

			#archive_models($m['webcamsId']);
		}
	}

	\Avos\Cron\debug("$count models have gone offline");

	//
	// not sure why but models are being left online
	// 

	/*
	$count=0;
	$ass = \Avos\Database\queryList("SELECT id FROM webcams WHERE site=:site AND status!='o'", array('site' => $site_short));

	foreach ($ass as $a) {
		if (!isset($omodels[$a])) {
			\Avos\Database\query("UPDATE webcams SET status='o' WHERE id=:id", array('id' => $a));
			$count++;
		}
	}

	\Avos\Cron\debug("hack... $count models set to offline");
	*/

	//
	// make sure we have a photo
	//

	#download_photos($site, $site_short);
}



//
//ownload_photos
//
function download_photos($site, $site_short)
{
    global $_AVOS;

    $urltop = strtolower($site) . '/';
    $topRelative = 'photos/' . strtolower($site) . '/';
    $top = WEB . $topRelative;

    $ctx = stream_context_create([
        'http' => ['timeout' => 20]
    ]);

    if (!file_exists($top)) {
        mkdir($top, 0755, true);
    }

    $interval = $_AVOS['config']['webcams']['refreshImagesInterval'] * 3600;
    $cwebpCmd = $_AVOS['config']['tools']['cwebp'];
    $cwebpQuality = (int)$_AVOS['config']['tools']['cwebpQuality'];

    $useJpegImages = $_AVOS['config']['webcams']['useJpegImages'];
    $useWebpImages = $_AVOS['config']['webcams']['useWebpImages'];

    $models = \Avos\Database\queryAssoc('
        SELECT id, site, nickname, imageUrl, numImages
        FROM webcams 
        WHERE site=:site AND status!="o" AND (numImages = 0 OR lastImageTime < UNIX_TIMESTAMP() - (' . $interval . ' * (100+numImages)/20-4) )',
        ['site' => $site_short]);

// Filter for a specific model - see if is getting the images
  //  $models = array_filter($models, function ($model) {
    //   return strtolower($model['nickname']) === 'fanta2k6'; 
    //});

    \Avos\Cron\debug("Downloading photos for " . count($models) . " models");

    foreach ($models as $m) {
        $nickname = strtolower($m['nickname']);

        // Fetch image content
        $imageContent = @file_get_contents($m['imageUrl'], false, $ctx);
        if ($imageContent === false) {
            \Avos\Cron\debug("Failed to fetch image for $nickname: {$m['imageUrl']}");
            continue;
        }

		// stripchat doesnt use a .jpg etc file extension
		//$ext = 'jpg';
        // Detect MIME type and set file extension
        $imageType = getimagesizefromstring($imageContent)['mime'];
        switch ($imageType) {
            case 'image/jpeg':
                $ext = 'jpg';
                break;
            case 'image/png':
                $ext = 'png';
                break;
            case 'image/gif':
                $ext = 'gif';
                break;
            case 'image/webp':
                $ext = 'webp';
                break;
            default:
                \Avos\Cron\debug("Unsupported image type for $nickname: $imageType");
                continue;
        }

        if (preg_match('!^(.)(.)(.)(.)!', $nickname, $c)) {
            $folder = strtolower("$c[1]/$c[2]/$c[3]/$c[4]/$nickname/");

            if (!file_exists($top . $folder)) {
                mkdir($top . $folder, 0755, true);
            }

            $image = ($m['numImages'] + 1) . '.' . $ext;
            $local = $top . $folder . $image;

			//print_r( $local);exit;

            if (file_put_contents($local, $imageContent) !== false) {
                \Avos\Cron\debug("Downloaded image for $nickname: $local");

                if ($useWebpImages && $ext !== 'webp') {
                    $webp = $top . $folder . ($m['numImages'] + 1) . '.webp';
                    if (@system("$cwebpCmd -quiet -q $cwebpQuality $local -o $webp") !== false) {
                        if (!$useJpegImages) {
                            unlink($local); // Remove original if only WebP is needed
                        }
                        cdnSync($topRelative . $folder);
                        \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=numImages+1 WHERE id=:id', $m);
                    } else {
                        unlink($local); // Remove if WebP conversion failed
                    }
                } else {
                    cdnSync($topRelative . $folder);
                    \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=numImages+1 WHERE id=:id', $m);
                }
            } else {
                \Avos\Cron\debug("Failed to save image for $nickname: $local");
            }
        }
    }
}


//
// verify_photos the munbars 
//



function verify_photos($m, $want = 30, $debug = 0)
{
    global $_AVOS;

    $cwebp = $_AVOS['config']['tools']['cwebp'];
    $imagePath = \Avos\Utils\imagePath($m);

    if (empty($imagePath)) {
        if ($debug) {
            echo "Error: No image path for model '{$m['nickname']}'\n";
        }
        return;
    }

    if ($debug) {
        echo "Processing {$m['site']}:{$m['nickname']} with {$m['numImages']} images at $imagePath\n";
    }

    $dir = WEB . $imagePath;
    $images = \Avos\Utils\list_files($dir);
    $count = count($images);

    if ($count === 0) {
        \Avos\Database\query('UPDATE webcams SET lastImageTime=0, numImages=0 WHERE id=:id', $m);
        \Avos\Database\query('UPDATE webcamsArchive SET lastImageTime=0, numImages=0 WHERE id=:id', $m);
        return;
    }

    // Extract numeric parts of filenames for sorting
    $numericParts = [];
    foreach ($images as $file) {
        if (preg_match('/(\d+)/', $file, $matches)) {
            $numericParts[] = (int)$matches[1];
        }
    }

    sort($numericParts);
    $images = array_unique($numericParts);

    if ($debug) {
        echo "Sorted image indices: " . implode(", ", $images) . "\n";
    }

    // Ensure all files are converted to WebP
    foreach ($images as $index => $image) {
        $webpFile = "$dir$image.webp";

        if (!file_exists($webpFile)) {
            if ($debug) {
                echo "Creating $image.webp\n";
            }

            foreach (['jpg', 'png', 'gif'] as $ext) {
                $originalFile = "$dir$image.$ext";
                if (file_exists($originalFile)) {
                    system("$cwebp -quiet -q 80 $originalFile -o $webpFile");
                    unlink($originalFile); // Remove original file after conversion
                }
            }
        }

        // If conversion failed, exclude the image
        if (!file_exists($webpFile)) {
            unset($images[$index]);
        }
    }

    // Remove any leftover non-WebP images
    foreach (['jpg', 'png', 'gif'] as $ext) {
        foreach ($images as $image) {
            $originalFile = "$dir$image.$ext";
            if (file_exists($originalFile)) {
                if ($debug) {
                    echo "Removing leftover file: $image.$ext\n";
                }
                unlink($originalFile);
            }
        }
    }

    // Limit the number of images to $want
    if (count($images) > $want) {
        $removeCount = count($images) - $want;

        // Calculate indices to remove evenly
        $indicesToRemove = [];
        $step = count($images) / $removeCount;

        for ($i = 0; $i < $removeCount; $i++) {
            $indicesToRemove[] = floor($i * $step);
        }

        // Remove selected images
        foreach ($indicesToRemove as $index) {
            $imageToRemove = $images[$index];
            if ($debug) {
                echo "Removing excess image: $imageToRemove.webp\n";
            }
            unlink("$dir$imageToRemove.webp");
            unset($images[$index]);
        }

        // Re-index the array after removal
        $images = array_values($images);
    }

    // Rename images to have contiguous numbering
    $images = array_values($images); // Re-index
    foreach ($images as $index => $image) {
        $targetFile = "$dir" . ($index + 1) . ".webp";
        $currentFile = "$dir$image.webp";

        if ($currentFile !== $targetFile) {
            if ($debug) {
                echo "Renaming $currentFile to $targetFile\n";
            }
            rename($currentFile, $targetFile);
        }
    }

    // Update the database
    $finalCount = count($images);
    $m['want'] = $finalCount;

    \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=:want WHERE id=:id', $m);
    \Avos\Database\query('UPDATE webcamsArchive SET lastImageTime=UNIX_TIMESTAMP(), numImages=:want WHERE id=:id', $m);

    if ($debug) {
        echo "Final image count: $finalCount\n";
    }
}









//olllddddddd


/* 

function download_photos($site, $site_short)
{
	global $_AVOS;

	$urltop = strtolower($site) . '/';
	$topRelative = 'photos/' . strtolower($site) . '/';
	$top = WEB . $topRelative;

	$ctx = stream_context_create(array(
		'http' => array(
			'timeout' => 20
		)
	));

	if (!file_exists($top)) {
		mkdir($top, 0755, true);
	}

	$intervalHourly = 3600;
	$sexy = " 
				OR (nickname='girlswannasex' AND lastImageTime < UNIX_TIMESTAMP() - $intervalHourly) 
				OR (nickname='carolyngross' AND lastImageTime < UNIX_TIMESTAMP() - $intervalHourly) 
		";
	$sexy = "";

	$interval = $_AVOS['config']['webcams']['refreshImagesInterval'] * 3600;
	$cwebpCmd = $_AVOS['config']['tools']['cwebp'];
	$cwebpQuality = (int)$_AVOS['config']['tools']['cwebpQuality'];

	$useJpegImages = $_AVOS['config']['webcams']['useJpegImages'];
	$useWebpImages = $_AVOS['config']['webcams']['useWebpImages'];

	$models = \Avos\Database\queryAssoc('
					SELECT id, site, nickname, imageUrl, numImages
					FROM webcams 
					WHERE site=:site AND status!="o" AND (numImages = 0 OR lastImageTime < UNIX_TIMESTAMP() - (' . $interval . ' * (100+numImages)/20-4) )', 
						array('site' => $site_short));

	\Avos\Cron\debug("downloading " . count($models) . " photos");

	foreach ($models as $m) {

		# jasmin only has 1 photo
		if ($site_short == 'lj' && $m['numImages'] == 1) {
			\Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP() WHERE id=:id', $m);
			continue;
		}

		# nickname must be already urlsafe - name might not be
		#$nickname = \Avos\Utils\urlsafe(strtolower($m['nickname']), '_');
		$nickname = strtolower($m['nickname']);

		// stripchat doesnt use a .jpg etc file extension
		$ext = 'jpg';

		if (preg_match('!\.(jpg|jpeg|gif|png)$!i', $m['imageUrl'], $e)) {
			$ext = $e[1];
        }
        else {
            continue;
        }

		if (preg_match('!^(.)(.)(.)(.)!', $nickname, $c)) {

			$folder = strtolower("$c[1]/$c[2]/$c[3]/$c[4]/$nickname/");
	
			if (!file_exists($top . $folder)) {
				if (DEBUG) echo "creating [$top$folder]\n";
				mkdir($top . $folder, 0755, true);
			}
	
			$remote = $m['imageUrl'];
			$image = ($m['numImages'] + 1) . '.' . $ext;
			$local = $top . $folder . $image;

			$webp = ($m['numImages'] + 1) . '.webp';
			$webp = $top . $folder . $webp;

			if (DEBUG) echo "downloading [$remote] => [$local]\n";
			#echo "downloading [$remote] => [$local]\n";
			#echo "downloading to $topRelative$folder\n";

			if (@copy($remote, $local)) {
				if (file_exists($local)) {
					// check isnt 0 bytes (usually from stripchat)
					$tmp = file_get_contents($local);
					if (strlen($tmp) > 0) {
						if ($useWebpImages) {
							if (@system("$cwebpCmd -quiet -q $cwebpQuality $local -o $webp") !== false) {

								if (!$useJpegImages) {
									unlink($local);
								}

                                if (file_exists($webp)) {
                                    cdnSync($topRelative . $folder);
                                    \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=numImages+1 WHERE id=:id', $m);
                                }
							}
                            else {
                                // for some reason the webp conversion failed so delete the jpg
                                unlink($local);
                            }
						}
                        else {
                            if (file_exists($local)) {
                                cdnSync($topRelative . $folder);
                                \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=numImages+1 WHERE id=:id', $m);
                            }
						}
					}
				}
			}
		}

        verify_photos($m, 20, 1);
	}

	//
	// delete empty folders
	//

	$cmd = "find $top -type d -empty -delete";
}
*/



//olllddddddd

/*

function verify_photos($m, $want=30, $debug=0)

{


    global $_AVOS;

	$cwebp = $_AVOS['config']['tools']['cwebp'];
    $imagePath = \Avos\Utils\imagePath($m);

    if (empty($imagePath)) {
        // known issue if models nickname is less than 4 letters
        if ($debug) echo "error! no image path for $m[nickname]\n";
        return;
    }

    if ($debug)  echo "$m[site]:$m[nickname]:$m[numImages]:$imagePath\n";
   
    $dir = WEB . $imagePath;

    //
    // in case of missing images
    //

    $images = \Avos\Utils\list_files($dir);
    $count = count($images);

    if ($count == 0) {
        \Avos\Database\query('UPDATE webcams SET lastImageTime=0, numImages=0 WHERE id=:id', $m);
        \Avos\Database\query('UPDATE webcamsArchive SET lastImageTime=0, numImages=0 WHERE id=:id', $m);
        return;
    }

    //
    // int sort the array by velue
    //

    $tmp = [];

    foreach ($images as $im) {
        if (preg_match('!(\d+)!', $im, $cap)) {
            $tmp[] = $cap[1];
        }
    }

    sort($tmp, SORT_NUMERIC);

    //
    // this is needed in case we have both a jpg and webp
    //

    $images = array_unique($tmp);

    if ($debug) echo "  " . implode(", ", $images) . "\n";

    //
    // make sure only webp files
    //

    foreach ($images as $i => $im) {

       

        //
        // check for failed webp conversion
        //

        if (!file_exists("$dir$im.webp")) {
            if ($debug) echo "    creating $im.webp\n";

            //
            // streamate uses png wtf
            //

            foreach (['jpg', 'png', 'gif'] as $ext) {
                if (file_exists("$dir$im.$ext")) {
                    system("$cwebp -quiet -q 80 $dir$im.$ext -o $dir$im.webp");
                    if ($debug) echo "    removing $im.$ext\n";
                    unlink("$dir$im.$ext");
                }
            }
        }

        //
        // shoudnt get here unless the original file jpg/png/gif didnt download
        // download properly - the original code didnt check for zero size files
        //

        if (!file_exists("$dir$im.webp")) {
            unset($images[$i]);
        }

        //
        // remove any non webp images
        //

        foreach (['jpg', 'png', 'gif'] as $ext) {
            if (file_exists("$dir$im.$ext")) {
                if ($debug) echo "    removing $im.$ext\n";
                unlink("$dir$im.$ext");
            }
        }
    }

    //
    // if we have more images than needed remove some
    //

    if (count($images) > $want) {

        $tmp = [];
        $skip = 1;
        $fac = $want / $count;

        foreach ($images as $im) {

            if ($skip >= 1) {
                $tmp[] = $im;
                $skip--;
            }
            else {
                if ($debug) echo "    removing $im.webp\n";
                @unlink("$dir$im.webp");
            }

            $skip += $fac;
        }

        $images = $tmp;
    }

    #echo "  should now have <= $want images\n";
    #if ($debug) echo "  tmp = " . implode(", ", $tmp) . "\n";

    //
    // create a contiguously numbered array (in case some images are missing)
    // with webp file extensions
    //

    $tmp = [];
    $next = 1;

    foreach ($images as $i => $im) {
        if ($im > $next) {
            #if ($debug) echo "    renaming $dir $im.webp => $next.webp\n";
            rename("$dir$im.webp", "$dir$next.webp");
        }

        $tmp[] = "$next.webp";
        $next++;
    }

    #if ($debug) echo "  tmp = " . implode(", ", $tmp) . "\n";

    //
    // shouldnt need this code
    //

    $images = \Avos\Utils\list_files($dir);
    $count = count($images);

    if ($debug) {
        echo "    images = $count\n    ";
        echo implode(", ", $images);
        echo "\n\n";
    }

    //
    // update the models webcams entry
    //

    $m['want'] = $count;

    \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=:want WHERE id=:id', $m);
    \Avos\Database\query('UPDATE webcamsArchive SET lastImageTime=UNIX_TIMESTAMP(), numImages=:want WHERE id=:id', $m);
}


*/



function cache_popular()
{
	global $_AVOS;

    $redis = \Avos\Redis\handle();
    $key = \Avos\Redis\getPrefix('popular');

    $now = time();
    $numModels = getConfig('webcams', 'popularModelsMax', 1000);
    $numDays = getConfig('webcams', 'popularModelsDays', 30);

    \Avos\Cron\debug("caching the $numModels most popular models from the last $numDays days");

    $popular = $redis->get($key);

    if ($popular === false) {
        $popular = [];
    }

    $models = getWebcams(['count' => $numModels, 'sort' => 'viewers']);

    //
    // insert update the current models data in the redis cache
    //

    foreach ($models as $m) {
        if (isset($popular[ $m['id'] ])) {
            if ($m['viewers'] > $popular[ $m['id'] ]['viewers']) {
                $popular[ $m['id'] ] = [ 'viewers' => $m['viewers'], 'ts' => $now ];
            }
            else {
                $popular[ $m['id'] ]['ts'] = $now;
            }
        }
        else {
            $popular[ $m['id'] ] = [ 'viewers' => $m['viewers'], 'ts' => $now ];
        }
    }

    //
    // remove models not updated in $numDays days
    //

    $tooOld = $now - $numDays * 60*60*24;

    foreach ($popular as $id => $m) {
        if ($m['ts'] < $tooOld) {
            unset($popular[$id]);
        }
    }

    //
    // trim so we only have $numModels models
    //

    uasort($popular, function($a, $b) {
        return $b['viewers'] <=> $a['viewers'];
    });

    //
    // add in rank
    //

    $rank = 1;
    foreach ($popular as $id => $m) {
        $popular[$id]['rank'] = $rank++;
    }

    //
    // stash back to redis
    //

    $redis->set($key, array_slice($popular, 0, $numModels, true));

    /*
    $redis = \Avos\Redis\handle();
    $key = \Avos\Redis\getPrefix('popular');
    $pop = $redis->get($key);

    $archive = \Avos\Database\queryAssoc('SELECT * FROM webcamsArchive WHERE id IN (' . implode(',', array_keys($pop)) . ')');
    $online = \Avos\Database\queryAssoc('SELECT * FROM webcams WHERE id IN (' . implode(',', array_keys($pop)) . ')');

    foreach ($archive as $m) {
        $pop[ $m['id'] ] = array_merge($m, ['viewers' => $pop[ $m['id'] ]['viewers']]);
    }

    foreach ($online as $m) {
        $pop[ $m['id'] ] = array_merge($m, ['viewers' => $pop[ $m['id'] ]['viewers']]);
    }

    setTemplateValue('models', $pop);
    setTemplateValue('online', $online);
    */
}
















function process_history()
{
	global $_AVOS;

	$redis = $_AVOS['redis'];
	$stats = \Avos\Database\queryAssoc('SELECT * FROM history WHERE statusTime != 0');

	\Avos\Cron\debug("processing " . count($stats) . " history records");

	//
	// process all unprocessed individual stats that are completed
	//

	foreach ($stats as $s) {

		if (DEBUG) echo "processing: id=$s[id], webcamsId=$s[webcamsId], status=$s[status], statusTime=$s[statusTime]\n";

		//
		// get year/month/day
		//

		$stat = array(
			'webcamsId' => $s['webcamsId'],
			'year' => date('y', $s['epochTime']),
			'month' => date('m', $s['epochTime']),
			'day' => date('d', $s['epochTime'])
		);

		/*
		 * pre redis
		 *

		$dailyStat = \Avos\Database\queryAssoc('SELECT * FROM dailyStats WHERE year=:year AND month=:month AND day=:day AND webcamsId=:webcamsId', $stat);

		if (count($dailyStat) == 0) {
			\Avos\Database\query('INSERT INTO dailyStats (webcamsId, year, month, day) VALUES (:webcamsId, :year, :month, :day)', $stat);
			$dailyStat = \Avos\Database\queryAssoc('SELECT * FROM dailyStats WHERE year=:year AND month=:month AND day=:day AND webcamsId=:webcamsId', $stat);
		}



		#echo "stat = "; print_r($stat); echo "\n";
		#echo "dailyStat = "; print_r($dailyStat); echo "\n"; #FISH
		
		//
		// merge in the new data
		//

		$smap = array(
			'o' => 'offline',
			'a' => 'away',
			'f' => 'freechat',
			'g' => 'group',
			'p' => 'private'
		);

		$dailyStat = $dailyStat[0];
		$dailyStat[ $smap[ $s['status'] ] . 'Sessions' ]++;
		$dailyStat[ $smap[ $s['status'] ] . 'Time' ] += $s['statusTime'];

		if ($s['statusTime'] > $dailyStat[ $smap[ $s['status'] ] . 'TimeMax' ]) {
			$dailyStat[ $smap[ $s['status'] ] . 'TimeMax' ] = $s['statusTime'];
		}

		//
		// save the new data
		//

		\Avos\Database\query('
			UPDATE dailyStats
			SET 
				awayTime=:awayTime, awaySessions=:awaySessions, awayTimeMax=:awayTimeMax,
				freechatTime=:freechatTime, freechatSessions=:freechatSessions, freechatTimeMax=:freechatTimeMax,
				groupTime=:groupTime, groupSessions=:groupSessions, groupTimeMax=:groupTimeMax,
				privateTime=:privateTime, privateSessions=:privateSessions, privateTimeMax=:privateTimeMax
			WHERE
				id=:id
			',
			$dailyStat);
		*/

		//------------------------------------------------------------------------------
		// make sure we have a dailyStats entry
		//

		//
		// redis
		//
		// * false - dont reverse entries - tho will only be 0 or 1 anyway
		// * false - dont expand the names ie: from 'as' to 'awaySessions'
		//

		$dailyStat = \Avos\Redis\getStat($s['webcamsId'], 'd', $stat['year'], $stat['month'], $stat['day'], false);
		
		#$year = sprintf("%02d", $stat['year']);
		#$month = sprintf("%02d", $stat['month']);
		#$day = sprintf("%02d", $stat['day']);

		#$key = "$s[webcamsId]/d/$year$month$day";
		#if (DEBUG) echo "$key\n";
		#$dailyStat = $redis->get($key);

		/*
		if (count($dailyStat) == 0) {
			if (DEBUG) echo "no existing daily stats record\n";
			$dailyStat = array(
							'as' => 0, 'at' => 0, 'atm' => 0,
							'fs' => 0, 'ft' => 0, 'ftm' => 0,
							'gs' => 0, 'gt' => 0, 'gtm' => 0,
							'ps' => 0, 'pt' => 0, 'ptm' => 0
						);
		}
		*/

		foreach (array('as', 'at', 'atm', 'fs', 'ft', 'ftm', 'gs', 'gt', 'gtm', 'ps', 'pt', 'ptm') as $afgp) {
			if (!isset($dailyStat[$afgp])) {
				$dailyStat[$afgp] = 0;
			}
		}

		#$dailyStat = $dailyStat[0];
		$dailyStat[ $s['status'] . 's' ]++;
		$dailyStat[ $s['status'] . 't' ] += $s['statusTime'];

		if ($s['statusTime'] > $dailyStat[ $s['status'] . 'tm' ]) {
			$dailyStat[ $s['status'] . 'tm' ] = $s['statusTime'];
		}

		if (DEBUG) echo "setting daily stats record\n";
		\Avos\Redis\setStat($s['webcamsId'], 'd', $stat['year'], $stat['month'], $stat['day'], $dailyStat);
		#$redis->set($key, $dailyStat);
		#if (DEBUG) echo "set daily stats record\n";

		//------------------------------------------------------------------------------
		// make sure we have a monthlyStats entry
		//

		$monthlyStat = \Avos\Redis\getStat($s['webcamsId'], 'm', $stat['year'], $stat['month'], '', false);

		#$key = "$s[webcamsId]/m/$year$month";
		#$monthlyStat = $redis->get($key);

		/*
		if (count($monthlyStat) == 0) {
			if (DEBUG) echo "no existing monthly stats record\n";
			$monthlyStat = array(
								'as' => 0, 'at' => 0, 'atm' => 0,
								'fs' => 0, 'ft' => 0, 'ftm' => 0,
								'gs' => 0, 'gt' => 0, 'gtm' => 0,
								'ps' => 0, 'pt' => 0, 'ptm' => 0
						);
		}
		*/

		foreach (array('as', 'at', 'atm', 'fs', 'ft', 'ftm', 'gs', 'gt', 'gtm', 'ps', 'pt', 'ptm') as $afgp) {
			if (!isset($monthlyStat[$afgp])) {
				$monthlyStat[$afgp] = 0;
			}
		}

		#$monthlyStat = $monthlyStat[0];
		$monthlyStat[ $s['status'] . 's' ]++;
		$monthlyStat[ $s['status'] . 't' ] += $s['statusTime'];

		if ($s['statusTime'] > $monthlyStat[ $s['status'] . 'tm' ]) {
			$monthlyStat[ $s['status'] . 'tm' ] = $s['statusTime'];
		}

		if (DEBUG) echo "setting monthly stats record\n";
		\Avos\Redis\setStat($s['webcamsId'], 'm', $stat['year'], $stat['month'], '', $monthlyStat);
		#$redis->set($key, $monthlyStat);
		#if (DEBUG) echo "set monthly stats record\n";

		/*
		 * pre redis
		 *

		$monthlyStat = \Avos\Database\queryAssoc('SELECT * FROM monthlyStats WHERE year=:year AND month=:month AND webcamsId=:webcamsId', $stat);

		if (count($monthlyStat) == 0) {
			\Avos\Database\query('INSERT INTO monthlyStats (webcamsId, year, month) VALUES (:webcamsId, :year, :month)', $stat);
		$monthlyStat = \Avos\Database\queryAssoc('SELECT * FROM monthlyStats WHERE year=:year AND month=:month AND webcamsId=:webcamsId', $stat);
		}

		//
		// merge in the new data
		//

		$monthlyStat = $monthlyStat[0];
		$monthlyStat[ $smap[ $s['status'] ] . 'Sessions' ]++;
		$monthlyStat[ $smap[ $s['status'] ] . 'Time' ] += $s['statusTime'];

		if ($s['statusTime'] > $monthlyStat[ $smap[ $s['status'] ] . 'TimeMax' ]) {
			$monthlyStat[ $smap[ $s['status'] ] . 'TimeMax' ] = $s['statusTime'];
		}

		//
		// save the new data
		//

		\Avos\Database\query('
			UPDATE monthlyStats
			SET 
				awayTime=:awayTime, awaySessions=:awaySessions, awayTimeMax=:awayTimeMax,
				freechatTime=:freechatTime, freechatSessions=:freechatSessions, freechatTimeMax=:freechatTimeMax,
				groupTime=:groupTime, groupSessions=:groupSessions, groupTimeMax=:groupTimeMax,
				privateTime=:privateTime, privateSessions=:privateSessions, privateTimeMax=:privateTimeMax
			WHERE
				id=:id
			',
			$monthlyStat);
		*/

		//
		// move the history entry to redis
		//

		$key = "$s[webcamsId]/h/$stat[year]$stat[month]";
		$mins = (int)(($s['epochTime'] - EPOCHSTART) / 60);
		$e = array('e' => $mins, 's' => $s['status'], 't' => $s['statusTime']);

		if ($redis->rPush($key, $e) !== FALSE) {
			if (DEBUG) echo "stored in redis\n";
			\Avos\Database\query("DELETE FROM history WHERE id=:id", $s);
			if (DEBUG) echo "deleted from mysq\n";
		}
	}
}

# if model not in db first call this
# return models id

function unarchive_model($r)
{
	// fetch model from webcamsArchive table

	$m = \Avos\Database\queryAssoc('SELECT * FROM webcamsArchive WHERE nickname=:nickname AND site=:site LIMIT 1', $r);

	if (count($m) == 1) {

		// check the model hasnt been hidden
		if ($m[0]['updatedOn'] == 0) {
			return -1;
		}

		// check its not already in webcams table

		$n = \Avos\Database\queryAssoc('SELECT id FROM webcams WHERE id=:id LIMIT 1', $m[0]);

		if (count($n) > 0) {
			echo "* already in webcams table\n";
			return $m[0]['id'];
		}

		// set updatedon
		$m[0]['updatedOn'] = time();

		// insert back into webcams table
		
		\Avos\Database\query('
			INSERT INTO webcams (
					id, addedOn, updatedOn, nickname, name, site, subject, age, 
                    imageUrl, lastImageTime, numImages, followers, gender, hd, tags,
                    scrapedOn, birthday, location, languages, links, earnedForGoal, neededForGoal, broadcastVR
				)
				VALUES (
					:id, :addedOn, :updatedOn, :nickname, :name, :site, :subject, :age, 
					:imageUrl, :lastImageTime, :numImages, :followers, :gender, :hd, :tags,
                    :scrapedOn, :birthday, :location, :languages, :links, :earnedForGoal, :neededForGoal, :broadcastVR
				)
			', $m[0]);

		#\Avos\Cron\debug("unarchiving model " . $m[0]['nickname'] . ' ' . $m[0]['id']);
	
		return $m[0]['id'];
	}

	return false;
}


function archive_models($id=false)
{
	global $_AVOS;

	$interval = $_AVOS['config']['webcams']['archiveInterval'];
	$oldest = time() - $interval * 86400;

	if ($id != false) {
		$models = \Avos\Database\queryAssoc("SELECT * FROM webcams WHERE id=$id");
	}
	else {
		#\Avos\Cron\debug("SELECT * FROM webcams WHERE updatedOn < UNIX_TIMESTAMP() - $interval * 86400");
		$models = \Avos\Database\queryAssoc("SELECT * FROM webcams WHERE updatedOn < UNIX_TIMESTAMP() - $interval * 86400");
		\Avos\Cron\debug("archiving " . (count($models)) . " models");
	}

	foreach ($models as $m) {
	
		//
		// first see if the model is already in the webcamsArchive table
		// all fields including id match the main table for a particular model
		//

		$chk = \Avos\Database\queryList("SELECT id FROM webcamsArchive WHERE id=:id", $m);

		if (count($chk) == 0) {

			#\Avos\Cron\debug("archiving model $m[nickname] $m[site] $m[id]");

			\Avos\Database\query('
				INSERT INTO webcamsArchive (
						id, addedOn, updatedOn, nickname, name, site, subject, age, 
						imageUrl, lastImageTime, numImages, followers, gender, hd, tags,
                        scrapedOn, birthday, location, languages, links, earnedForGoal, neededForGoal, broadcastVR
					)
					VALUES (
						:id, :addedOn, :updatedOn, :nickname, :name, :site, :subject, :age, 
						:imageUrl, :lastImageTime, :numImages, :followers, :gender, :hd, :tags,
                        :scrapedOn, :birthday, :location, :languages, :links, "earnedForGoal, :neededForGoal, :broadcastVR
					)
				', $m);

			\Avos\Database\query('DELETE FROM webcams WHERE id=:id', $m);
		}
		else {

			#\Avos\Cron\debug("updating archive for $m[nickname] $m[site] $m[id]");
			
			\Avos\Database\query('
				UPDATE webcamsArchive
				SET 
					updatedOn=:updatedOn, name=:name, subject=:subject, age=:age, 
					imageUrl=:imageUrl, lastImageTime=:lastImageTime, numImages=:numImages, 
					followers=:followers, gender=:gender, hd=:hd, tags=:tags, links=:links
				WHERE
					id=:id
				', $m);

			\Avos\Database\query('DELETE FROM webcams WHERE id=:id', $m);
		}
	}
}


function trimLogs()
{
	global $_AVOS;

	$dir = $_AVOS['config']['site']['avosDir'] . 'logs/';
	$max = $_AVOS['config']['site']['logsMaxSize'];
	$logs = \Avos\Utils\list_files($dir);

	foreach ($logs as $l) {
		$lines = file($dir . $l);

		if (count($lines) > $max) {
			\Avos\Cron\debug("trimming $l");
			$trimmed = array_slice($lines , 0 - $max);
			file_put_contents($dir . $l, implode("", $trimmed));
		}
	}
}


function cdnSync($dirRelative = 'photos')
{
	global $_AVOS;

	$cdnUrl = '';
	$cdnPath = '';
	$dirRelative = rtrim($dirRelative, '/');
	$cdnDirRelative = dirname($dirRelative) . '/';
	$webDir = $_AVOS['config']['site']['webDir'];
	$rsyncCmd = $_AVOS['config']['tools']['rsync'];

	if (isset($_AVOS['config']['site']['cdnUrl']) && !empty($_AVOS['config']['site']['cdnUrl'])) {
		$cdnUrl = $_AVOS['config']['site']['cdnUrl'];

		// remove protocol and any trailing path info
		if (preg_match('!^https?://(.+)?(/.*/)!', $cdnUrl, $c)) {
			$cdnUrl = $c[1];
		}
	}
	else {
		// no cdn defined
		return;
	}

	if (isset($_AVOS['config']['site']['cdnPath'])) {
		$cdnPath = $_AVOS['config']['site']['cdnPath'];
	}

	#\Avos\Cron\debug("local: $webDir$dirRelative");
	#\Avos\Cron\debug("remote: $cdnUrl:$cdnPath$cdnDirRelative");

	$cmd = "$rsyncCmd -avr $webDir$dirRelative $cdnUrl:$cdnPath$cdnDirRelative";
	$fh = popen($cmd, 'r');

	while($line = fgets($fh)) {
		$line = rtrim($line);
		if (preg_match('!(.*\.(jpg|webp))$!', $line, $c)) {
			$image = "$webDir$cdnDirRelative$line";
			$imageDir = dirname($image);
			unlink($image);
			#\Avos\Cron\debug("$cdnDirRelative$line");

			// might not exist if synching jpg+webp
			if (file_exists($imageDir)) {
				@rmdir($imageDir);
			}
		}
		flush();
	}

	pclose($fh);
	system($cmd);
}


function refreshCategories()
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

    \Avos\Database\disableCache();

	//
	// force query2sql to expand the category or tag
	//

    $_AVOS['page']['type'] = 'webcams';

    //
    // optionally store all webcams data not just 'lite'
    //

    $webcamsLite = getConfig('site', 'cronWebcamsLite', 1);

    //
    // see if we are using cats/tags from a shared redis/mysql
    //

    $categoriesPrefix = getConfig('redis', 'categoriesPrefix', '');

    if (!empty($categoriesPrefix)) {
        return refreshCategoriesShared($categoriesPrefix);
    }

	//
	// categories
	//

	$stats = array();
    \Avos\Database\query("UPDATE webcamsCategories SET keep=0");
    $lc = listWebcamsCategories();

	foreach ($lc as $c) {
        $category = $c['category'];
        $stats[$category] = $c;
        $stats[$category]['fullname'] = $c['name'];
        $stats[$category]['online'] = 0;
        $stats[$category]['rooms'] = 0;
        $stats[$category]['viewers'] = 0;

        \Avos\Cron\debug("category $category");

        // need to expand cats/tags here in case contains ||
        $filter = \getWebcamsCategory($category, 'categories');

        if ($filter !== FALSE) {
            #$sql = \Avos\Utils\query2sql(array('category' => $category), true);
            $sql = \Avos\Utils\query2sql($filter['filter']);
            #\Avos\Database\query('INSERT INTO webcamsCategories (name, webcamsId) SELECT "' . $category . '", id FROM webcams WHERE ' . $sql);
            $models = \Avos\Database\queryAssoc('SELECT "' . $category . '" AS category, id FROM webcams WHERE ' . $sql);

            foreach ($models as $m) {
                \Avos\Database\query('INSERT INTO webcamsCategories (name, webcamsId) VALUES (:category, :id) ON DUPLICATE KEY UPDATE keep=1', $m);
            }

            \Avos\Database\query("DELETE FROM webcamsCategories WHERE keep=0 AND name='$category'");
        }
	}

	\Avos\Database\query("DELETE FROM webcamsCategories WHERE keep=0");
    $counts = array('categories' => $stats);

	//
	// save the categories list of models in redis
	//

    foreach ($lc as $c) {
        $models = \Avos\Database\queryAssoc("SELECT wc.webcamsId AS i, w.site AS s, w.gender AS g FROM webcamsCategories wc, webcams w WHERE wc.name=:category AND wc.webcamsId=w.id", $c);
        $key = \Avos\Redis\getPrefix("webcams/tag/$c[category]");
        \Avos\Cron\debug("saving to [$key]");
        $redis->set($key, $models);
        $redis->expire($key, 600);
    }

    /*
	//
	// cache the number of webcam models per category/gender
	//

	\Avos\Cron\debug("refreshing categories counts");
	$perPage = $_AVOS['config']['webcams']['perPage'];
    #$perPage = 20; #shouldnt want more than 20 models in a related cams section or pages like tags etc

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsCategories c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&category=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&category=$name$sort");
            }
	}

    $counts = array('categories' => $stats);
    */

	//
	// tags
	//

	$stats = array();
	\Avos\Database\query("UPDATE webcamsTags SET keep=0");
	$lc = listWebcamsCategories('tags');

	foreach ($lc as $c) {
        $category = $c['tag'];
        $stats[$category] = $c;
        $stats[$category]['fullname'] = $c['name'];
        $stats[$category]['online'] = 0;
        $stats[$category]['rooms'] = 0;
        $stats[$category]['viewers'] = 0;

        \Avos\Cron\debug("tag $category");

        // need to expand cats/tags here in case contains ||
        $filter = \getWebcamsCategory($category, 'tags');

        if ($filter !== FALSE) {
            #$sql = \Avos\Utils\query2sql(array('tag' => $category), true);
            $sql = \Avos\Utils\query2sql($filter['filter']);
            #\Avos\Database\query('INSERT INTO webcamsTags (name, webcamsId) SELECT "' . $category . '", id FROM webcams WHERE ' . $sql);
            $models = \Avos\Database\queryAssoc('SELECT "' . $category . '" AS category, id FROM webcams WHERE ' . $sql);

            foreach ($models as $m) {
                \Avos\Database\query('INSERT INTO webcamsTags (name, webcamsId) VALUES (:category, :id) ON DUPLICATE KEY UPDATE keep=1', $m);
            }

            \Avos\Database\query("DELETE FROM webcamsTags WHERE keep=0 AND name='$category'");
        }
    }

	\Avos\Database\query("DELETE FROM webcamsTags WHERE keep=0");

	//
	// save the tags list of models in redis
	//

    foreach ($lc as $c) {
        $models = \Avos\Database\queryAssoc("SELECT wc.webcamsId AS i, w.site AS s, w.gender AS g FROM webcamsTags wc, webcams w WHERE wc.name=:tag AND wc.webcamsId=w.id", $c);
        $key = \Avos\Redis\getPrefix("webcams/tag/$c[tag]");
        \Avos\Cron\debug("saving to [$key]");
        $redis->set($key, $models);
        $redis->expire($key, 600);
    }

    $counts['tags'] = $stats;
    refreshCategoriesCounts($counts);

    /*
	//
	// cache the number of webcam models per tag/gender
	//

	\Avos\Cron\debug("refreshing tags counts");

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsTags c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&tag=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&tag=$name$sort");
            }
	}

    $counts['tags'] = $stats;

	//
	// cache the number of webcam models per site
	//

	$stats = array();

	//
	// online models is easy
	//

	\Avos\Cron\debug("refreshing webcams counts");
	$online = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcams WHERE status!='o' GROUP BY site");

	foreach ($online as $s) {
		$stats[$s['site']] = array('online' => $s['num']);
	}

	//
	// archived models
	//

	$offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcamsArchive GROUP BY site");

	foreach ($offline as $s) {
		$stats[$s['site']]['archived'] = $s['num'];
	}

	//
	// offline models needs two tables searching
	//

	$offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcams WHERE status='o' GROUP BY site");

	foreach ($offline as $s) {
		$stats[$s['site']]['offline'] = $s['num'];
	}

	$offline = \Avos\Database\queryAssoc("
					SELECT COUNT(*) AS num, site 
					FROM webcamsArchive
					WHERE id NOT IN (
						SELECT id FROM webcams
					)	
					GROUP BY site");

	foreach ($offline as $s) {
		if (isset($stats[$s['site']]['offline'])) {
			$stats[$s['site']]['offline'] += $s['num'];
		}
		else {
			$stats[$s['site']]['offline'] = $s['num'];
		}
	}

	// force query2sql to add status!=o if not already set
    $_AVOS['page']['type'] = 'webcams';
	
	foreach ($stats as $site => $s) {

		// in case no archived or online models

		if (!isset($s['online'])) {
			$stats[$site]['online'] = 0;
		}

		if (!isset($s['archived'])) {
			$stats[$site]['archived'] = 0;
		}

		$stats[$site]['fullname'] = \Avos\Scaffold\expandSiteName($site);
		$stats[$site]['site'] = $stats[$site]['fullname'];
		$stats[$site]['total'] = $stats[$site]['online'] + $stats[$site]['offline'];

        if ($webcamsLite) {
            $stats[$site]['models'] = \getWebcamsLite("count=$perPage&site=$site");
        }
        else {
            $stats[$site]['models'] = \getWebcams("count=$perPage&site=$site");
        }

		$stats[$site]['matches'] = $_AVOS['page']['matches'];
        $stats[$site]['pages'] = $_AVOS['page']['pages'];
    }

    // for cron output
    $total = 0;
    foreach ($stats as $s => $v) {
        $total += $v['total'];
    }
    \Avos\Cron\debug("total cams: $total");

	// reset for any further requests
	$_AVOS['page']['type'] = '';

	$counts['sites'] = $stats;
	#\Avos\Database\query('UPDATE cache SET value=:value WHERE name="webcams/counts"', array('value' => serialize($counts)));

	//
	// save online counts for cats+tags
	//

	if (\getConfig('webcams', 'getCatsTags')) {
		\Avos\Cron\debug("counting cats/tags intersections");
		$cats = \listWebcamsCategories('categories');
		$tags = \listWebcamsCategories('tags');
		$_AVOS['page']['type'] = 'webcams';
		$catsTags = array();

		foreach ($cats as $c) {
			foreach ($tags as $t) {
				$sql = \Avos\Utils\query2sql(array('category' => $c['category'], 'tag' => $t['tag']));
				list($count, $viewers) = \Avos\Database\queryList("SELECT COUNT(*), SUM(viewers) FROM webcams WHERE $sql");
				$catsTags[$c['category']][$t['tag']]['category'] = $c['category'];
				$catsTags[$c['category']][$t['tag']]['tag'] = $t['tag'];
				$catsTags[$c['category']][$t['tag']]['fullname'] = $t['fullname'];
				$catsTags[$c['category']][$t['tag']]['online'] = $count;
				$catsTags[$c['category']][$t['tag']]['rooms'] = $count;
				$catsTags[$c['category']][$t['tag']]['viewers'] = $viewers;

                if ($webcamsLite) {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcamsLite("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
                else {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcams("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
			}
		}

		$counts['catstags'] = $catsTags;
	}

	//
	// save the data in redis
	//

	$key = \Avos\Redis\getPrefix('webcams/counts');
	\Avos\Cron\debug("saving to [$key]");
    $redis->set($key, $counts);
    */
}

function refreshCategoriesCounts($counts)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

    \Avos\Database\disableCache();

	//
	// force query2sql to expand the category or tag
	//

    $_AVOS['page']['type'] = 'webcams';

    //
    // optionally store all webcams data not just 'lite'
    //

    $webcamsLite = getConfig('site', 'cronWebcamsLite', 1);

	//
	// cache the number of webcam models per category/gender
	//

	\Avos\Cron\debug("refreshing categories counts");
	$perPage = $_AVOS['config']['webcams']['perPage'];
    #$perPage = 20; #shouldnt want more than 20 models in a related cams section or pages like tags etc
    $stats = $counts['categories'];

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsCategories c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&category=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&category=$name$sort");
            }
	}

	$counts['categories'] = $stats;

	//
	// cache the number of webcam models per tag/gender
	//

	\Avos\Cron\debug("refreshing tags counts");
    $stats = $counts['tags'];

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsTags c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&tag=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&tag=$name$sort");
            }
	}

	$counts['tags'] = $stats;

	//
	// cache the number of webcam models per site
	//

	$stats = array();

	//
	// online models is easy
	//

	\Avos\Cron\debug("refreshing webcams counts");
	$online = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcams WHERE status!='o' GROUP BY site");

	foreach ($online as $s) {
		$stats[$s['site']] = array('online' => $s['num']);
	}

    if (getConfig('webcams', 'countArchive', 0)) {
        //
        // archived models
        //

        $offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcamsArchive GROUP BY site");

        foreach ($offline as $s) {
            $stats[$s['site']]['archived'] = $s['num'];
        }
    }

	//
	// offline models needs two tables searching
	//

	$offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcams WHERE status='o' GROUP BY site");

	foreach ($offline as $s) {
		$stats[$s['site']]['offline'] = $s['num'];
	}

    if (getConfig('webcams', 'countArchive', 0)) {
        $offline = \Avos\Database\queryAssoc("
                        SELECT COUNT(*) AS num, site 
                        FROM webcamsArchive
                        WHERE id NOT IN (
                            SELECT id FROM webcams
                        )	
                        GROUP BY site");

        foreach ($offline as $s) {
            if (isset($stats[$s['site']]['offline'])) {
                $stats[$s['site']]['offline'] += $s['num'];
            }
            else {
                $stats[$s['site']]['offline'] = $s['num'];
            }
        }
    }

	// force query2sql to add status!=o if not already set
    $_AVOS['page']['type'] = 'webcams';
	
	foreach ($stats as $site => $s) {

		// in case no archived or online models

		if (!isset($s['online'])) {
			$stats[$site]['online'] = 0;
		}

		if (!isset($s['archived'])) {
			$stats[$site]['archived'] = 0;
		}

		$stats[$site]['fullname'] = \Avos\Scaffold\expandSiteName($site);
		$stats[$site]['site'] = $stats[$site]['fullname'];
		$stats[$site]['total'] = $stats[$site]['online'] + $stats[$site]['offline'];

        if ($webcamsLite) {
            $stats[$site]['models'] = \getWebcamsLite("count=$perPage&site=$site");
        }
        else {
            $stats[$site]['models'] = \getWebcams("count=$perPage&site=$site");
        }

		$stats[$site]['matches'] = $_AVOS['page']['matches'];
        $stats[$site]['pages'] = $_AVOS['page']['pages'];
    }

    // for cron output
    $total = 0;
    foreach ($stats as $s => $v) {
        $total += $v['total'];
    }
    \Avos\Cron\debug("total cams: $total");

	// reset for any further requests
	$_AVOS['page']['type'] = '';

	$counts['sites'] = $stats;
	#\Avos\Database\query('UPDATE cache SET value=:value WHERE name="webcams/counts"', array('value' => serialize($counts)));

	//
	// save online counts for cats+tags
	//

    if (!empty(getUrl('webcamsCategoryTag'))) {
		\Avos\Cron\debug("counting cats/tags intersections");
		$cats = \listWebcamsCategories('categories');
		$tags = \listWebcamsCategories('tags');
		$_AVOS['page']['type'] = 'webcams';
		$catsTags = array();

		foreach ($cats as $c) {
			foreach ($tags as $t) {
				$sql = \Avos\Utils\query2sql(array('category' => $c['category'], 'tag' => $t['tag']));
				list($count, $viewers) = \Avos\Database\queryList("SELECT COUNT(*), SUM(viewers) FROM webcams WHERE $sql");
				$catsTags[$c['category']][$t['tag']]['category'] = $c['category'];
				$catsTags[$c['category']][$t['tag']]['tag'] = $t['tag'];
				$catsTags[$c['category']][$t['tag']]['fullname'] = $t['fullname'];
				$catsTags[$c['category']][$t['tag']]['online'] = $count;
				$catsTags[$c['category']][$t['tag']]['rooms'] = $count;
				$catsTags[$c['category']][$t['tag']]['viewers'] = $viewers;

                if ($webcamsLite) {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcamsLite("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
                else {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcams("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
			}
		}

		$counts['catstags'] = $catsTags;
	}

	//
	// save the data in redis
	//

	$key = \Avos\Redis\getPrefix('webcams/counts');
	\Avos\Cron\debug("saving to [$key]");
	$redis->set($key, $counts);
}

function refreshCategoriesShared($sharedPrefix)
{
    global $_AVOS;

	//
	// map cached cats/tags in the shared redis prefix to our own
	//

    \Avos\Cron\debug("synching stats from redis:$sharedPrefix");

    //
    // populate our own webcamsCategories+Tags tables from the shared one
    //

    $db = getConfig('mysql', 'categoriesDatabase', '');

    if (empty($db)) {
        $db = getConfig('mysql', 'webcamsDatabase', '');
    }

    \Avos\Cron\debug("synching tables from mysql:$db");

    $prefix = \Avos\Redis\getPrefix();
    $key = "$sharedPrefix/webcams/counts";
    $redis = \Avos\Redis\handle();
    $shared = $redis->get($key);
    $sitesFilter = \getConfig('webcams', 'sitesFilter', '');
    $counts = [];

    //
    // same actions for categories then tags
    //

    foreach (['categories', 'tags'] as $type) {

        $stats = [];
        $key = $type == 'categories' ? 'category' : 'tag';
        $cats = \listWebcamsCategories($type);

        $table = 'webcams' . ucfirst($type);
        \Avos\Database\query("UPDATE $table SET keep=0");

        //
        // loop round each category/tag defined for this site
        //

        foreach ($cats as $c) {
            if (isset($shared['tags'][$c['filter']])) {

                $stats[$c[$key]] = $c;
                $stats[$c[$key]]['fullname'] = $c['name'];
                $stats[$c[$key]]['online'] = 0;
                $stats[$c[$key]]['rooms'] = 0;
                $stats[$c[$key]]['viewers'] = 0;

                //
                // found a matching category/tag in the shared redis install
                //

                #\Avos\Cron\debug("$key " . $c['filter'] . "... $c[$key]");
                $shared['tags'][$c['filter']][$key] = $c[$key];
                #$counts[$type][$c[$key]] = $shared[$type][$c['filter']];

                //
                // sync our local webcamsCategories/Tags table to the shared mysql
                //

                #\Avos\Database\query("INSERT INTO $table (name, webcamsId) SELECT DISTINCT '$c[$key]', webcamsId FROM $db.$table WHERE name = '$c[filter]'");
                $rkey = "$sharedPrefix/webcams/tag/$c[filter]";
                $models = $redis->get($rkey);
                $matched = 0;

                $catGender = $c['gender'];
                $catGenderEmpty = empty($catGender);

                foreach ($models as $m) {
                    $m['name'] = $c[$key];
                    if ((empty($sitesFilter) || strpos($sitesFilter, $m['s']) !== false) && ($catGenderEmpty || strpos($catGender, $m['g']) !== false )) {
                        \Avos\Database\query("INSERT INTO $table (name, webcamsId) VALUES (:name, :i) ON DUPLICATE KEY UPDATE keep=1", $m);
                        $matched++;
                    }
                }

                \Avos\Cron\debug("$key $c[filter]... $rkey matched $matched models");
            }
            else {

                //
                // no matching category/tag in the shared redis install
                //

                \Avos\Cron\debug("$key " . $c['filter'] . "... ERROR - SKIPPED");
            }

            $counts[$type] = $stats;
        }

        //
        // clean out any stale entries
        //

        \Avos\Database\query("DELETE FROM $table WHERE keep=0");
    }

    refreshCategoriesCounts($counts);

    /*
	//
    // save online counts for cats+tags if:
    //   the [urls] contains webcamsCategoryTag rule
    //   [webcams] contains getCatsTags=1 (will override)
	//

    $urls = getConfig('urls');
    $getCatsTags = 0;

    foreach ($urls as $name => $u) {
        if ($name == 'webcamsCategoryTag') {
            $getCatsTags = 1;
        }
    }

	if (\getConfig('webcams', 'getCatsTags', $getCatsTags)) {
		\Avos\Cron\debug("counting cats/tags intersections");
		$cats = \listWebcamsCategories('categories');
		$tags = \listWebcamsCategories('tags');
        $catsTags = array();

		foreach ($cats as $c) {
            foreach ($tags as $t) {

                $category = $c['category'];
                $tag = $t['tag'];

                $sharedCategory = $c['filter'];
                $sharedTag = $t['filter'];

                if (isset($shared['catstags'][$sharedCategory][$sharedTag])) {
                    $counts['catstags'][$category][$tag] = $shared['catstags'][$sharedCategory][$sharedTag];
                }
			}
		}
	}

	//
	// save the data in redis
	//

	$key = \Avos\Redis\getPrefix('webcams/counts');
	\Avos\Cron\debug("saving to [$key]");
    $redis->set($key, $counts);
     */
}

function refreshVideosTags()
{
	global $_AVOS;
	$videos = \Avos\Database\queryAssoc('SELECT id, tags FROM videos');
	\Avos\Database\query("UPDATE videosTags SET keep=0");

	foreach ($videos as $v) {
		$tags = explode(',', $v['tags']);

		foreach ($tags as $t) {
			if (!empty($t)) {
				\Avos\Database\query("INSERT INTO videosTags (name, videosId) VALUES (:name, :id)",
					array(
						'name' => $t,
						'id' => $v['id']
					)
				);
			}
		}
	}

	\Avos\Database\query("DELETE FROM videosTags WHERE keep=0");
}

function refreshPhotosTags()
{
	global $_AVOS;
	$videos = \Avos\Database\queryAssoc('SELECT id, tags FROM photos');
	\Avos\Database\query("UPDATE photosTags SET keep=0");

	foreach ($videos as $v) {
		$tags = explode(',', $v['tags']);

		foreach ($tags as $t) {
			if (!empty($t)) {
				\Avos\Database\query("INSERT INTO photosTags (name, photosId) VALUES (:name, :id)",
					array(
						'name' => $t,
						'id' => $v['id']
					)
				);
			}
		}
	}

	\Avos\Database\query("DELETE FROM photosTags WHERE keep=0");
}

function refreshSitemap()
{
	global $_AVOS;

	$webDir = $_AVOS['config']['site']['webDir'];
	$url = rtrim($_AVOS['config']['site']['url'], '/');
	$max = 50000;
	$indexes = 0;
	$gzip = '';
	$gz = '';
    $timestamp = getConfig('webcams', 'sitemapsTimestamp', 'updatedOn');
    \Avos\Cron\debug("web: $webDir");

	if (isset($_AVOS['config']['site']['gzipSitemaps']) && $_AVOS['config']['site']['gzipSitemaps'] == true) {
		$gzip = $_AVOS['config']['tools']['gzip'];
		$gz = '.gz';
	}

	$urlsetHeader = '<' . '?xml version="1.0" encoding="UTF-8"?' . ">\n" . '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
	$urlsetFooter = '</urlset>' . "\n";

	
	if (getConfig('urls')) {
		$urls = \Avos\Scaffold\getUrlConfig()['urls'];
	
		// Fetch the counts for different types
		$tmp = \Avos\Database\queryList("select count(*) from videos");
		$counts['videos'] = $tmp[0];
	
		$tmp = \Avos\Database\queryList("select count(*) from photos");
		$counts['photos'] = $tmp[0];
	
		$tmp = \Avos\Database\queryList("select count(*) from ArticleBlogPosts;");
		$counts['blogs'] = $tmp[0];
	
		$tmp = \Avos\Database\queryList("select count(*) from promote;");
		$counts['promote'] = $tmp[0];
	
		foreach ($urls as $u) {
			$tmpl = $u[0];
			$type = $u[1];
			$path = $u[2];
	
			// Skip URLs that do not match required types
			if (!preg_match('!(none|webcams?|videos?|photos?|blogs?|users?|casino?)!', $type)) continue;
			if ($type == 'redirect') continue;
			if ($type == 'users') continue;
			if (preg_match('!(test|fish|foo|game)!', $path)) continue; // Skip the test files

			// Skip URLs containing placeholders like :site or :nickname
			if (preg_match('!:(site|nickname|id|title|_month|_year|siteId|username|urlTitle|tag)!', $path)) continue;
	
			// Skip .css or similar static file extensions
			if (preg_match('!\.(css|js|json|xml)$!', $path)) continue;
	
			// Replace known placeholders except :page
			$path = str_replace(
				[':id', ':siteId', ':title', ':urlTitle', ':nickname', ':kw', ':Title', ':siteNames', ':username', ':_month', ':_year'],
				'*',
				$path
			);
	
			// Check if the URL uses pagination (:page placeholder)
			if (strpos($path, ':page') !== false) {
				// Determine items per page based on type
				$perPage = 36; // Default value
				if ($type === 'photos') {
					$perPage = getConfig('photos', 'perPage');
				} elseif ($type === 'blogs') {
					$perPage = getConfig('blogs', 'perPage');
				} elseif ($type === 'videos') {
					$perPage = getConfig('videos', 'perPage');
				}
	
				// Get total items count from $counts array
				$totalItems = 0;
				if (isset($counts['photos']) && $type === 'photos') {
					$totalItems = $counts['photos'];
				} elseif (isset($counts['blogs']) && $type === 'blogs') {
					$totalItems = $counts['blogs'];
				} elseif (isset($counts['videos']) && $type === 'videos') {
					$totalItems = $counts['videos'];
				}
	
				// Calculate total pages
				$totalPages = ceil($totalItems / $perPage);
	
				// Generate URLs for all pages
				for ($i = 1; $i <= $totalPages; $i++) {
					if ($i === 1) {
						// For the first page, remove the :page part entirely
						echo str_replace(':page', '', $path) . "\n";
					} else {
						// Replace :page with the actual page number
						echo str_replace(':page', $i, $path) . "\n";
					}
				}
			} else {
				// Output URLs without :page placeholder
				echo $path . "\n";
			}
		}
	}
	
	exit;
	
	
	
	

	//
	// home + webcams categories
	//

    if (getConfig('webcams', 'sitemapsCategories', 1)) {
        $counts = getWebcamsCounts();
        $urlset = "<url><loc>$url/</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";


		//print_r($counts['id']);exit;


        foreach ($counts['categories'] as $c) {
            $curl = $url .encodeUrl('webcamsCategory', $c);
            $urlset .= "<url><loc>$curl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";
        }

        foreach ($counts['tags'] as $c) {
            $curl = $url . encodeUrl('webcamsTag', $c);
            $urlset .= "<url><loc>$curl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";
        }

        \Avos\Cron\debug('writing categories sitemap.' . $indexes . '.xml');
        file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
        if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
        $indexes++;
    }




//
// blogs sitemap
//

$done = 0;
$offset = 0;
$blogsUrl = getConfig('blog', 'sitemapsUrlName');

while (!$done) {
    // Query to fetch the necessary fields from ArticleBlogPosts table
    $items = \Avos\Database\queryAssoc("
        SELECT id, Title, publishedOn 
        FROM ArticleBlogPosts 
        WHERE publishedOn IS NOT NULL 
        ORDER BY publishedOn DESC 
        LIMIT $offset, $max
    ");

    if (count($items) > 0) {
        $urlset = '';

        foreach ($items as $i) {
            // Process the title to replace spaces with hyphens but keep colons
            $i['Title'] = strtolower($i['Title']); // Convert to lowercase
            $i['Title'] = preg_replace('/\s+/', '-', $i['Title']); // Replace spaces with hyphens
            // Keep colons and other allowed special characters
            $i['Title'] = preg_replace('/[^a-z0-9:-]+/', '-', $i['Title']); // Replace other special chars with hyphen
            $i['Title'] = trim($i['Title'], '-'); // Remove trailing hyphens
            
            // Generate the final URL
            $murl = $url . encodeUrl($blogsUrl, $i);

            // Format the `publishedOn` datetime to `Y-m-d`
            $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', strtotime($i['publishedOn'])) . "</lastmod></url>\n";
        }

        \Avos\Cron\debug('writing blogs sitemap.' . $indexes . '.xml');
        
        // Write the sitemap XML file
        file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
        
        // Compress the file if gzip is available
        if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");

        // Increment the offset and index for pagination
        $offset += $max;
        $indexes++;
    } else {
        // No more items to process, exit loop
        $done = 1;
    }
}




//
// promote sitemap
//

$done = 0;
$offset = 0;
$promoteUrl = getConfig('promote', 'sitemapsUrlName');
while (!$done) {
    // Query to fetch the necessary fields from the promote table
    $items = \Avos\Database\queryAssoc("
        SELECT siteId, SiteLink, siteNames, publishedOn, title 
        FROM promote 
        WHERE publishedOn IS NOT NULL 
        ORDER BY publishedOn DESC  
        LIMIT $offset, $max
    ");
    if (count($items) > 0) {
        $urlset = '';
        foreach ($items as $i) {
            // Ensure that the correct URL format is used
            $i['SiteLink'] = trim($i['SiteLink']); // Remove any extra spaces
            // Use the correct title or SiteLink as needed
			
            $murl = $url . encodeUrl($promoteUrl, $i);
            // Format the `publishedOn` datetime to `Y-m-d`
            $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', strtotime($i['publishedOn'])) . "</lastmod></url>\n";
        }
        \Avos\Cron\debug('writing promote sitemap.' . $indexes . '.xml');
        // Write the sitemap XML file
        file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
        // Compress the file if gzip is available
        if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");

        // Increment the offset and index for pagination
        $offset += $max;
        $indexes++;
    } else {
        // No more items to process, exit loop
        $done = 1;
    }
}

/* i move id down after webcams -- added later 
	//
	// videos
	//

	$done = 0; 
	$offset = 0;
	$max = 50000;
    $videosUrl = getConfig('videos', 'sitemapsUrlName');

	while (!$done) {
		$items = \Avos\Database\queryAssoc("SELECT id, urlTitle, publishedOn FROM videos WHERE publishedOn>0 ORDER BY publishedOn DESC LIMIT $offset, $max");

		if (count($items) > 0) {

			$urlset = '';

			foreach ($items as $i) {
				$i['title'] = $i['urlTitle'];
				$murl = $url . encodeUrl($videosUrl, $i);
				$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $i['publishedOn']) . "</lastmod></url>\n";
			}

	 		\Avos\Cron\debug('writing videos sitemap.' . $indexes . '.xml');
			file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
			if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
			$offset += $max;
			$indexes++;
		}
		else {
			$done = 1;
		}
	}*/

    //
    // photos
    //

	$done = 0;
	$offset = 0;
    $photosUrl = getConfig('photos', 'sitemapsUrlName');

	while (!$done) {
		$items = \Avos\Database\queryAssoc("SELECT id, urlTitle, publishedOn FROM photos WHERE publishedOn>0 ORDER BY publishedOn DESC LIMIT $offset, $max");

		if (count($items) > 0) {

			$urlset = '';

			foreach ($items as $i) {
				$i['title'] = $i['urlTitle'];
				$murl = $url . encodeUrl($photosUrl, $i);
				$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $i['publishedOn']) . "</lastmod></url>\n";
			}

	 		\Avos\Cron\debug('writing photos sitemap.' . $indexes . '.xml');
			file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
			if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
			$offset += $max;
			$indexes++;
		}
		else {
			$done = 1;
		}
	}

	//
	// webcams
	//

    if (getConfig('webcams', 'sitemapsWebcams', 1)) {

        $done = 0;
        $offset = 0;
        $webcamsProfile = getConfig('webcams', 'sitemapsUrlName');
        $webcamsProfileFn = '';

        if (preg_match('!(.*)\(\)$!', $webcamsProfile, $c) && function_exists('\\Avos\\User\\' . $c[1])) {
            $webcamsProfileFn = '\\Avos\\User\\' . $c[1];
        }
        else if (function_exists('\\Avos\\User\\webcamsProfileFn')) {
            $webcamsProfileFn = '\\Avos\\User\\webcamsProfileFn';
        }

        $sitesSql = '';
        $sitesFilter = str_replace(',', '|', \getConfig('webcams', 'sitesFilter', ''));
        if (!empty($sitesFilter)) $sitesSql = "WHERE " . \Avos\Utils\query2sql("site=$sitesFilter");

        while (!$done) {
            $models = \Avos\Database\queryAssoc("SELECT nickname, site, addedOn, updatedOn FROM webcams $sitesSql ORDER BY updatedOn DESC LIMIT $offset, $max");

            if (count($models) > 0) {

                $urlset = '';

                foreach ($models as $m) {
                    $m['site'] = \Avos\Scaffold\longSiteName($m['site']);

                    if (getConfig('site', 'lowerCaseUrls')) {
                        $m['site'] = strtolower($m['site']);
                    }

                    if (!empty($webcamsProfileFn)) {
                        $murl = $url . call_user_func($webcamsProfileFn, $m);
                    }
                    else {
                        $murl = $url . encodeUrl($webcamsProfile, $m);
                    }

                    $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $m[$timestamp]) . "</lastmod></url>\n";
                }

                \Avos\Cron\debug('writing webcams sitemap.' . $indexes . '.xml');
                file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
                if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
                $offset += $max;
                $indexes++;
            }
            else {
                $done = 1;
            }
        }

        //
        // webcamsArchive
        //

		/*old sitemap version for webcams
        if (getConfig('webcams', 'sitemapsArchive', 1)) {

            $done = 0;
            $offset = 0;
            if (!empty($sitesFilter)) $sitesSql = \Avos\Utils\query2sql("site=$sitesFilter") . " AND";

            while (!$done) {
                $models = \Avos\Database\queryAssoc("SELECT nickname, site, addedOn, updatedOn FROM webcamsArchive WHERE $sitesSql id NOT IN (SELECT id FROM webcams)  ORDER BY updatedOn DESC LIMIT $offset, $max");

                if (count($models) > 0) {

                    $urlset = '';

                    foreach ($models as $m) {
                        $m['site'] = \Avos\Scaffold\longSiteName($m['site']);

                        if (getConfig('site', 'lowerCaseUrls')) {
                            $m['site'] = strtolower($m['site']);
                        }

                        if (!empty($webcamsProfileFn)) {
                            $murl = $url . call_user_func($webcamsProfileFn, $m);
                        }
                        else {
                            $murl = $url . encodeUrl($webcamsProfile, $m);
                        }

                        $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $m[$timestamp]) . "</lastmod></url>\n";
                    }

                    \Avos\Cron\debug('writing webcamsArchive sitemap.' . $indexes . '.xml');
                    file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
                    if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
                    $offset += $max;
                    $indexes++;
                }
                else {
                    $done = 1;
                }
            }
        }
    }*/





	//
	// videos
	//

	$done = 0; 
	$offset = 0;
	$max = 50000;
    $videosUrl = getConfig('videos', 'sitemapsUrlName');

	while (!$done) {
		$items = \Avos\Database\queryAssoc("SELECT id, urlTitle, publishedOn FROM videos WHERE publishedOn>0 ORDER BY publishedOn DESC LIMIT $offset, $max");

		if (count($items) > 0) {

			$urlset = '';

			foreach ($items as $i) {
				$i['title'] = $i['urlTitle'];
				$murl = $url . encodeUrl($videosUrl, $i);
				$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $i['publishedOn']) . "</lastmod></url>\n";
			}

	 		\Avos\Cron\debug('writing videos sitemap.' . $indexes . '.xml');
			file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
			if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
			$offset += $max;
			$indexes++;
		}
		else {
			$done = 1;
		}
	}


















/* sitemap version for webcams razvan ideea */
	if (getConfig('webcams', 'sitemapsArchive', 1)) {

		$lastOnline = getConfig('webcams', 'sitemapsLastOnline', 0);
		$lastOnlineSql = '';

		if ($lastOnline > 0) {
			$lastOnlineSql = 'updatedOn > ' . (time() - $lastOnline * 60*60*24) . ' AND';
		}

		$done = 0;
		$offset = 0;
		if (!empty($sitesFilter)) $sitesSql = \Avos\Utils\query2sql("site=$sitesFilter") . " AND";

		while (!$done) {
			$models = \Avos\Database\queryAssoc("SELECT nickname, site, addedOn, updatedOn FROM webcamsArchive WHERE $sitesSql $lastOnlineSql id NOT IN (SELECT id FROM webcams)  ORDER BY updatedOn DESC LIMIT $offset, $max");

			if (count($models) > 0) {

				$urlset = '';

				foreach ($models as $m) {
					$m['site'] = \Avos\Scaffold\longSiteName($m['site']);

					if (getConfig('site', 'lowerCaseUrls')) {
						$m['site'] = strtolower($m['site']);
					}

					if (!empty($webcamsProfileFn)) {
						$murl = $url . call_user_func($webcamsProfileFn, $m);
					}
					else {
						$murl = $url . encodeUrl($webcamsProfile, $m);
					}

					$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $m[$timestamp]) . "</lastmod></url>\n";
				}

				\Avos\Cron\debug('writing webcamsArchive sitemap.' . $indexes . '.xml');
				file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
				if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
				$offset += $max;
				$indexes++;
			}
			else {
				$done = 1;
			}
		}
	}
}

/* sitemap version for webcams razvan ideea */



	//
	// write the index
	//

	$index  = '<' . '?xml version="1.0" encoding="UTF-8"?' . '>' . "\n";
	$index .= '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
	$date = date(DATE_W3C);

	for ($i = 0; $i < $indexes; $i++) {
		$index .= " <sitemap>\n  <loc>$url/sitemap.$i.xml$gz</loc>\n  <lastmod>$date</lastmod>\n </sitemap>\n";
	 }

	 $index .= '</sitemapindex>' . "\n";

	 \Avos\Cron\debug('writing index sitemap.xml');
     file_put_contents($webDir . 'sitemap.xml', $index);

     //
     // ping search engines
     //



	 //deprecating ping search engines
	 /*    
     $ping = 'https://www.google.com/webmasters/tools/ping?sitemap=' . urlencode($url) . '/sitemap.xml';
     $html = file_get_contents($ping);
     $chk = strpos($html, 'success') !== false ? 'ok' : 'error';
     \Avos\Cron\debug("google:$chk $ping");

     $key = getConfig('site', 'indexNowKey', '');
     if (!empty($key)) {
         if (file_exists($webDir . $key . '.txt')) {
             $ping = "https://api.indexnow.org/indexnow?key=$key&url=$url";
             if (file_get_contents($ping) !== false) {
                 \Avos\Cron\debug("indexnow:submitted $ping");
             }
         }
         else {
             \Avos\Cron\debug("site.indexNowKey file does not exist as $webDir$key.txt");
         }
     }
	 
	 
	 */











}

function purgeHistory()
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	//
	// on redis deadlock a closed history record will be left in mysql
	// if older than 12 hours delete else move to redis
	// also if statusTime > 4hrs delete
	//

	$h = \Avos\Database\queryAssoc("
					SELECT * FROM history 
					WHERE (statusTime > 0 AND UNIX_TIMESTAMP() - epochTime > 3600*12)
					OR statusTime > 3600*4
				");

	foreach ($h as $s) {
		$year = date('y', $s['epochTime']);
		$month = date('m', $s['epochTime']);

		$key = "$s[webcamsId]/h/$year$month";
		$mins = (int)(($s['epochTime'] - EPOCHSTART) / 60);
		$e = array('e' => $mins, 's' => $s['status'], 't' => $s['statusTime']);

		echo "$key - "; print_r($e);

		/*
		if ($redis->rPush($key, $e) !== FALSE) {
			if (DEBUG) echo "stored in redis\n";
			\Avos\Database\query("DELETE FROM history WHERE id=:id", $s);
			if (DEBUG) echo "deleted from mysq\n";
		}
		 */
	}
}

function purge($mode='daily')
{
	global $_AVOS;
	$redis = $_AVOS['redis'];
	
	list($maxw) = \Avos\Database\queryList("SELECT max(id) FROM webcams");
	list($maxa) = \Avos\Database\queryList("SELECT max(id) FROM webcamsArchive");
	
	if ($maxa > $maxw) {
		$maxw = $maxa;
	}
	
	\Avos\Cron\debug("max webcams.id=$maxw webcamsArchive.id=$maxa");
	
	//
	// moving dailyStats
	//
	
	if ($mode == 'daily') {
		$moveDailyStats = $_AVOS['config']['webcams']['moveDailyStats'];
		
		if ((int)$moveDailyStats > 0) {
			$alldays = \Avos\Utils\yearsMonthsKeys((int)($moveDailyStats/30)+1, true);
			$today = date('ymd');
			\Avos\Cron\debug("moving dailyStats from $moveDailyStats days ago, today is $today");
			$days = array();
		
			// first cut off from today and forwards
			for ($i=0; $i<count($alldays) && $alldays[$i] != $today; $i++) {
				$days[] = $alldays[$i];
			}
	
			// cut off moveDailyStats days ago
			$days = array_splice($days, 0, count($days) - $moveDailyStats);
		
			// only look back for 3 days (cron should be run every day)
			$days = array_splice($days, count($days)-5, count($days));
		
			$count = 0;
		
			for ($i=1; $i<=$maxw; $i++) {
				$purged = array();
				foreach ($days as $day) {
					$key = "$i/d/$day";
					if ($redis->type($key) == \Redis::REDIS_STRING) {
						$stat = $redis->get($key);
		
						if (\Avos\SQLite\set($key, $stat) != false) {
							$purged[] = $day;
							$redis->del($key);
							if ($count++ % 1000 == 0) {
								sleep(1);
							}
						}
					}
				}
	
				if (count($purged) > 0) {
					\Avos\Cron\debug("$i/d/" . implode(",", $purged));
				}
			}
		}
		
		\Avos\Cron\debug("purged $count dailyStats records");
		return;
	}
	
	//
	// monthlyStats and ranks, $mode='monthly'
	//
	
	$moveStatusHistory = $_AVOS['config']['webcams']['moveStatusHistory'];
	$moveMonthlyStats = $_AVOS['config']['webcams']['moveMonthlyStats'];
	$moveRanks = $_AVOS['config']['webcams']['moveRanks'];
	
	$day = date('j');
	$count = 1;
	
	$month = date('ym');
	\Avos\Cron\debug("month is $month");
	\Avos\Cron\debug("moving statusHistory from $moveStatusHistory months ago");
	\Avos\Cron\debug("moving monthlyStats from $moveMonthlyStats months ago");
	\Avos\Cron\debug("moving ranks from $moveRanks months ago");

	// cut off moveStatusHistory months ago
	if ((int)$moveStatusHistory > 0) {
		$months = \Avos\Utils\yearsMonthsKeys($moveStatusHistory+1);
		$monthsStatusHistory = array_splice($months, 0, count($months) - $moveStatusHistory);
	}

	// cut off moveMonthlyStats months ago
	if ((int)$moveMonthlyStats > 0) {
		$months = \Avos\Utils\yearsMonthsKeys($moveMonthlyStats+1);
		$monthsStats = array_splice($months, 0, count($months) - $moveMonthlyStats);
	}

	// cut off moveMonthlyStats months ago
	if ((int)$moveRanks > 0) {
		$months = \Avos\Utils\yearsMonthsKeys($moveRanks+1);
		$monthsRanks = array_splice($months, 0, count($months) - $moveRanks);
	}

	for ($i=1; $i<=$maxw; $i++) {

		//
		// statusHistory
		//

		if ((int)$moveStatusHistory > 0) {
			foreach ($monthsStatusHistory as $month) {
				$key = "$i/h/$month";
				if ($redis->type($key) == \Redis::REDIS_LIST) {
					$stat = $redis->lRange($key, 0, -1);

					if (\Avos\SQLite\set($key, $stat) != false) {
						$redis->del($key);
						\Avos\Cron\debug($key);
					}

					if ($count++ % 500 == 0) {
						sleep(1);
					}
				}
			}
		}
		#else {
		#	\Avos\Cron\debug("never moving statusHistory");
		#}

		//
		// monthlyStats
		//

		if ((int)$moveMonthlyStats > 0) {
			foreach ($monthsStats as $month) {
				$key = "$i/m/$month";
				if ($redis->type($key) == \Redis::REDIS_STRING) {
					$stat = $redis->get($key);

					if (\Avos\SQLite\set($key, $stat) != false) {
						$redis->del($key);
						\Avos\Cron\debug($key);
					}

					if ($count++ % 500 == 0) {
						sleep(1);
					}
				}
			}
		}
		#else {
		#	\Avos\Cron\debug("never moving monthlyStats");
		#}

		//
		// ranks
		//

		if ((int)$moveRanks > 0) {
			foreach ($monthsRanks as $month) {
				$key = "$i/r/$month";
				if ($redis->type($key) == \Redis::REDIS_LIST) {
					$stat = $redis->lRange($key, 0, -1);

					if (\Avos\SQLite\set($key, $stat) != false) {
						$redis->del($key);
						\Avos\Cron\debug($key);
					}

					if ($count++ % 500 == 0) {
						sleep(1);
					}
				}
			}
		}
		#else {
		#	\Avos\Cron\debug("never moving ranks");
		#}
	}

	\Avos\Cron\debug("purged $count monthly records");
}

function backups($backupnow = false)
{
	global $_AVOS;
	$conf = $_AVOS['config']['backups'];
    $redis = $_AVOS['redis'];

	$backups = \Avos\Redis\getPrefix('backups');
	$backupFolders = \Avos\Redis\getPrefix('backupFolders');
	$info = $redis->hGetAll($backups);
	$avosDir = $_AVOS['config']['site']['avosDir'];
	$tar = $_AVOS['config']['tools']['tar'];
	$bzip2 = $_AVOS['config']['tools']['bzip2'];
	$now = time();

	//
	// clean out old backups
	//

    $folders = $redis->lRange($backupFolders, 0, -1);
	$keep = getConfig('backups', 'keepBackupsCount', 3);
	$have = count($folders);
	$delete = array_slice($folders, 0, $have - $keep);

	foreach ($delete as $f) {
		$folder = $conf['backupsFolder'] . $f;
		\Avos\Cron\debug("rm -rf $folder");
		system("rm -rf $folder");
		$redis->blPop($backupFolders, 10);
	}

	//
	// backup
	//

//force backups - - - $info['lastEpoch'] = '';

	//$info['lastEpoch'] = '';
	if ($backupnow == true || !isset($info['lastEpoch']) || $info['lastEpoch'] < $now - $conf['backupIntervalDays'] * 86400 - 360) {


	

		//
		// create folder
		//

		$folder = $conf['backupsFolder'] . date('ymd');
		\Avos\Cron\debug("backing up to $folder");

		if (file_exists($folder)) {
			\Avos\Cron\debug("backup folder already exists");
			return;
		}

		if (@mkdir($folder, 0755) == false) {
			\Avos\Cron\debug("unable to create backup folder");
			return;
		}

		//
		// files
		//

		if ($conf['backupFiles'] == 1) {
			\Avos\Cron\debug("archiving files");
            system("cd '$avosDir' && cd .. && $tar czf $folder/files.tgz --exclude=avos/db/* --exclude=avos/incoming/* --exclude=avos/backups/* avos");

            // web folder
            $webDir = getConfig('site', 'webDir');
            $imagesUrl = getConfig('webcams', 'imagesUrl');
            $videosUrl = getConfig('videos', 'videosUrl');
            $photosUrl = getConfig('photos', 'photosUrl');

            system("cd '$webDir' && cd .. && $tar czf $folder/web.tgz --exclude=web$imagesUrl* --exclude=web$videosUrl --exclude=web$photosUrl web");
        }

		//
		// mysql
		//

		if ($conf['backupMysql'] == 1) {

			$mysql = $_AVOS['config']['tools']['mysqldump'];
			$hostname = $_AVOS['config']['mysql']['hostname'];
			$database = $_AVOS['config']['mysql']['database'];
			$username = $_AVOS['config']['mysql']['username'];
			$password = $_AVOS['config']['mysql']['password'];

			\Avos\Cron\debug("dumping mysql");
			system("$mysql -h $hostname -u $username -p$password $database > $folder/mysql.sql");

			\Avos\Cron\debug("compressing mysql");
			system("$bzip2 $folder/mysql.sql");
		}

		//
		// redis
		//

		if ($conf['backupRedis'] == 1) {
			\Avos\Cron\debug("copying redis snapshot");

			if (isset($_AVOS['config']['redis']['dumpsDir'])) {
				$dir = $_AVOS['config']['redis']['dumpsDir'];
			}
			else {
				$dir = "/var/lib/redis/";
			}

			system("cp ${dir}dump.rdb $folder/redis.rdb");

			\Avos\Cron\debug("compressing redis");
			system("$bzip2 $folder/redis.rdb");
		}

		//
		// sqlite
		//

		if ($conf['backupSQLite'] == 1) {
			\Avos\Cron\debug("achiving sqlite");
			system("cd '$avosDir' && $tar cf $folder/sqlite.tar db");

			\Avos\Cron\debug("compressing sqlite");
			system("$bzip2 $folder/sqlite.tar");
		}
		
		$redis->rPush($backupFolders, date('ymd'));
		$redis->hSet($backups, 'lastEpoch', $now);
	}
	else {
		$date = date('ymd', $info['lastEpoch']);
		\Avos\Cron\debug("too soon to backup, last backup was created on $date");
	}
}

function refreshSiteTags()
{
	global $_AVOS;

	$tags = getSiteTags();
	$_AVOS['redis']->set('siteTags', $tags);
}

function getSiteTags()
{
    // Analise tags usage per site
    \Avos\Database\disableCache();

	$alltags = array();
	
	$Archive = '';
	$Archive = 'Archive';
    $q = \Avos\Database\queryAssoc("SELECT site, tags FROM webcams$Archive WHERE tags != ''");
    $tags = array();

    foreach ($q as $i => $m) {
        $tags = explode(',', strtolower(trim($m['tags'], ',')));

        foreach ($tags as $t) {
            $t = trim($t);
            if ($t === '') {
                continue; // Skip empty tags
            }

            if (!isset($alltags[$t])) {
                $alltags[$t] = array(
                    'cb' => 0, 'bc' => 0, 'sc' => 0, 'cs' => 0, 
                    'ff' => 0, 'sm' => 0, 'sr' => 0, 'lj' => 0, 
                    'so' => 0, 'cr' => 0, 'xc' => 0, 'c4' => 0, 'xl' => 0, 'il' => 0, 'mf' => 0, 'am' => 0
                );
            }

            if (isset($alltags[$t][$m['site']])) {
                $alltags[$t][$m['site']]++;
            } else {
                // Initialize site counter if not already set
                $alltags[$t][$m['site']] = 1;
            }
        }
    }

    $keys = array_keys($alltags);
    sort($keys);
    $min = 200;
    $ret = array();

    foreach ($keys as $k) {
        $count = 
            $alltags[$k]['cb'] +
            $alltags[$k]['bc'] +
            $alltags[$k]['sc'] +
            $alltags[$k]['cs'] +
            $alltags[$k]['ff'] +
            $alltags[$k]['sm'] +
            $alltags[$k]['sr'] +
            $alltags[$k]['lj'] +
            $alltags[$k]['so'] +
            $alltags[$k]['cr'] +
            $alltags[$k]['xc'] +
            $alltags[$k]['c4'] +
			$alltags[$k]['il'] +
			$alltags[$k]['mf'] +
			$alltags[$k]['am'] +
            $alltags[$k]['xl'];

        if ($count > $min) {
            $ret[$k] = $alltags[$k];
        }
    }

    return $ret;
}