<?php

/*

6431
$m->blocked_regions
https://api.pinklabel.com/api/v1/cams/online.json?aff_id=6431&prog=rs&limit=10&order_by=viewers_desc
Array
(
    [0] => stdClass Object
    (
        [countryIsoCode] => 250
        [regionIsoCode] => PDL
    )
)
*/

namespace Sites\Cam4;

function get_shortname()
{
    return 'c4';
}

function fetch_models_data()
{
    global $_AVOS;

    $affiliate = getConfig('webcams', 'Cam4')['affiliate'];
    $url = "https://api.pinklabel.com/api/v1/cams/online.json?aff_id=$affiliate&prog=rs&limit=10000&order_by=viewers_desc";

    $context = stream_context_create([
        'http' => [
            'ignore_errors' => true
        ],
    ]);

    $raw = @file_get_contents($url, false, $context);
    $json = json_decode($raw);

    if ($raw === false) {
        throw new \RuntimeException('Error: Unable to fetch models data.');
    }

    $http_response_header = $http_response_header ?? [];
    $response_code = null;
    if (preg_match('#HTTP/\d+\.\d+\s+(\d{3})#', $http_response_header[0], $matches)) {
        $response_code = (int) $matches[1];
    }

    if ($response_code !== 200) {
        throw new \RuntimeException("Error: HTTP $response_code received from server.");
    }

    $json = json_decode($raw);

    if (json_last_error() !== JSON_ERROR_NONE) {
        throw new \RuntimeException('Error decoding JSON or invalid JSON structure.');
    }

    return $json;
}

function process_model($m)
{
    $gender_map = [
        "male" => 'm',
        "female" => 'f',
        "shemale" => 't',
        "unknown" => 't',
        "" => ''
    ];

    $gender = empty($m->gender) ? '' : $gender_map[$m->gender];

    if (!empty($m->gender) && !($gender == 'm' && (int)$m->viewers < 6)) {

        if ($m->broadcast_type != 'solo') {
            $gender = 'c';
        }

        //
        // geo bans
        //

        $blocked = [];

        /*
        if (!empty($m->blocked_regions)) {
            if (isset($m->geobans->blockedCountries)) {
                foreach ($m->geobans->blockedCountries as $c) {
                    $blocked[$c] = 1;
                }

                foreach ($m->geobans->blockedRegions as $c => $r) {
                    $blocked[$c] = 1;
                }
            }
        }
         */

        $blocked = array_keys($blocked);

        $status_map = [
            "NORMAL" => 'f',
            "PRIVATE_SHOW" => 'p',
            "GROUP_SHOW" => 'g'
        ];

        $status = isset($m->status) && isset($status_map[$m->status]) ? $status_map[$m->status] : 'f'; 

        return [
            'site' => 'c4',
            'nickname' => $m->nickname,
            'name' => $m->nickname,
            'viewers' => (int)$m->viewers,
            'age' => (int)$m->age > 18 ? (int)$m->age : 24,
            'gender' => $gender,
            'earnedForGoal'  => 0, // Replace with actual value if available
            'neededForGoal'  => 0, // Replace with actual value if available
            'broadcastVR'    => 0, // Replace with actual value if available
            'blocked' => $blocked,
            'status' => $status,
            'externalId' => $m->id,
            'location' => isset($m->country) ? \Avos\Utils\countryCode($m->country) : '',
            'imageUrl' => isset($m->thumb_big) ? $m->thumb_big : '',
            'profileImageUrl' => isset($m->profile_thumb) ? $m->profile_thumb : '',
            'streamUrl' => isset($m->preview_url) ? $m->preview_url : '',
            'subject' => isset($m->status) ? $m->status : "",
            'tags' => get_tags($m->show_tags),
            'languages' => isset($m->languages) ? implode(',', $m->languages) : ''
        ];
    }

    return null;
}

function get_tags($tags)
{
    $ret = [];

    if (!is_array($tags)) {
        return '';
    }

    foreach ($tags as $t) {
        if (preg_match('!.*\/(.+)!', $t, $c)) {
            $ret[] = $c[1];
        } else {
            $ret[] = $t;
        }
    }

    return implode(',', $ret);
}

function online_status()
{
    global $_AVOS;
    try {
        $models_data = fetch_models_data();

        $models = [];
        foreach ($models_data as $m) {
            $model = process_model($m);
            if ($model) {
                $models[] = $model;
            }
        }
        
        return $models;

    } catch (\RuntimeException $e) {
        return [];
    }
}

?>
