<?php

namespace Sites\LiveJasmin;

function get_shortname()
{
    return 'lj';
}

function online_status()
{
    global $_AVOS;

    $url = "https://wmcdpt.com/api/model/feed?siteId=jasmin&psId=" . $_AVOS['config']['webcams']['LiveJasmin']['affiliate'] . "&psTool=213_1&psProgram=revs&campaignId=&category=girl&limit=500&imageSizes=1024x576&imageType=any&showOffline=0&onlyFreeStatus=0&extendedDetails=1&responseFormat=xml&performerId=&subAffId={SUBAFFID}&accessKey=" . $_AVOS['config']['webcams']['LiveJasmin']['accessKey'] . "&legacyRedirect=1&filters=&customOrder=most_popular";

    $raw = @file_get_contents($url);
    if ($raw === false) {
        die('Error fetching data from the API.');
    }

    $xml = simplexml_load_string($raw);
    if ($xml === false) {
        die('Error parsing XML data.');
    }

    $models = [];
    foreach ($xml->data->models->model as $m) {
        $tags = [];
        foreach ($m->details->appearances->appearance as $a) {
            $tags[] = (string)$a;
        }
        foreach ($m->details->willingnesses->willingness as $a) {
            $tags[] = (string)$a;
        }

        $blocked = [];
        foreach ($m->bannedCountries->bannedCountry as $c) {
            $blocked[] = strtolower((string)$c);
        }

        if ((string)$m->status != 'offline') {
			//$about = isset($m->details->about->biography) ? (string)$m->details->about->biography : '';
			$subject = isset($m->details->about->turnsOn) ? (string)$m->details->about->turnsOn : '';
		
            $models[] = [
                'site' => 'lj',
                'nickname' => (string)$m->performerId,
                'name' => (string)$m->displayName,
                'country' => (string)$m->country,
                'ethnicity' => (string)$m->ethnicity,
                'blocked' => $blocked,
				//'about' => 	$about,
				'subject' => $subject,
                'tags' => implode(',', $tags),
                'age' => (string)$m->persons->person[0]->age,
                'imageUrl' => (string)$m->profilePictureUrl->size1024x576,
                'earnedForGoal' => 0, // Replace with actual value if available
                'neededForGoal' => 0, // Replace with actual value if available
                'broadcastVR' => 0 ,// Replace with actual value if available
                'status' => str_replace(
                    ['free_chat', 'member_chat', 'members_only', 'private_chat', 'offline'],
                    ['f', 'g', 'g', 'p', 'o'],
                    (string)$m->status
                ),
                'gender' => str_replace(
                    ['female', 'male', 'hermaphrodite', 'shemale', 'transgendered'],
                    ['f', 'm', 't', 't', 't'],
                    (string)$m->persons->person[0]->sex
                )
            ];
        } 
    }

    return $models;
}

function request($mode, $token, $args = [])
{
    $url = 'https://partner-api.modelcenter.jasmin.com/v1/' . $mode . '?';

    foreach ($args as $k => $v) {
        if (is_array($v)) {
            foreach ($v as $vv) {
                $url .= '&' . $k . '[]=' . $vv;
            }
        } else {
            $url .= '&' . $k . '=' . $v;
        }
    }

    $url = str_replace('?&', '?', $url);

    $req = curl_init($url);
    curl_setopt_array($req, [
        CURLOPT_HTTPHEADER  => ["Accept: application/json", "Authorization: Bearer $token"],
        CURLOPT_RETURNTRANSFER  => true,
        CURLOPT_VERBOSE     => 0
    ]);

    $out = curl_exec($req);
    curl_close($req);

    $data = json_decode($out);
    $ret = \JSM\Utils\object2array($data);

    return $ret;
}

function performerStates($token)
{
    return request('performer-states', $token);
}

function performerStats($token, $fromDate, $toDate, $models = [])
{
    return request('reports/performers', $token, ['fromDate' => $fromDate, 'toDate' => $toDate]);
}
