<?php
include('bootstrap.php');

$minImagesRequired = 30;

// Fetch models with more than the desired number of images
$models = \Avos\Database\queryAssoc(
    'SELECT id, nickname, numImages, site 
     FROM webcams 
     WHERE numImages > :want 
     ORDER BY numImages DESC', 
    ['want' => $minImagesRequired]
);

foreach ($models as $model) {
    $imagePath = \Avos\Utils\imagePath($model);
   // $directory = WEB . $imagePath;
    $directory = rtrim(WEB, '/') . '/' . ltrim($imagePath, '/');

    #echo $directory ;exit;
    
    echo "{$model['site']}:{$model['nickname']}:{$model['numImages']}:$imagePath\n";
    echo "  Directory: $directory\n";

    // Check if directory exists
    if (!is_dir($directory)) {
        echo "  Directory not found: $directory\n";
        continue;
    }

    // List and count the image files
    $imageFiles = \Avos\Utils\list_files($directory);
    $totalImages = count($imageFiles);

    if ($totalImages === 0) {
        echo "  No images found for {$model['nickname']}\n";
        continue;
    }

    // Convert filenames to integers (assuming format like '1.webp', '2.webp')
    $imageNumbers = array_map(fn($file) => (int)str_replace('.webp', '', $file), $imageFiles);
    sort($imageNumbers); // Sort in ascending order

    // Calculate step factor for pruning
    $stepFactor = $minImagesRequired / $totalImages;
    $prunedImages = [];
    $skipCounter = 1;

    // Select images to keep based on step factor
    foreach ($imageNumbers as $num) {
        if ($skipCounter >= 1) {
            $prunedImages[] = "$num.webp";
            $skipCounter--;
        }
        $skipCounter += $stepFactor;
    }

    // Rename selected images to ensure a contiguous set (1.webp, 2.webp, ...)
    $renamedFiles = [];
    foreach ($prunedImages as $index => $image) {
        $newName = ($index + 1) . ".webp"; 
        echo "    Renaming: $directory/$image to $directory/$newName\n";
        rename("$directory/$image", "$directory/$newName");
        $renamedFiles[$image] = true;
    }

    // Remove leftover images not in the pruned set
    foreach ($imageNumbers as $num) {
        $filename = "$num.webp";
        if (!isset($renamedFiles[$filename]) && $num > $minImagesRequired) {
            echo "    Deleting: $directory/$filename\n";
            unlink("$directory/$filename");
        }
    }

    echo "  Total images processed: $totalImages\n";

    // Update database with the new number of images
    $model['want'] = min($totalImages, $minImagesRequired);
    \Avos\Database\query(
        'UPDATE webcams 
         SET lastImageTime = UNIX_TIMESTAMP(), numImages = :want 
         WHERE id = :id', 
        $model
    );
    \Avos\Database\query(
        'UPDATE webcamsArchive 
         SET lastImageTime = UNIX_TIMESTAMP(), numImages = :want 
         WHERE id = :id', 
        $model
    );
}

echo "Pruned " . count($models) . " models. Ooooooer missus!\n";
?>
