<?
namespace Avos\Cron;

//
// online status:
//
// o: offline
// a: away
// f: freechat
// g: group
// p: private
//

$_cron_start_time = 0;
$_module = '';

function debugModule($module)
{
	global $_module;
	$_module = $module;
}

function debug($str)
{
	global $_module;

	echo date('ymd H:i:s') . ' ';
	if (!empty($_module)) echo "[$_module] ";
	else echo "* ";

    echo "$str\n";
    flush();
}

function online_status_update($site)
{
	global $_AVOS;

	$time = time();
	$now = (int)(($time - EPOCHSTART) / 60);
	$omodels = array();

	$added = 0;
	$changed = 0;
	$unarchived = 0;

	$models = call_user_func('\\Sites\\' . $site . '\\online_status');

	// dont update status if the feed returns an error

	if ($models === false) {
		\Avos\Cron\debug("error from feed: not modifying models status");
		return;
	}

	$site_short = call_user_func('\\Sites\\' . $site . '\\get_shortname');
	$model_info_cb = '\\Sites\\' . $site . '\\get_model';

	if (!function_exists($model_info_cb)) {
		$model_info_cb = false;
	}

	\Avos\Cron\debug("processing " . count($models) . " online models");

	$genderRank = array();
	$onlineCount = 0;

	$yymm = date('ym', $time);
	
	foreach ($models as $i => $m) {
	
		if (DEBUG) \Avos\Cron\debug("$m[site] $m[nickname] $m[status]");
		if ($m['status'] != 'o') $onlineCount++;

		//
		// data fixup
		//

		foreach (['subject', 'about'] as $f) {
			if (!isset($m[$f])) {
				$m[$f] = '';
			}
		}

		foreach (['viewers', 'followers', 'age', 'hd'] as $f) {
			if (!isset($m[$f])) {
				$m[$f] = 0;
			}
		}

		$m['age'] = (int)$m['age'];
		#if ($m['age'] == 0) $m['age'] = 99;
        if ($m['age'] == 0 && strpos($m['tags'], 'teen') !== false) $m['age'] = 19;

		$m['subject'] = str_replace('\\', ' ', $m['subject']);
		$m['subject'] = \Avos\Utils\convert_ascii($m['subject']);
		$m['subject'] = \Avos\Utils\truncate($m['subject'], 255);

		$m['about'] = str_replace('\\', ' ', $m['about']);
		$m['about'] = \Avos\Utils\convert_ascii($m['about']);
		$m['about'] = \Avos\Utils\truncate($m['about'], 255);

        $m['name'] = str_replace('\\', ' ', $m['name']);

		if (!empty($m['location'])) {
			$m['location'] = substr(iconv('UTF-8', 'ASCII//TRANSLIT', $m['location']), 0, 32);
		}
		else {
			$m['location'] = '';
		}

		if (!empty($m['languages'])) {
			$m['languages'] = substr($m['languages'], 0, 128);
		}
		else {
			$m['languages'] = '';
		}

        if (!isset($m['body'])) $m['body'] = '';

        if (!empty($m['birthday'])) {
		$m['birthday'] = strtotime($m['birthday']);
		if ($m['birthday'] < 0) {
			$m['birthday'] = 0;
		}
        }
        else {
            $m['birthday'] = 0;
        }

        if ($m['viewers'] < 0) {
            // bongacams
            $m['viewers'] = 0;
        }

		//
		// we need tags to start and end with a comma
        // plus check isnt longer than WEBCAMS_MAX_TAGSLEN
		//

		if (!empty($m['tags'])) {
			if (substr($m['tags'], -1) != ',') {
				$m['tags'] .= ',';
			}

			if ($m['tags'][0] != ',') {
				$m['tags'] = ',' . $m['tags'];
			}

            if (strlen($m['tags']) > WEBCAMS_MAX_TAGSLEN) {
                $m['tags'] = substr($m['tags'], 0, WEBCAMS_MAX_TAGSLEN);

                if ($m['tags'][-1] != ',') {
                    $idx = -2;
                    while ($m['tags'][$idx] != ',' && -$idx-- < WEBCAMS_MAX_TAGSLEN);
                    $m['tags'] = substr($m['tags'], 0, $idx) . ',';
                }
            }
		}

		//
		// site and site/gender ranks
		//

		$m['rank'] = $i + 1;

		if (isset($genderRank[ $m['gender'] ])) {
			$genderRank[ $m['gender'] ] ++;
		}
		else {
			$genderRank[ $m['gender'] ] = 1;
		}

		$m['genderRank'] = $genderRank[ $m['gender'] ];

		if ($site_short == 'sm') {
			$m['rank'] = 0;
			$m['genderRank'] = 0;
		}
	
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// fetch existing model id and add if needed
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
		$chk = \Avos\Database\queryList('SELECT id, updatedon FROM webcams WHERE nickname=:nickname AND site=:site', $m);
		$updatedon = 0;

		//
		// if doesnt exist next check the webcamsArchive table
		//

		if (count($chk) == 0) {
			$id = \Avos\Cron\unarchive_model($m);

			// skip if the model has been hidden
			if ($id == -1) {
				continue;
			}

			if ($id !== false) {
				$chk = array( $id );
				$unarchived++;
			}
		}
			
		//
		// insert the model if we dont have an entry
		//
	
		if (count($chk) > 0) {
			$id = $chk[0];
			#$updatedon = $chk[1];
			if (DEBUG) \Avos\Cron\debug(" exists, id=$id");

			// to decide whether to store rank/viewers/etc info
			if ($m['status'] == 'f') {
				\Avos\Redis\setModel($id, 'ft', $now);
			}
		}
		else {
			#echo "adding ($m[nickname])\n";

			// get more info if needed
			if ($model_info_cb) {
				$extra = call_user_func($model_info_cb, $m);

				foreach ($extra as $k => $v) {
					$m[$k] = $v;
				}
			}

			$id = \Avos\Database\query('
				INSERT INTO webcams (addedOn, updatedOn, nickname, name, site, status, statusTime, subject, about, age, imageUrl, viewers, followers, gender, hd, rank, genderRank, tags, birthday, location, languages, body)
				VALUES (UNIX_TIMESTAMP(), UNIX_TIMESTAMP(), :nickname, :name, :site, :status, 0, :subject, :about, :age, :imageUrl, :viewers, :followers, :gender, :hd, :rank, :genderRank, :tags, :birthday, :location, :languages, :body)
				', $m);
	
			if (DEBUG) echo "  added, id=$id\n";
			$added++;

			// to decide whether to store rank/viewers/etc info
			if ($m['status'] == 'f') {
				\Avos\Redis\setModel($id, 'ft', $now);
			}
		}

		#if (preg_match('!worshipmis!', $m['nickname'])) { echo "BUGGERED = "; print_r($m); exit; }

		//
		// for quick hash lookup via webcamsId to see who is online
		//
	
		$omodels[$id] = 1;
	
		//
		// merge database id with the model info
		//
	
		$m['epochTime'] = $time;
		$m['webcamsId'] = $id;
		$models[$i]['webcamsId'] = $id;
		$id = 0;

		#if ($m['webcamsId'] == 89135) \Avos\Cron\debug("model $m[webcamsId] = $m[status]"); #FISH

		//
		// set rank info - streamate returns models in alphabetic order
		//

		$freechatStart = \Avos\Redis\getModel($m['webcamsId'], 'ft');
		$freechatMins = $now - $freechatStart;
		$freechatDelay = $_AVOS['config']['webcams']['delayRankSaving'];

		#echo "$m[webcamsId] now=$now freechatStart=$freechatStart freechatMins=$freechatMins\n";

		if ($site_short != 'sm' && $m['status'] == 'f' && $freechatMins > $freechatDelay) {
			$lastRankTime = \Avos\Redis\getModel($m['webcamsId'], 'lastRankTime');
			\Avos\Redis\setModel($m['webcamsId'], 'rank', $m['rank']);
			\Avos\Redis\setModel($m['webcamsId'], 'genderRank', $m['genderRank']);

			if (empty($lastRankTime) || $lastRankTime + $_AVOS['config']['webcams']['refreshRankInterval'] < $now) {
				 \Avos\Redis\setModel($m['webcamsId'], 'lastRankTime', $now);
				 \Avos\Redis\setModel($m['webcamsId'], "r/$yymm", 
				 	array(
						'e' => $now, 
						'r' => $m['rank'], 
						'g' => $m['genderRank'],
						'v' => $m['viewers'],
						'f' => $m['followers']
						)
					, true);
			}
		}

		//
		// store any extra info in redis if we have it
		//

		foreach (array('externalId', 'streamUrl', 'photos', 'blocked') as $f) {
			if (!empty($m[$f])) {
                \Avos\Redis\setModel($m['webcamsId'], $f, $m[$f]);
                #\Avos\Cron\debug("$m[site]/$m[nickname]/$f: " . $m[$f]);
			}
		}

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// fetch last history record
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
		$hid = 0;
		$chk = \Avos\Database\queryAssoc('SELECT * FROM history WHERE webcamsId=:webcamsId AND statusTime=0 ORDER BY id DESC LIMIT 1', $m);
	
		if (count($chk) > 0) {
	
			$history = $chk[0];
	
			//
			// check if status has changed
			//
	
			if ($history['status'] != $m['status']) {
	
				$history['statusTime'] = $time - $history['epochTime'];
				if (DEBUG) echo "  CHG: model $m[webcamsId] has changed from " . $history['status'] . " (" . $history['statusTime'] . "s) to $m[status]\n";
				$changed++;

				//
				// no clue why - but limit stattsTime
				//

				if ($history['statusTime'] > 86400) $history['statusTime'] = 86400;
	
				\Avos\Database\query('
					UPDATE history SET statusTime=:statusTime WHERE id=:id
					', $history);

				// to decide whether to store rank/viewers/etc info
				if ($m['status'] == 'f') {
					\Avos\Redis\setModel($m['webcamsId'], 'ft', $now);
				}
			}
			else {
				$hid = $history['id'];
				if (DEBUG) echo "  found existing status of $m[status]\n";
			}
		}
	
		//
		// add new history element if new model or status has changed and update the model status
		//

        // only update body if we have data as for cb needs to come from scraping
        $body = '';
        if (!empty($m['body'])) {
            $body = ", body=:body";
        }

		if ($hid == 0) {

			#if ($m['webcamsId'] == 0) { print_r($m); exit; }

			\Avos\Database\query('
				INSERT INTO history (webcamsId, epochTime, status)
				VALUES (:webcamsId, :epochTime, :status)
				', $m);

            if (!empty($m['about'])) {
                \Avos\Database\query("
                    UPDATE webcams 
                    SET 
                        status=:status, imageUrl=:imageUrl, updatedOn=UNIX_TIMESTAMP(), gender=:gender, age=:age,
                        viewers=:viewers, followers=:followers, rank=:rank, genderRank=:genderRank,
                        subject=:subject, about=:about, tags=:tags, location=:location, languages=:languages $body
                    WHERE id=:webcamsId
                ", $m);
            }
            else {
                \Avos\Database\query("
                    UPDATE webcams 
                    SET 
                        status=:status, imageUrl=:imageUrl, updatedOn=UNIX_TIMESTAMP(), gender=:gender, age=:age,
                        viewers=:viewers, followers=:followers, rank=:rank, genderRank=:genderRank,
                        subject=:subject, tags=:tags, location=:location, languages=:languages $body
                    WHERE id=:webcamsId
                ", $m);
            }
	
			if (DEBUG) echo "  added new status of $m[status]\n";
		}
		elseif ($updatedon + $_AVOS['config']['webcams']['refreshViewersInterval'] * 60 < $time) {

			//
			// if no new status just update followers, etc
			//

			if (DEBUG) echo "$m[site] $m[nickname] setting rank to $m[rank]\n";

			\Avos\Database\query("
				UPDATE webcams 
				SET 
					viewers=:viewers, followers=:followers,
                    rank=:rank, genderRank=:genderRank, gender=:gender, 
                    age=:age, imageUrl=:imageUrl
				WHERE id=:webcamsId
				", $m);
		}
	}

	#\Avos\Cron\debug("found $onlineCount models online");
	\Avos\Cron\debug("added $added models, unarchived $unarchived and $changed have changed online status");

	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// set statusTime for any models no longer online
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	$chk = \Avos\Database\queryAssoc('
				SELECT h.* 
				FROM history h 
				LEFT JOIN webcams w ON h.webcamsId=w.id 
				WHERE h.statusTime=0 AND w.site=:site
			', 
			array('site' => $site_short)
		);

	$count = 0;

	foreach ($chk as $m) {
		if (!isset($omodels[$m['webcamsId']])) {
	
			if (DEBUG) echo "CHG: model id $m[webcamsId] logged off\n";
			$count++;
	
			//
			// update model entry
			//
	
			$m['status'] = 'o';
			\Avos\Database\query('UPDATE webcams SET status=:status, updatedOn=UNIX_TIMESTAMP(), rank=0 WHERE id=:webcamsId', $m);
	
			//
			// update history entry with statusTime
			//
	
			$m['statusTime'] = $time - $m['epochTime'];
			\Avos\Database\query('UPDATE history SET statusTime=:statusTime WHERE id=:id', $m);

			//
			// move to archive?
			//

			#archive_models($m['webcamsId']);
		}
	}

	\Avos\Cron\debug("$count models have gone offline");

	//
	// not sure why but models are being left online
	// 

	/*
	$count=0;
	$ass = \Avos\Database\queryList("SELECT id FROM webcams WHERE site=:site AND status!='o'", array('site' => $site_short));

	foreach ($ass as $a) {
		if (!isset($omodels[$a])) {
			\Avos\Database\query("UPDATE webcams SET status='o' WHERE id=:id", array('id' => $a));
			$count++;
		}
	}

	\Avos\Cron\debug("hack... $count models set to offline");
	*/

	//
	// make sure we have a photo
	//

	#download_photos($site, $site_short);
}

function download_photos($site, $site_short)
{
	global $_AVOS;

	$urltop = strtolower($site) . '/';
	$topRelative = 'photos/' . strtolower($site) . '/';
	$top = WEB . $topRelative;

	$ctx = stream_context_create(array(
		'http' => array(
			'timeout' => 20
		)
	));

	if (!file_exists($top)) {
		mkdir($top, 0755, true);
	}

	$intervalHourly = 3600;
	$sexy = " 
				OR (nickname='girlswannasex' AND lastImageTime < UNIX_TIMESTAMP() - $intervalHourly) 
				OR (nickname='carolyngross' AND lastImageTime < UNIX_TIMESTAMP() - $intervalHourly) 
		";
	$sexy = "";

	$interval = $_AVOS['config']['webcams']['refreshImagesInterval'] * 3600;
	$cwebpCmd = $_AVOS['config']['tools']['cwebp'];
	$cwebpQuality = (int)$_AVOS['config']['tools']['cwebpQuality'];

	$useJpegImages = $_AVOS['config']['webcams']['useJpegImages'];
	$useWebpImages = $_AVOS['config']['webcams']['useWebpImages'];

	$models = \Avos\Database\queryAssoc('
					SELECT id, site, nickname, imageUrl, numImages
					FROM webcams 
					WHERE site=:site AND status!="o" AND (numImages = 0 OR lastImageTime < UNIX_TIMESTAMP() - (' . $interval . ' * (100+numImages)/20-4) )', 
						array('site' => $site_short));

	\Avos\Cron\debug("downloading " . count($models) . " photos");

	foreach ($models as $m) {

		# jasmin only has 1 photo
		if ($site_short == 'lj' && $m['numImages'] == 1) {
			\Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP() WHERE id=:id', $m);
			continue;
		}

		# nickname must be already urlsafe - name might not be
		#$nickname = \Avos\Utils\urlsafe(strtolower($m['nickname']), '_');
		$nickname = strtolower($m['nickname']);

		// stripchat doesnt use a .jpg etc file extension
		$ext = 'jpg';

		if (preg_match('!\.(jpg|jpeg|gif|png)$!i', $m['imageUrl'], $e)) {
			$ext = $e[1];
        }
        else {
            $ext = 'jpg';
        }

		if (preg_match('!^(.)(.)(.)(.)!', $nickname, $c)) {

			$folder = strtolower("$c[1]/$c[2]/$c[3]/$c[4]/$nickname/");
	
			if (!file_exists($top . $folder)) {
				if (DEBUG) echo "creating [$top$folder]\n";
				mkdir($top . $folder, 0755, true);
			}
	
			$remote = $m['imageUrl'];
			$image = ($m['numImages'] + 1) . '.' . $ext;
			$local = $top . $folder . $image;

			$webp = ($m['numImages'] + 1) . '.webp';
			$webp = $top . $folder . $webp;

			if (DEBUG) echo "downloading [$remote] => [$local]\n";
			#echo "downloading [$remote] => [$local]\n";
			#echo "downloading to $topRelative$folder\n";

			if (@copy($remote, $local)) {
				if (file_exists($local)) {
					// check isnt 0 bytes (usually from stripchat)
					$tmp = file_get_contents($local);
					if (strlen($tmp) > 0) {
						if ($useWebpImages) {
							if (@system("$cwebpCmd -quiet -q $cwebpQuality $local -o $webp") !== false) {

								if (!$useJpegImages) {
									unlink($local);
								}

                                if (file_exists($webp)) {
                                    cdnSync($topRelative . $folder);
                                    \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=numImages+1 WHERE id=:id', $m);
                                }
							}
                            else {
                                // for some reason the webp conversion failed so delete the jpg
                                unlink($local);
                            }
						}
                        else {
                            if (file_exists($local)) {
                                cdnSync($topRelative . $folder);
                                \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=numImages+1 WHERE id=:id', $m);
                            }
						}
					}
				}
			}
		}

        verify_photos($m, 20, 1);
	}

	//
	// delete empty folders
	//

	$cmd = "find $top -type d -empty -delete";
}

function verify_photos($m, $want=30, $debug=0)
{
    global $_AVOS;

	$cwebp = $_AVOS['config']['tools']['cwebp'];
    $imagePath = \Avos\Utils\imagePath($m);

    if (empty($imagePath)) {
        // known issue if models nickname is less than 4 letters
        if ($debug) echo "error! no image path for $m[nickname]\n";
        return;
    }

    if ($debug)  echo "$m[site]:$m[nickname]:$m[numImages]:$imagePath\n";
   
    $dir = WEB . $imagePath;

    //
    // in case of missing images
    //

    $images = \Avos\Utils\list_files($dir);
    $count = count($images);

    if ($count == 0) {
        \Avos\Database\query('UPDATE webcams SET lastImageTime=0, numImages=0 WHERE id=:id', $m);
        \Avos\Database\query('UPDATE webcamsArchive SET lastImageTime=0, numImages=0 WHERE id=:id', $m);
        return;
    }

    //
    // int sort the array by velue
    //

    $tmp = [];

    foreach ($images as $im) {
        if (preg_match('!(\d+)!', $im, $cap)) {
            $tmp[] = $cap[1];
        }
    }

    sort($tmp, SORT_NUMERIC);

    //
    // this is needed in case we have both a jpg and webp
    //

    $images = array_unique($tmp);

    if ($debug) echo "  " . implode(", ", $images) . "\n";

    //
    // make sure only webp files
    //

    foreach ($images as $i => $im) {

        /*
        if ($debug) {
            echo "  checking for $dir$im.webp";

            if (!file_exists("$dir$im.webp")) {
                echo " - no webp";
            }

            if (!file_exists("$dir$im.jpg")) {
                echo " - no jpg";
            }

            echo "\n";
        }
        */

        //
        // check for failed webp conversion
        //

        if (!file_exists("$dir$im.webp")) {
            if ($debug) echo "    creating $im.webp\n";

            //
            // streamate uses png wtf
            //

            foreach (['jpg', 'png', 'gif'] as $ext) {
                if (file_exists("$dir$im.$ext")) {
                    system("$cwebp -quiet -q 80 $dir$im.$ext -o $dir$im.webp");
                    if ($debug) echo "    removing $im.$ext\n";
                    unlink("$dir$im.$ext");
                }
            }
        }

        //
        // shoudnt get here unless the original file jpg/png/gif didnt download
        // download properly - the original code didnt check for zero size files
        //

        if (!file_exists("$dir$im.webp")) {
            unset($images[$i]);
        }

        //
        // remove any non webp images
        //

        foreach (['jpg', 'png', 'gif'] as $ext) {
            if (file_exists("$dir$im.$ext")) {
                if ($debug) echo "    removing $im.$ext\n";
                unlink("$dir$im.$ext");
            }
        }
    }

    //
    // if we have more images than needed remove some
    //

    if (count($images) > $want) {

        $tmp = [];
        $skip = 1;
        $fac = $want / $count;

        foreach ($images as $im) {

            if ($skip >= 1) {
                $tmp[] = $im;
                $skip--;
            }
            else {
                if ($debug) echo "    removing $im.webp\n";
                @unlink("$dir$im.webp");
            }

            $skip += $fac;
        }

        $images = $tmp;
    }

    #echo "  should now have <= $want images\n";
    #if ($debug) echo "  tmp = " . implode(", ", $tmp) . "\n";

    //
    // create a contiguously numbered array (in case some images are missing)
    // with webp file extensions
    //

    $tmp = [];
    $next = 1;

    foreach ($images as $i => $im) {
        if ($im > $next) {
            #if ($debug) echo "    renaming $dir $im.webp => $next.webp\n";
            rename("$dir$im.webp", "$dir$next.webp");
        }

        $tmp[] = "$next.webp";
        $next++;
    }

    #if ($debug) echo "  tmp = " . implode(", ", $tmp) . "\n";

    //
    // shouldnt need this code
    //

    $images = \Avos\Utils\list_files($dir);
    $count = count($images);

    if ($debug) {
        echo "    images = $count\n    ";
        echo implode(", ", $images);
        echo "\n\n";
    }

    //
    // update the models webcams entry
    //

    $m['want'] = $count;

    \Avos\Database\query('UPDATE webcams SET lastImageTime=UNIX_TIMESTAMP(), numImages=:want WHERE id=:id', $m);
    \Avos\Database\query('UPDATE webcamsArchive SET lastImageTime=UNIX_TIMESTAMP(), numImages=:want WHERE id=:id', $m);
}

function cache_popular()
{
    $redis = \Avos\Redis\handle();
    $key = \Avos\Redis\getSitesPrefix('popular');

    $now = time();
    $numModels = getConfig('webcams', 'popularModelsMax', 1000);
    $numDays = getConfig('webcams', 'popularModelsDays', 30);

    \Avos\Cron\debug("caching the $numModels most popular models from the last $numDays days");

    $popular = false;

    if ($redis->type($key) != \Redis::REDIS_NOT_FOUND) {
        $popular = $redis->get($key);
    }

    if ($popular === false) {
        $popular = [];
    }

    $models = getWebcams(['count' => $numModels, 'sort' => 'viewers']);

    //
    // insert update the current models data in the redis cache
    //

    foreach ($models as $m) {
        if (isset($popular[ $m['id'] ])) {
            if ($m['viewers'] > $popular[ $m['id'] ]['viewers']) {
                $popular[ $m['id'] ] = [ 'viewers' => $m['viewers'], 'ts' => $now ];
            }
            else {
                $popular[ $m['id'] ]['ts'] = $now;
            }
        }
        else {
            $popular[ $m['id'] ] = [ 'viewers' => $m['viewers'], 'ts' => $now ];
        }
    }

    //
    // remove models not updated in $numDays days
    //

    $tooOld = $now - $numDays * 60*60*24;

    foreach ($popular as $id => $m) {
        if ($m['ts'] < $tooOld) {
            unset($popular[$id]);
        }
    }

    //
    // sort by viewers
    //

    uasort($popular, function($a, $b) {
        return $b['viewers'] <=> $a['viewers'];
    });

    //
    // add in rank
    //

    $rank = 1;
    foreach ($popular as $id => $m) {
        $popular[$id]['rank'] = $rank++;
    }

    //
    // stash back to redis
    //

    $redis->set($key, array_slice($popular, 0, $numModels, true));

    /*
    $redis = \Avos\Redis\handle();
    $key = \Avos\Redis\getPrefix('popular');
    $pop = $redis->get($key);

    $archive = \Avos\Database\queryAssoc('SELECT * FROM webcamsArchive WHERE id IN (' . implode(',', array_keys($pop)) . ')');
    $online = \Avos\Database\queryAssoc('SELECT * FROM webcams WHERE id IN (' . implode(',', array_keys($pop)) . ')');

    foreach ($archive as $m) {
        $pop[ $m['id'] ] = array_merge($m, ['viewers' => $pop[ $m['id'] ]['viewers']]);
    }

    foreach ($online as $m) {
        $pop[ $m['id'] ] = array_merge($m, ['viewers' => $pop[ $m['id'] ]['viewers']]);
    }

    setTemplateValue('models', $pop);
    setTemplateValue('online', $online);
    */

    /////////////////////////////////////////////////////////////////////////////////////////////
    //
    // now for any model in the current popular list see if it outranks their
    // previously stored value
    //

    $key = \Avos\Redis\getSitesPrefix('m');
    $cu = 0;
    $cs = 0;

    $sites = str_replace(',', '|', \getConfig('webcams', 'sitesFilter', ''));

    if (!empty($sites)) {
        $sites = "AND " . \Avos\Utils\query2sql("site=$sites");
    }

    $models = \Avos\Database\queryAssoc("SELECT id FROM webcams WHERE status != 'o' AND viewers > 5 $sites ORDER BY viewers DESC LIMIT :numModels", [ 'numModels' => $numModels ]);

    foreach ($models as $i => $m) {
        $mkey = "$key/$m[id]";
        $rank = $i+1;

        if ($redis->type($mkey) == \Redis::REDIS_HASH) {
            $r = $redis->hGetAll($mkey);

            if (isset($m['rank']) && $r['r'] > $rank) {
                $cu++;
                $redis->hSet($mkey, 'ts', (int)$now);
                $redis->hSet($mkey, 'r', $m['rank']);
            }
        }
        else {
            $cs++;
            $redis->hSet($mkey, 'ts', (int)$now);
            $redis->hSet($mkey, 'r', $rank);
        }
    }

    \Avos\Cron\debug("set $cs model ranks and updated $cu");

    /////////////////////////////////////////////////////////////////////////////////////////////
    //
    // update snapshot for today with max viewers+followers for each popular model
    //

    $popularModelsMaxViewersDays = getConfig('webcams', 'popularModelsMaxViewersDays', 30);
    $day = date('ymd');

    \Avos\Cron\debug("updating popular models max viewers+followers snapshot $day");
    $snapshots = \Avos\Redis\get(':popular/snapshots', []);

    if (isset($snapshots[$day])) {
        $snapshot = $snapshots[$day];
    }
    else {
        $snapshot = [];
    }

    //
    // loop round each popular model
    //

    $rank = 1;
    foreach ($popular as $id => $m) {

        $q = \Avos\Database\queryAssoc("SELECT viewers, followers FROM webcams WHERE id=:id AND status !='o'", [ 'id' => $id ]);

        if (count($q) == 1) {

            //
            // ensure we keep a record of the highest viewers and followers for today for each model
            //

            if (isset($snapshot[$id])) {

                $snapshot[$id]['rank'] = $rank;

                if ($q[0]['viewers'] > $snapshot[$id]['viewers']) {
                    $snapshot[$id]['viewers'] = $q[0]['viewers'];
                }
                if ($q[0]['followers'] > $snapshot[$id]['followers']) {
                    $snapshot[$id]['followers'] = $q[0]['followers'];
                }
            }
            else {
                $snapshot[$id] = [ 'viewers' => $q[0]['viewers'], 'followers' => $q[0]['followers'], 'rank' => $rank ];
            }
        }

        $rank++;
    }

    //
    // trim the snapshots to popularModelsMaxViewersDays
    //

    $snapshots[$day] = $snapshot;
    $snapshots = \Avos\Utils\array_psplice($snapshots, -$popularModelsMaxViewersDays, $popularModelsMaxViewersDays);

    //
    // store updated snapshots
    //

    \Avos\Redis\set(':popular/snapshots', $snapshots);
}



function cache_trending()
{
    $trendingModelsSnapshots = getConfig('webcams', 'trendingModelsSnapshots', 10);
    $trendingModelsMax = getConfig('webcams', 'trendingModelsMax', 1000);
    $trendingModels = \Avos\Redis\get(':trending', [ 'snapshots' => [], 'models' => [] ]);

    //
    // only store trendingModelsMax but the trending score is based on both
    // the viewers delta plus absolute value
    //
    // dont call getWebcams() since we only need modelId and viewer count
    //

    $sites = str_replace(',', '|', \getConfig('webcams', 'sitesFilter', ''));

    if (!empty($sites)) {
        $sites = "AND " . \Avos\Utils\query2sql("site=$sites");
    }

    $models = \Avos\Database\queryAssoc("SELECT id, viewers FROM webcams WHERE status != 'o' AND viewers > 5 $sites ORDER BY viewers DESC LIMIT :max", [ 'max' => $trendingModelsMax ]);
    \Avos\Cron\debug("found " . count($models) . " models");

    //
    // append number of viewers indexed by modelId
    //

    while (count($trendingModels['snapshots']) >= $trendingModelsSnapshots) {
        array_shift($trendingModels['snapshots']);
    }

    $mhash = [];

    foreach ($models as $m) {
        $mhash[ $m['id'] ] = $m['viewers'];
    }

    $trendingModels['snapshots'][] = $mhash;
    \Avos\Cron\debug("have " . count($trendingModels['snapshots']) . " snapshots");

    //
    // calculate trending value
    //

    $absolute = [];
    $relative = [];
    $average = [];
    $nmodels = 0;

    foreach ($models as $m) {

        $ascore = 0;
        $rscore = 0;

        for ($i = 1; $i < count($trendingModels['snapshots']); $i++) {

            if (empty($trendingModels['snapshots'][$i][ $m['id'] ])) continue;
            if (empty($trendingModels['snapshots'][$i-1][ $m['id'] ])) continue;

            //
            // absolute changes in viewers
            //

            if (isset($trendingModels['snapshots'][$i][ $m['id'] ])) {
                $ascore += ($trendingModels['snapshots'][$i][ $m['id'] ] - $trendingModels['snapshots'][$i-1][ $m['id'] ]) * sqrt($i);
            }
            else {
                $ascore = 0;
                $i = $trendingModelsSnapshots;
            }

            //
            // relative changes in viewers
            //

            if (isset($trendingModels['snapshots'][$i][ $m['id'] ])) {
                $rscore += ($trendingModels['snapshots'][$i][ $m['id'] ] - $trendingModels['snapshots'][$i-1][ $m['id'] ]) * sqrt($i) / $trendingModels['snapshots'][$i-1][ $m['id'] ];
            }
            else {
                $rscore = 0;
                $i = $trendingModelsSnapshots;
            }
        }

        if ($ascore > 0) {
            $absolute[ $m['id'] ] = $ascore;
            $relative[ $m['id'] ] = $rscore;
            $nmodels++;
        }
    }

    \Avos\Cron\debug("found $nmodels models in every snapshot");

    // used to use 'models' for 'absolute'
    unset($trendingModels['models']);

    arsort($absolute);
    $absolute = array_keys($absolute);
    $trendingModels['absolute'] = $absolute;

    arsort($relative);
    $relative = array_keys($relative);
    $trendingModels['relative'] = $relative;

    //
    // calculate average trending rank
    //

    foreach ($absolute as $i => $m) {
        foreach ($relative as $ii => $m2) {
            if ($m == $m2) {
                // $i starts at 0 not 1 so add abs+rel+2, no need to divide by 2
                $average[$m] = ($i + $ii + 2) / 2;
            }
        }
    }

    arsort($average);
    $average = array_keys($average);
    $trendingModels['average'] = $average;

    \Avos\Redis\set(':trending', $trendingModels);
}


//
// different to popular/trending models since we needed to store a list of popular
// models based on viewers for popularModelsDays days whether currently online or not
// then save snapshots from the last few minutes of online models and compare to
// calculate a trending score from the deltas
//
// here we create a score per category/tag/site based on the number of online models 
// and total viewers (optionally weighted by popularCategoriesRoomsWeight and 
// popularCategoriesViewersWeight) then save several snapshots over time and compare to calculate trending
//


function cache_popular_categories()
{
    $now = time();
    $numSnapshots = getConfig('webcams', 'popularCategoriesSnapshots', 30);
    $pcrw = getConfig('webcams', 'popularCategoriesRoomsWeight', 100);
    $pcvw = getConfig('webcams', 'popularCategoriesViewersWeight', 1);

    \Avos\Cron\debug("caching popularity of categories/tags");

    $popular = \Avos\Redis\get('popular/categories');

    if ($popular === false) {
        $popular = [
            'snapshots' => [
                'categories' => [], 'tags' => [], 'sites' => []
            ], 
            'trending' => []
        ];
    }

    $cats = \Avos\Redis\get('webcams/counts');

    foreach ($cats as $type => $info) {
        $snapshot = [];
        foreach ($info as $urlname => $cat) {
            $snapshot[$urlname] = [
                'fullname' => isset($cat['fullname']) ? $cat['fullname'] : $urlname, 
                'online' => isset($cat['online']) ? $cat['online'] : 0,
                'viewers' => isset($cat['viewers']) ? $cat['viewers'] : 0
            ];
        }

        //
        // sort by viewers
        //

        uasort($snapshot, function($a, $b) use ($pcrw, $pcvw) {
            return ($b['online'] * $pcrw + $b['viewers'] * $pcvw) <=> ($a['online'] * $pcrw + $a['viewers'] * $pcvw);
        });

        //
        // append new snapshot and ensure we only have a max of popularCategoriesSnapshots
        //

        $popular['snapshots'][$type][] = $snapshot;

        while (count($popular['snapshots'][$type]) >= $numSnapshots) {
            array_shift($popular['snapshots'][$type]);
        }
    }

    //
    // calculate the current trending rating aka for trending models
    //

    $popular['trending'] = [];

    foreach ($popular['snapshots'] as $type => $snapshots) {

        for ($i = 1; $i < count($snapshots); $i++) {
            foreach ($snapshots[$i] as $name => $stats) {

                $popular['trending'][$type][$name] = 0;

                if (!empty($snapshots[$i-1][$name])) {
                    $a = $stats;
                    $b = $snapshots[$i-1][$name];

                    $popular['trending'][$type][$name] += 
                        sqrt($i) * (
                            (
                                $a['online'] * $pcrw + $a['viewers'] * $pcvw
                            )
                            - (
                                $b['online'] * $pcrw + $b['viewers'] * $pcvw
                            )
                        );
                }
            }
        }
    }

    //
    // sort the trending lists and replace the scores with cat data
    //

    foreach ($popular['trending'] as $type => $stats) {
        arsort($popular['trending'][$type]);

        $i = 1;
        foreach ($popular['trending'][$type] as $name => $rating) {
            if (!empty($cats[$type][$name])) {
                $popular['trending'][$type][$name] = $cats[$type][$name];
                unset($popular['trending'][$type][$name]['models']);
                $popular['trending'][$type][$name]['rating'] = $rating;
                $popular['trending'][$type][$name]['rank'] = $i++;
            }
        }
    }

    /////////////////////////////////////////////////////////////////////////////////////////////
    //
    // update snapshot for today with max viewers+followers for each cat/tag/site
    //

    $popularCategoriesMaxViewersDays = getConfig('webcams', 'popularCategoriesMaxViewersDays', 30);
    $day = date('ymd');

    \Avos\Cron\debug("updating popular categories max viewers+followers snapshot $day");

    if (isset($popular['daily'][$day])) {
        $snapshot = $popular['daily'][$day];
    }
    else {
        $snapshot = [];
    }

    //
    // loop round each cat/tag/site
    //

    foreach ($popular['snapshots'] as $mode => $list) {
        foreach (end($list) as $name => $s) {

            if (isset($snapshot[$mode][$name])) {
                if ($s['online'] > $snapshot[$mode][$name]['online']) {
                    $snapshot[$mode][$name]['online'] = $s['online'];
                }

                if ($s['viewers'] > $snapshot[$mode][$name]['viewers']) {
                    $snapshot[$mode][$name]['viewers'] = $s['viewers'];
                }
            }
            else {
                $snapshot[$mode][$name]['online'] = $s['online'];
                $snapshot[$mode][$name]['viewers'] = $s['viewers'];
            }
        }
    }

    //
    // store in redis
    //

    $popular['daily'][$day] = $snapshot;
    $popular['daily'] = \Avos\Utils\array_psplice($popular['daily'], -$popularCategoriesMaxViewersDays, $popularCategoriesMaxViewersDays);
    \Avos\Redis\set('popular/categories', $popular);
}

function cache_site_totals()
{
    $allModelsRoomsViewersSnapshotsDays = getConfig('webcams', 'allModelsRoomsViewersSnapshotsDays', 30);
    $now = date('ymdH');
    $totals = \Avos\Redis\get('webcams/totals', ['now' => $now, 'online' => 0, 'viewers' => 0, 'count' => 0, 'snapshots' => []]);

    if ($now != $totals['now']) {
        $totals['snapshots'][$totals['now']] = [ 
            floor($totals['online'] / $totals['count']), 
            floor($totals['viewers'] / $totals['count']) 
        ];

        $totals['now'] = $now;
        $totals['online'] = 0;
        $totals['viewers'] = 0;
        $totals['count'] = 0;
    }

    list($online, $viewers) = \Avos\Database\queryList("SELECT COUNT(id) AS online, SUM(viewers) AS viewers FROM webcams WHERE status != 'o'");

    $totals['online'] += $online;
    $totals['viewers'] += $viewers;
    $totals['count']++;

    debug("caching totals now=$now count=$totals[count] online=$totals[online] viewers=$totals[viewers]");

    $maxSnapshots = $allModelsRoomsViewersSnapshotsDays * 24;
    $totals['snapshots'] = \Avos\Utils\array_psplice($totals['snapshots'], -$maxSnapshots, $maxSnapshots);
    \Avos\Redis\set('webcams/totals', $totals);
}

function cache_models_updates()
{
    $updates = \Avos\Redis\get('webcams/updates', ['cache' => [], 'stream' => []]);
    $statusStreamModelsMax = getConfig('webcams', 'statusStreamModelsMax', 100);
    $now = time();

    //
    // the updates stream contains the following updates on models:
    // * logged on or status change
    // * been online over 5, 8, 10, 12 hours
    // * increased their wanking into top 1, 10, 50, 100
    // * gained over 1000 viewers
    //

    //
    // fetch the list of models we want to check:
    // 1 - all online popular models
    // 2 - max $statusStreamModelsMax online trending webcams
    // 3 - max $statusStreamModelsMax online models with the most viewers
    //
    // use relative mode for trending webcams since will also sort by absolute number of viewers
    //

    $models = [];

    foreach (
        [ array_slice(getPopularWebcams(true), 0, $statusStreamModelsMax), 
        array_slice(getTrendingWebcams('relative', true), 0, $statusStreamModelsMax) ] as $l) {
            foreach ($l as $m) {
                $models[$m['id']] = 1;
            }
    }

    $sites = str_replace(',', '|', \getConfig('webcams', 'sitesFilter', ''));

    if (!empty($sites)) {
        $sites = "AND " . \Avos\Utils\query2sql("site=$sites");
    }

    foreach (\Avos\Database\queryList("SELECT id FROM webcams WHERE status != 'o' $sites ORDER BY viewers DESC LIMIT 100") as $id) {
        $models[$id] = 1;
    }

    $models = extractWebcamsFields(array_keys($models), true);

    //
    // if a model isnt already in the cache then they logged on
    //

    foreach ($models as $pos => $m) {

        $m['timestamp'] = $now;
        $m['rank'] = $pos+1;
        $m['onlineMins'] = 0;
        $q = \Avos\Database\queryList('SELECT epochTime FROM history WHERE webcamsId=:id', $m);

        if (!empty($q[0])) {
            $m['onlineMins'] = floor(($now - $q[0]) / 60);
        }

        //
        // ensure model is in the cache
        //

        cache_models_updates__append($updates, $m);

        //
        // if current onlineMins is smaller than the cache then set onlineMinsBase so we can
        // calculate total online minutes = onlineMins + onlineMinsBase
        //

        if ($m['onlineMins'] < $updates['cache'][$m['id']]['onlineMins']) {
            $updates['cache'][$m['id']]['onlineMinsBase'] = $updates['cache'][$m['id']]['onlineMins'];
        }

        //
        // status change
        //

        if ($m['status'] != $updates['cache'][$m['id']]['status']) {
            cache_models_updates__append($updates, $m, [
                'update' => 'status', 
                'oldStatus' => $updates['cache'][$m['id']]['status']
            ]);
        }

        //
        // see if model has been online a long time
        //

        $onlineMins = $updates['cache'][$m['id']]['onlineMinsBase'] + $m['onlineMins'];

        foreach ([5, 8, 10, 12] as $hours) {
            if ($onlineMins > $hours * 3600 && $updates['cache'][$m['id']]['onlineMinsAlarm' . $hours] == 0) {
                $updates['cache'][$m['id']]['onlineMinsAlarm' . $hours] = 1;

                cache_models_updates__append($updates, $m, [
                    'update' => 'hours',
                    'onlineHours' => $hours
                ]);
            }
        }

        //
        // see if model has jumped up the wankings
        //

        $onlineMins = $updates['cache'][$m['id']]['onlineMinsBase'] + $m['onlineMins'];

        foreach ([1, 10, 50, 100] as $rank) {
            if ($m['rank'] <= $rank && $updates['cache'][$m['id']]['rankAlarm' . $rank] == 0) {
                $updates['cache'][$m['id']]['rankAlarm' . $rank] = 1;

                cache_models_updates__append($updates, $m, [
                    'update' => 'rank',
                    'rankUpdate' => $rank
                ]);
            }
        }

        //
        // see if model has gained a certain number of viewers over the time logged
        //

        if ($m['viewers'] > $updates['cache'][$m['id']]['viewersMin'] + 1000) {

            cache_models_updates__append($updates, $m, [
                'update' => 'viewers',
                'viewersIncr' => $m['viewers'] - $updates['cache'][$m['id']]['viewersMin'],
                'viewersIncrMins' => floor(($now - $updates['cache'][$m['id']]['viewersMinTimestamp']) / 60)
            ]);

            $updates['cache'][$m['id']]['viewersMin'] = $m['viewers'];
            $updates['cache'][$m['id']]['viewersMinTimestamp'] = $now;
        }

        //
        // sync the cache with the current values
        //

        foreach (['timestamp', 'rank', 'status', 'viewers', 'onlineMins'] as $f) {
            $updates['cache'][$m['id']][$f] = $m[$f];
        }
    }

    //
    // delete any model from the cache if older than 1 hour
    //

    foreach ($updates['cache'] as $id => $m) {
        if ($m['timestamp'] < $now - 3600) { # 1 hour
            unset($updates['cache'][$id]);
            \Avos\Cron\debug("expiring $m[site]:$m[nickname] cache");
        }
    }

    //
    // remove old items from the stream
    //

    $updatesStreamMaxHours = getConfig('webcams', 'updatesStreamMaxHours', 8);

    $i = 0;
    while ($updates['stream'][$i]['timestamp'] < $now - $updatesStreamMaxHours * 3600) $i++;

    if ($i > 0) {
        \Avos\Cron\debug("expiring oldest $i items from stream");
        $updates['stream'] = array_splice($updates['stream'], $i);
    }

    //
    // delete entries from the stream older than $webcamUpdatesMaxAge
    //

    \Avos\Redis\set('webcams/updates', $updates);
}

function cache_models_updates__append(&$updates, $m, $news=[])
{

    //
    // model logged on if no history record
    //

    if (!isset($updates['cache'][$m['id']])) {

        $updates['cache'][$m['id']] = [
            'timestamp' => $m['timestamp'],
            'site' => $m['site'],
            'nickname' => $m['nickname'],
            'status' => $m['status'],
            'rank' => $m['rank'],
            'rankMin' => $m['rank'],
            'rankMinTimestamp' => $m['timestamp'],
            'viewers' => $m['viewers'],
            'viewersMin' => $m['viewers'],
            'viewersMinTimestamp' => $m['timestamp'],
            'rankAlarm1' => $m['rank'] == 1 ? 1 : 0,
            'rankAlarm10' => $m['rank'] <= 10 ? 1 : 0,
            'rankAlarm50' => $m['rank'] <= 50 ? 1 : 0,
            'rankAlarm100' => $m['rank'] <= 100 ? 1 : 0,
            'rankAlarm200' => $m['rank'] <= 200 ? 1 : 0,
            'rankAlarm300' => $m['rank'] <= 300 ? 1 : 0,
            'onlineMins' => $m['onlineMins'],
            'onlineMinsBase' => 0,
            'onlineMinsAlarm5' => 0,
            'onlineMinsAlarm8' => 0,
            'onlineMinsAlarm10' => 0,
            'onlineMinsAlarm12' => 0
        ];

        //
        // if already online for a few minutes then not just logged on
        // either an error or joined the trending or popular lists
        //

        if ($m['onlineMins'] < 180) {
            cache_models_updates__append($updates, $m, ['update' => 'loggedOn']);
        }
    }

    if (!isset($news['update'])) {
        return;
    }

    //
    // log the update
    //

    $updates['stream'][] = array_merge([
        'id' => $m['id'],
        'timestamp' => $m['timestamp'],
        'rank' => $m['rank'],
        'site' => $m['site'],
        'nickname' => $m['nickname'],
        'status' => $m['status'],
        'viewers' => $m['viewers']
    ], $news);

    $info = "$m[siteShortName]:$m[nickname] $news[update]";
    unset($news['update']);

    \Avos\Cron\debug($info . implode(',', array_map(function($k) use ($news) { return " $k=" . $news[$k]; }, array_keys($news))));
}

function process_history()
{
	global $_AVOS;

	$redis = $_AVOS['redis'];
	$stats = \Avos\Database\queryAssoc('SELECT * FROM history WHERE statusTime != 0');

	\Avos\Cron\debug("processing " . count($stats) . " history records");

	//
	// process all unprocessed individual stats that are completed
	//

	foreach ($stats as $s) {

		if (DEBUG) echo "processing: id=$s[id], webcamsId=$s[webcamsId], status=$s[status], statusTime=$s[statusTime]\n";

		//
		// get year/month/day
		//

		$stat = array(
			'webcamsId' => $s['webcamsId'],
			'year' => date('y', $s['epochTime']),
			'month' => date('m', $s['epochTime']),
			'day' => date('d', $s['epochTime'])
		);

		/*
		 * pre redis
		 *

		$dailyStat = \Avos\Database\queryAssoc('SELECT * FROM dailyStats WHERE year=:year AND month=:month AND day=:day AND webcamsId=:webcamsId', $stat);

		if (count($dailyStat) == 0) {
			\Avos\Database\query('INSERT INTO dailyStats (webcamsId, year, month, day) VALUES (:webcamsId, :year, :month, :day)', $stat);
			$dailyStat = \Avos\Database\queryAssoc('SELECT * FROM dailyStats WHERE year=:year AND month=:month AND day=:day AND webcamsId=:webcamsId', $stat);
		}



		#echo "stat = "; print_r($stat); echo "\n";
		#echo "dailyStat = "; print_r($dailyStat); echo "\n"; #FISH
		
		//
		// merge in the new data
		//

		$smap = array(
			'o' => 'offline',
			'a' => 'away',
			'f' => 'freechat',
			'g' => 'group',
			'p' => 'private'
		);

		$dailyStat = $dailyStat[0];
		$dailyStat[ $smap[ $s['status'] ] . 'Sessions' ]++;
		$dailyStat[ $smap[ $s['status'] ] . 'Time' ] += $s['statusTime'];

		if ($s['statusTime'] > $dailyStat[ $smap[ $s['status'] ] . 'TimeMax' ]) {
			$dailyStat[ $smap[ $s['status'] ] . 'TimeMax' ] = $s['statusTime'];
		}

		//
		// save the new data
		//

		\Avos\Database\query('
			UPDATE dailyStats
			SET 
				awayTime=:awayTime, awaySessions=:awaySessions, awayTimeMax=:awayTimeMax,
				freechatTime=:freechatTime, freechatSessions=:freechatSessions, freechatTimeMax=:freechatTimeMax,
				groupTime=:groupTime, groupSessions=:groupSessions, groupTimeMax=:groupTimeMax,
				privateTime=:privateTime, privateSessions=:privateSessions, privateTimeMax=:privateTimeMax
			WHERE
				id=:id
			',
			$dailyStat);
		*/

		//------------------------------------------------------------------------------
		// make sure we have a dailyStats entry
		//

		//
		// redis
		//
		// * false - dont reverse entries - tho will only be 0 or 1 anyway
		// * false - dont expand the names ie: from 'as' to 'awaySessions'
		//

		$dailyStat = \Avos\Redis\getStat($s['webcamsId'], 'd', $stat['year'], $stat['month'], $stat['day'], false);
		
		#$year = sprintf("%02d", $stat['year']);
		#$month = sprintf("%02d", $stat['month']);
		#$day = sprintf("%02d", $stat['day']);

		#$key = "$s[webcamsId]/d/$year$month$day";
		#if (DEBUG) echo "$key\n";
		#$dailyStat = $redis->get($key);

		/*
		if (count($dailyStat) == 0) {
			if (DEBUG) echo "no existing daily stats record\n";
			$dailyStat = array(
							'as' => 0, 'at' => 0, 'atm' => 0,
							'fs' => 0, 'ft' => 0, 'ftm' => 0,
							'gs' => 0, 'gt' => 0, 'gtm' => 0,
							'ps' => 0, 'pt' => 0, 'ptm' => 0
						);
		}
		*/

		foreach (array('as', 'at', 'atm', 'fs', 'ft', 'ftm', 'gs', 'gt', 'gtm', 'ps', 'pt', 'ptm') as $afgp) {
			if (!isset($dailyStat[$afgp])) {
				$dailyStat[$afgp] = 0;
			}
		}

		#$dailyStat = $dailyStat[0];
		$dailyStat[ $s['status'] . 's' ]++;
		$dailyStat[ $s['status'] . 't' ] += $s['statusTime'];

		if ($s['statusTime'] > $dailyStat[ $s['status'] . 'tm' ]) {
			$dailyStat[ $s['status'] . 'tm' ] = $s['statusTime'];
		}

		if (DEBUG) echo "setting daily stats record\n";
		\Avos\Redis\setStat($s['webcamsId'], 'd', $stat['year'], $stat['month'], $stat['day'], $dailyStat);
		#$redis->set($key, $dailyStat);
		#if (DEBUG) echo "set daily stats record\n";

		//------------------------------------------------------------------------------
		// make sure we have a monthlyStats entry
		//

		$monthlyStat = \Avos\Redis\getStat($s['webcamsId'], 'm', $stat['year'], $stat['month'], '', false);

		#$key = "$s[webcamsId]/m/$year$month";
		#$monthlyStat = $redis->get($key);

		/*
		if (count($monthlyStat) == 0) {
			if (DEBUG) echo "no existing monthly stats record\n";
			$monthlyStat = array(
								'as' => 0, 'at' => 0, 'atm' => 0,
								'fs' => 0, 'ft' => 0, 'ftm' => 0,
								'gs' => 0, 'gt' => 0, 'gtm' => 0,
								'ps' => 0, 'pt' => 0, 'ptm' => 0
						);
		}
		*/

		foreach (array('as', 'at', 'atm', 'fs', 'ft', 'ftm', 'gs', 'gt', 'gtm', 'ps', 'pt', 'ptm') as $afgp) {
			if (!isset($monthlyStat[$afgp])) {
				$monthlyStat[$afgp] = 0;
			}
		}

		#$monthlyStat = $monthlyStat[0];
		$monthlyStat[ $s['status'] . 's' ]++;
		$monthlyStat[ $s['status'] . 't' ] += $s['statusTime'];

		if ($s['statusTime'] > $monthlyStat[ $s['status'] . 'tm' ]) {
			$monthlyStat[ $s['status'] . 'tm' ] = $s['statusTime'];
		}

		if (DEBUG) echo "setting monthly stats record\n";
		\Avos\Redis\setStat($s['webcamsId'], 'm', $stat['year'], $stat['month'], '', $monthlyStat);
		#$redis->set($key, $monthlyStat);
		#if (DEBUG) echo "set monthly stats record\n";

		/*
		 * pre redis
		 *

		$monthlyStat = \Avos\Database\queryAssoc('SELECT * FROM monthlyStats WHERE year=:year AND month=:month AND webcamsId=:webcamsId', $stat);

		if (count($monthlyStat) == 0) {
			\Avos\Database\query('INSERT INTO monthlyStats (webcamsId, year, month) VALUES (:webcamsId, :year, :month)', $stat);
		$monthlyStat = \Avos\Database\queryAssoc('SELECT * FROM monthlyStats WHERE year=:year AND month=:month AND webcamsId=:webcamsId', $stat);
		}

		//
		// merge in the new data
		//

		$monthlyStat = $monthlyStat[0];
		$monthlyStat[ $smap[ $s['status'] ] . 'Sessions' ]++;
		$monthlyStat[ $smap[ $s['status'] ] . 'Time' ] += $s['statusTime'];

		if ($s['statusTime'] > $monthlyStat[ $smap[ $s['status'] ] . 'TimeMax' ]) {
			$monthlyStat[ $smap[ $s['status'] ] . 'TimeMax' ] = $s['statusTime'];
		}

		//
		// save the new data
		//

		\Avos\Database\query('
			UPDATE monthlyStats
			SET 
				awayTime=:awayTime, awaySessions=:awaySessions, awayTimeMax=:awayTimeMax,
				freechatTime=:freechatTime, freechatSessions=:freechatSessions, freechatTimeMax=:freechatTimeMax,
				groupTime=:groupTime, groupSessions=:groupSessions, groupTimeMax=:groupTimeMax,
				privateTime=:privateTime, privateSessions=:privateSessions, privateTimeMax=:privateTimeMax
			WHERE
				id=:id
			',
			$monthlyStat);
		*/

		//
		// move the history entry to redis
		//

		$key = "$s[webcamsId]/h/$stat[year]$stat[month]";
		$mins = (int)(($s['epochTime'] - EPOCHSTART) / 60);
		$e = array('e' => $mins, 's' => $s['status'], 't' => $s['statusTime']);

		if ($redis->rPush($key, $e) !== FALSE) {
			if (DEBUG) echo "stored in redis\n";
			\Avos\Database\query("DELETE FROM history WHERE id=:id", $s);
			if (DEBUG) echo "deleted from mysq\n";
		}
	}
}

# if model not in db first call this
# return models id

function unarchive_model($r)
{
	// fetch model from webcamsArchive table

	$m = \Avos\Database\queryAssoc('SELECT * FROM webcamsArchive WHERE nickname=:nickname AND site=:site LIMIT 1', $r);

	if (count($m) == 1) {

		// check the model hasnt been hidden
		if ($m[0]['updatedOn'] == 0) {
			return -1;
		}

		// check its not already in webcams table

		$n = \Avos\Database\queryAssoc('SELECT id FROM webcams WHERE id=:id LIMIT 1', $m[0]);

		if (count($n) > 0) {
			echo "* already in webcams table\n";
			return $m[0]['id'];
		}

		// set updatedon
		$m[0]['updatedOn'] = time();

		// insert back into webcams table

		\Avos\Database\query('
			INSERT INTO webcams (
					id, addedOn, updatedOn, nickname, name, site, subject, about, age, 
                    imageUrl, lastImageTime, numImages, followers, gender, hd, tags,
                    scrapedOn, birthday, location, languages, links
				)
				VALUES (
					:id, :addedOn, :updatedOn, :nickname, :name, :site, :subject, :about, :age, 
					:imageUrl, :lastImageTime, :numImages, :followers, :gender, :hd, :tags,
                    :scrapedOn, :birthday, :location, :languages, :links
				)
			', $m[0]);

		#\Avos\Cron\debug("unarchiving model " . $m[0]['nickname'] . ' ' . $m[0]['id']);

        // reset the purged flag
        \Avos\Database\query('UPDATE webcamsArchive SET purged=0 WHERE id=:id', $m[0]);

		return $m[0]['id'];
	}

	return false;
}


function archive_models($id=false)
{
	global $_AVOS;

	$interval = $_AVOS['config']['webcams']['archiveInterval'];
	$oldest = time() - $interval * 86400;

	if ($id != false) {
		$models = \Avos\Database\queryAssoc("SELECT * FROM webcams WHERE id=$id");
	}
	else {
		#\Avos\Cron\debug("SELECT * FROM webcams WHERE updatedOn < UNIX_TIMESTAMP() - $interval * 86400");
		$models = \Avos\Database\queryAssoc("SELECT * FROM webcams WHERE updatedOn < UNIX_TIMESTAMP() - $interval * 86400");
		\Avos\Cron\debug("archiving " . (count($models)) . " models");
	}

	foreach ($models as $m) {
	
		//
		// first see if the model is already in the webcamsArchive table
		// all fields including id match the main table for a particular model
		//

		$chk = \Avos\Database\queryList("SELECT id FROM webcamsArchive WHERE id=:id", $m);

		if (count($chk) == 0) {

			#\Avos\Cron\debug("archiving model $m[nickname] $m[site] $m[id]");

			\Avos\Database\query('
				INSERT INTO webcamsArchive (
						id, addedOn, updatedOn, nickname, name, site, subject, about, age, 
						imageUrl, lastImageTime, numImages, followers, gender, hd, tags,
                        scrapedOn, birthday, location, languages, links, viewers
					)
					VALUES (
						:id, :addedOn, :updatedOn, :nickname, :name, :site, :subject, :about, :age, 
						:imageUrl, :lastImageTime, :numImages, :followers, :gender, :hd, :tags,
                        :scrapedOn, :birthday, :location, :languages, :links, :viewers
					)
				', $m);

			\Avos\Database\query('DELETE FROM webcams WHERE id=:id', $m);
		}
		else {

			#\Avos\Cron\debug("updating archive for $m[nickname] $m[site] $m[id]");

            if (!empty($m['about'])) {
                \Avos\Database\query('
                    UPDATE webcamsArchive
                    SET 
                        updatedOn=:updatedOn, name=:name, subject=:subject, about=:about, age=:age, 
                        imageUrl=:imageUrl, lastImageTime=:lastImageTime, numImages=:numImages, 
                        followers=:followers, gender=:gender, hd=:hd, tags=:tags, links=:links, viewers=:viewers
                    WHERE
                        id=:id
                    ', $m);
            }
            else {
                \Avos\Database\query('
                    UPDATE webcamsArchive
                    SET 
                        updatedOn=:updatedOn, name=:name, subject=:subject, age=:age, 
                        imageUrl=:imageUrl, lastImageTime=:lastImageTime, numImages=:numImages, 
                        followers=:followers, gender=:gender, hd=:hd, tags=:tags, links=:links, viewers=:viewers
                    WHERE
                        id=:id
                    ', $m);
            }
			\Avos\Database\query('DELETE FROM webcams WHERE id=:id', $m);
		}
	}
}


function trimLogs()
{
	global $_AVOS;

	$dir = $_AVOS['config']['site']['avosDir'] . 'logs/';
	$max = $_AVOS['config']['site']['logsMaxSize'];
	$logs = \Avos\Utils\list_files($dir);
    $tmpfile = "/tmp/trimlogs." . getmypid();

	foreach ($logs as $l) {
		$lines = file($dir . $l);

		if (count($lines) > $max) {
            \Avos\Cron\debug("trimming $l");
			//$trimmed = array_slice($lines , 0 - $max);
			//file_put_contents($dir . $l, implode("", $trimmed));
            system("cat $dir$l | tail -$max > $tmpfile");
            system("mv $tmpfile $dir$l");
		}
    }

    if (file_exists($tmpfile)) {
        unlink($tmpfile);
    }
}


function expireRequestsLog()
{
    if (($expiry = getConfig('site', 'logRequestsExpiry', 0)) > 0) {

        $redis = \Avos\Redis\handle();
        $prefix = \Avos\Redis\getPrefix('requests');
        $len = $redis->lLen($prefix);
        $oldest = time() - $expiry*3600;
        $deleted = 0;

        for ($i=0; $i<$len; $i++) {
            if ($redis->lindex($prefix, 0)[0] < $oldest) {
                $redis->lPop($prefix);
                $deleted++;
            }
            else {
                break;
            }
        }

        if ($deleted > 0) {
            \Avos\Cron\debug("removed $deleted items from requests log in redis $prefix");
        }
    }
}


function cdnSync($dirRelative = 'photos')
{
	global $_AVOS;

	$cdnUrl = '';
	$cdnPath = '';
	$dirRelative = rtrim($dirRelative, '/');
	$cdnDirRelative = dirname($dirRelative) . '/';
	$webDir = $_AVOS['config']['site']['webDir'];
	$rsyncCmd = $_AVOS['config']['tools']['rsync'];

	if (isset($_AVOS['config']['site']['cdnUrl']) && !empty($_AVOS['config']['site']['cdnUrl'])) {
		$cdnUrl = $_AVOS['config']['site']['cdnUrl'];

		// remove protocol and any trailing path info
		if (preg_match('!^https?://(.+)?(/.*/)!', $cdnUrl, $c)) {
			$cdnUrl = $c[1];
		}
	}
	else {
		// no cdn defined
		return;
	}

	if (isset($_AVOS['config']['site']['cdnPath'])) {
		$cdnPath = $_AVOS['config']['site']['cdnPath'];
	}

	#\Avos\Cron\debug("local: $webDir$dirRelative");
	#\Avos\Cron\debug("remote: $cdnUrl:$cdnPath$cdnDirRelative");

	$cmd = "$rsyncCmd -avr $webDir$dirRelative $cdnUrl:$cdnPath$cdnDirRelative";
	$fh = popen($cmd, 'r');

	while($line = fgets($fh)) {
		$line = rtrim($line);
		if (preg_match('!(.*\.(jpg|webp))$!', $line, $c)) {
			$image = "$webDir$cdnDirRelative$line";
			$imageDir = dirname($image);
			unlink($image);
			#\Avos\Cron\debug("$cdnDirRelative$line");

			// might not exist if synching jpg+webp
			if (file_exists($imageDir)) {
				@rmdir($imageDir);
			}
		}
		flush();
	}

	pclose($fh);
	system($cmd);
}


function refreshCategories()
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

    \Avos\Database\disableCache();

	//
	// force query2sql to expand the category or tag
	//

    $_AVOS['page']['type'] = 'webcams';

    //
    // optionally store all webcams data not just 'lite'
    //

    $webcamsLite = getConfig('site', 'cronWebcamsLite', 1);

    //
    // see if we are using cats/tags from a shared redis/mysql
    //

    $categoriesPrefix = getConfig('redis', 'categoriesPrefix', '');

    if (!empty($categoriesPrefix)) {
        return refreshCategoriesShared($categoriesPrefix);
    }

	//
	// categories
	//

	$stats = array();
    \Avos\Database\query("UPDATE webcamsCategories SET keep=0");
    $lc = listWebcamsCategories();

	foreach ($lc as $c) {
        $category = $c['category'];
        $stats[$category] = $c;
        $stats[$category]['fullname'] = $c['name'];
        $stats[$category]['online'] = 0;
        $stats[$category]['rooms'] = 0;
        $stats[$category]['viewers'] = 0;

        \Avos\Cron\debug("category $category");

        // need to expand cats/tags here in case contains ||
        $filter = \getWebcamsCategory($category, 'categories');

        if ($filter !== FALSE) {
            #$sql = \Avos\Utils\query2sql(array('category' => $category), true);
            $sql = \Avos\Utils\query2sql($filter['filter']);
            #\Avos\Database\query('INSERT INTO webcamsCategories (name, webcamsId) SELECT "' . $category . '", id FROM webcams WHERE ' . $sql);
            $models = \Avos\Database\queryAssoc('SELECT "' . $category . '" AS category, id FROM webcams WHERE ' . $sql);

            foreach ($models as $m) {
                \Avos\Database\query('INSERT INTO webcamsCategories (name, webcamsId) VALUES (:category, :id) ON DUPLICATE KEY UPDATE keep=1', $m);
            }

            \Avos\Database\query("DELETE FROM webcamsCategories WHERE keep=0 AND name='$category'");
        }
	}

	\Avos\Database\query("DELETE FROM webcamsCategories WHERE keep=0");
    $counts = array('categories' => $stats);

	//
	// save the categories list of models in redis
	//

    foreach ($lc as $c) {
        $models = \Avos\Database\queryAssoc("SELECT wc.webcamsId AS i, w.site AS s, w.gender AS g FROM webcamsCategories wc, webcams w WHERE wc.name=:category AND wc.webcamsId=w.id", $c);
        $key = \Avos\Redis\getPrefix("webcams/tag/$c[category]");
        \Avos\Cron\debug("saving to [$key]");
        $redis->set($key, $models);
        $redis->expire($key, 3600);
    }

    /*
	//
	// cache the number of webcam models per category/gender
	//

	\Avos\Cron\debug("refreshing categories counts");
	$perPage = $_AVOS['config']['webcams']['perPage'];
    #$perPage = 20; #shouldnt want more than 20 models in a related cams section or pages like tags etc

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsCategories c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&category=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&category=$name$sort");
            }
	}

    $counts = array('categories' => $stats);
    */

	//
	// tags
	//

	$stats = array();
	\Avos\Database\query("UPDATE webcamsTags SET keep=0");
	$lc = listWebcamsCategories('tags');

	foreach ($lc as $c) {
        $category = $c['tag'];
        $stats[$category] = $c;
        $stats[$category]['fullname'] = $c['name'];
        $stats[$category]['online'] = 0;
        $stats[$category]['rooms'] = 0;
        $stats[$category]['viewers'] = 0;

        \Avos\Cron\debug("tag $category");

        // need to expand cats/tags here in case contains ||
        $filter = \getWebcamsCategory($category, 'tags');

        if ($filter !== FALSE) {
            #$sql = \Avos\Utils\query2sql(array('tag' => $category), true);
            $sql = \Avos\Utils\query2sql($filter['filter']);
            #\Avos\Database\query('INSERT INTO webcamsTags (name, webcamsId) SELECT "' . $category . '", id FROM webcams WHERE ' . $sql);
            $models = \Avos\Database\queryAssoc('SELECT "' . $category . '" AS category, id FROM webcams WHERE ' . $sql);

            foreach ($models as $m) {
                \Avos\Database\query('INSERT INTO webcamsTags (name, webcamsId) VALUES (:category, :id) ON DUPLICATE KEY UPDATE keep=1', $m);
            }

            \Avos\Database\query("DELETE FROM webcamsTags WHERE keep=0 AND name='$category'");
        }
    }

	\Avos\Database\query("DELETE FROM webcamsTags WHERE keep=0");

	//
	// save the tags list of models in redis
	//

    foreach ($lc as $c) {
        $models = \Avos\Database\queryAssoc("SELECT wc.webcamsId AS i, w.site AS s, w.gender AS g FROM webcamsTags wc, webcams w WHERE wc.name=:tag AND wc.webcamsId=w.id", $c);
        $key = \Avos\Redis\getPrefix("webcams/tag/$c[tag]");
        \Avos\Cron\debug("saving to [$key]");
        $redis->set($key, $models);
        $redis->expire($key, 3600);
    }

    $counts['tags'] = $stats;
    refreshCategoriesCounts($counts);

    /*
	//
	// cache the number of webcam models per tag/gender
	//

	\Avos\Cron\debug("refreshing tags counts");

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsTags c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&tag=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&tag=$name$sort");
            }
	}

    $counts['tags'] = $stats;

	//
	// cache the number of webcam models per site
	//

	$stats = array();

	//
	// online models is easy
	//

	\Avos\Cron\debug("refreshing webcams counts");
	$online = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcams WHERE status!='o' GROUP BY site");

	foreach ($online as $s) {
		$stats[$s['site']] = array('online' => $s['num']);
	}

	//
	// archived models
	//

	$offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcamsArchive GROUP BY site");

	foreach ($offline as $s) {
		$stats[$s['site']]['archived'] = $s['num'];
	}

	//
	// offline models needs two tables searching
	//

	$offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcams WHERE status='o' GROUP BY site");

	foreach ($offline as $s) {
		$stats[$s['site']]['offline'] = $s['num'];
	}

	$offline = \Avos\Database\queryAssoc("
					SELECT COUNT(*) AS num, site 
					FROM webcamsArchive
					WHERE id NOT IN (
						SELECT id FROM webcams
					)	
					GROUP BY site");

	foreach ($offline as $s) {
		if (isset($stats[$s['site']]['offline'])) {
			$stats[$s['site']]['offline'] += $s['num'];
		}
		else {
			$stats[$s['site']]['offline'] = $s['num'];
		}
	}

	// force query2sql to add status!=o if not already set
    $_AVOS['page']['type'] = 'webcams';
	
	foreach ($stats as $site => $s) {

		// in case no archived or online models

		if (!isset($s['online'])) {
			$stats[$site]['online'] = 0;
		}

		if (!isset($s['archived'])) {
			$stats[$site]['archived'] = 0;
		}

		$stats[$site]['fullname'] = \Avos\Scaffold\expandSiteName($site);
		$stats[$site]['site'] = $stats[$site]['fullname'];
		$stats[$site]['total'] = $stats[$site]['online'] + $stats[$site]['offline'];

        if ($webcamsLite) {
            $stats[$site]['models'] = \getWebcamsLite("count=$perPage&site=$site");
        }
        else {
            $stats[$site]['models'] = \getWebcams("count=$perPage&site=$site");
        }

		$stats[$site]['matches'] = $_AVOS['page']['matches'];
        $stats[$site]['pages'] = $_AVOS['page']['pages'];
    }

    // for cron output
    $total = 0;
    foreach ($stats as $s => $v) {
        $total += $v['total'];
    }
    \Avos\Cron\debug("total cams: $total");

	// reset for any further requests
	$_AVOS['page']['type'] = '';

	$counts['sites'] = $stats;
	#\Avos\Database\query('UPDATE cache SET value=:value WHERE name="webcams/counts"', array('value' => serialize($counts)));

	//
	// save online counts for cats+tags
	//

	if (\getConfig('webcams', 'getCatsTags')) {
		\Avos\Cron\debug("counting cats/tags intersections");
		$cats = \listWebcamsCategories('categories');
		$tags = \listWebcamsCategories('tags');
		$_AVOS['page']['type'] = 'webcams';
		$catsTags = array();

		foreach ($cats as $c) {
			foreach ($tags as $t) {
				$sql = \Avos\Utils\query2sql(array('category' => $c['category'], 'tag' => $t['tag']));
				list($count, $viewers) = \Avos\Database\queryList("SELECT COUNT(*), SUM(viewers) FROM webcams WHERE $sql");
				$catsTags[$c['category']][$t['tag']]['category'] = $c['category'];
				$catsTags[$c['category']][$t['tag']]['tag'] = $t['tag'];
				$catsTags[$c['category']][$t['tag']]['fullname'] = $t['fullname'];
				$catsTags[$c['category']][$t['tag']]['online'] = $count;
				$catsTags[$c['category']][$t['tag']]['rooms'] = $count;
				$catsTags[$c['category']][$t['tag']]['viewers'] = $viewers;

                if ($webcamsLite) {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcamsLite("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
                else {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcams("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
			}
		}

		$counts['catstags'] = $catsTags;
	}

	//
	// save the data in redis
	//

	$key = \Avos\Redis\getPrefix('webcams/counts');
	\Avos\Cron\debug("saving to [$key]");
    $redis->set($key, $counts);
    */
}

function refreshCategoriesCounts($counts)
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

    \Avos\Database\disableCache();

	//
	// force query2sql to expand the category or tag
	//

    $_AVOS['page']['type'] = 'webcams';

    //
    // optionally store all webcams data not just 'lite'
    //

    $webcamsLite = getConfig('site', 'cronWebcamsLite', 1);

	//
	// cache the number of webcam models per category/gender
	//

	\Avos\Cron\debug("refreshing categories counts");
	$perPage = $_AVOS['config']['webcams']['perPage'];
    #$perPage = 20; #shouldnt want more than 20 models in a related cams section or pages like tags etc
    $stats = $counts['categories'];

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsCategories c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&category=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&category=$name$sort");
            }
	}

	$counts['categories'] = $stats;

	//
	// cache the number of webcam models per tag/gender
	//

	\Avos\Cron\debug("refreshing tags counts");
    $stats = $counts['tags'];

	$online = \Avos\Database\queryAssoc("
			SELECT c.name, w.gender, COUNT(*) AS rooms, SUM(viewers) AS viewers 
			FROM webcamsTags c 
			LEFT JOIN webcams w ON c.webcamsId=w.id 
			GROUP BY c.name, w.gender
			");

	foreach ($online as $s) {
        $stats[$s['name']]['online'] += $s['rooms'];
        $stats[$s['name']]['rooms'] += $s['rooms'];
        $stats[$s['name']]['viewers'] += $s['viewers'];
		$stats[$s['name']]['bygender'][$s['gender']]['online'] = $s['rooms'];
		$stats[$s['name']]['bygender'][$s['gender']]['viewers'] = $s['viewers'];
    }

    foreach ($stats as $name => $c) {

            // see if we have a sort=
            $sort = '';
            parse_str($c['filter'], $filter);
            if (isset($filter['sort'])) {
                $sort = "&sort=$filter[sort]";
            }

            if ($webcamsLite) {
                $stats[$name]['models'] = \getWebcamsLite("count=$perPage&tag=$name$sort");
            }
            else {
                $stats[$name]['models'] = \getWebcams("count=$perPage&tag=$name$sort");
            }
	}

	$counts['tags'] = $stats;

	//
	// cache the number of webcam models per site
	//

	$stats = array();

	//
	// online models is easy
	//

	\Avos\Cron\debug("refreshing webcams counts");
	$online = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site, SUM(viewers) AS viewers FROM webcams WHERE status!='o' GROUP BY site");

	foreach ($online as $s) {
		$stats[$s['site']] = array('online' => $s['num'], 'viewers' => $s['viewers']);
	}

    if (getConfig('webcams', 'countArchive', 0)) {
        //
        // archived models
        //

        $offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcamsArchive GROUP BY site");

        foreach ($offline as $s) {
            $stats[$s['site']]['archived'] = $s['num'];
        }
    }

	//
	// offline models needs two tables searching
	//

	$offline = \Avos\Database\queryAssoc("SELECT COUNT(*) AS num, site FROM webcams WHERE status='o' GROUP BY site");

	foreach ($offline as $s) {
		$stats[$s['site']]['offline'] = $s['num'];
	}

    if (getConfig('webcams', 'countArchive', 0)) {
        $offline = \Avos\Database\queryAssoc("
                        SELECT COUNT(*) AS num, site 
                        FROM webcamsArchive
                        WHERE id NOT IN (
                            SELECT id FROM webcams
                        )	
                        GROUP BY site");

        foreach ($offline as $s) {
            if (isset($stats[$s['site']]['offline'])) {
                $stats[$s['site']]['offline'] += $s['num'];
            }
            else {
                $stats[$s['site']]['offline'] = $s['num'];
            }
        }
    }

	// force query2sql to add status!=o if not already set
    $_AVOS['page']['type'] = 'webcams';
	
	foreach ($stats as $site => $s) {

		// in case no archived or online models

		if (!isset($s['online'])) {
			$stats[$site]['online'] = 0;
		}

		if (!isset($s['archived'])) {
			$stats[$site]['archived'] = 0;
		}

		$stats[$site]['fullname'] = \Avos\Scaffold\expandSiteName($site);
		$stats[$site]['site'] = $stats[$site]['fullname'];
		$stats[$site]['total'] = $stats[$site]['online'] + $stats[$site]['offline'];

        if ($webcamsLite) {
            $stats[$site]['models'] = \getWebcamsLite("count=$perPage&site=$site");
        }
        else {
            $stats[$site]['models'] = \getWebcams("count=$perPage&site=$site");
        }

		$stats[$site]['matches'] = $_AVOS['page']['matches'];
        $stats[$site]['pages'] = $_AVOS['page']['pages'];
    }

    // for cron output
    $total = 0;
    foreach ($stats as $s => $v) {
        $total += $v['total'];
    }
    \Avos\Cron\debug("total cams: $total");

	// reset for any further requests
	$_AVOS['page']['type'] = '';

	$counts['sites'] = $stats;
	#\Avos\Database\query('UPDATE cache SET value=:value WHERE name="webcams/counts"', array('value' => serialize($counts)));

	//
	// save online counts for cats+tags
	//

    if (!empty(getUrl('webcamsCategoryTag'))) {
		\Avos\Cron\debug("counting cats/tags intersections");
		$cats = \listWebcamsCategories('categories');
		$tags = \listWebcamsCategories('tags');
		$_AVOS['page']['type'] = 'webcams';
		$catsTags = array();

		foreach ($cats as $c) {
			foreach ($tags as $t) {
				$sql = \Avos\Utils\query2sql(array('category' => $c['category'], 'tag' => $t['tag']));
				list($count, $viewers) = \Avos\Database\queryList("SELECT COUNT(*), SUM(viewers) FROM webcams WHERE $sql");
				$catsTags[$c['category']][$t['tag']]['category'] = $c['category'];
				$catsTags[$c['category']][$t['tag']]['tag'] = $t['tag'];
				$catsTags[$c['category']][$t['tag']]['fullname'] = $t['fullname'];
				$catsTags[$c['category']][$t['tag']]['online'] = $count;
				$catsTags[$c['category']][$t['tag']]['rooms'] = $count;
				$catsTags[$c['category']][$t['tag']]['viewers'] = $viewers;

                if ($webcamsLite) {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcamsLite("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
                else {
                    $catsTags[$c['category']][$t['tag']]['models'] = \getWebcams("count=$perPage&category=$c[category]&tag=$t[tag]");
                }
			}
		}

		$counts['catstags'] = $catsTags;
	}

	//
	// save the data in redis
	//

	$key = \Avos\Redis\getPrefix('webcams/counts');
	\Avos\Cron\debug("saving to [$key]");
	$redis->set($key, $counts);
}

function refreshCategoriesShared($sharedPrefix)
{
    global $_AVOS;

	//
	// map cached cats/tags in the shared redis prefix to our own
	//

    \Avos\Cron\debug("synching stats from redis:$sharedPrefix");

    //
    // populate our own webcamsCategories+Tags tables from the shared one
    //

    $db = getConfig('mysql', 'categoriesDatabase', '');

    if (empty($db)) {
        $db = getConfig('mysql', 'webcamsDatabase', '');
    }

    \Avos\Cron\debug("synching tables from mysql:$db");

    $prefix = \Avos\Redis\getPrefix();
    $key = "$sharedPrefix/webcams/counts";
    $redis = \Avos\Redis\handle();
    $shared = $redis->get($key);
    $sitesFilter = \getConfig('webcams', 'sitesFilter', '');
    $counts = [];

    //
    // same actions for categories then tags
    //

    foreach (['categories', 'tags'] as $type) {

        $stats = [];
        $key = $type == 'categories' ? 'category' : 'tag';
        $cats = \listWebcamsCategories($type);

        $table = 'webcams' . ucfirst($type);
        \Avos\Database\query("UPDATE $table SET keep=0");

        //
        // loop round each category/tag defined for this site
        //

        foreach ($cats as $c) {
            if (isset($shared['tags'][$c['filter']])) {

                $stats[$c[$key]] = $c;
                $stats[$c[$key]]['fullname'] = $c['name'];
                $stats[$c[$key]]['online'] = 0;
                $stats[$c[$key]]['rooms'] = 0;
                $stats[$c[$key]]['viewers'] = 0;

                //
                // found a matching category/tag in the shared redis install
                //

                #\Avos\Cron\debug("$key " . $c['filter'] . "... $c[$key]");
                $shared['tags'][$c['filter']][$key] = $c[$key];
                #$counts[$type][$c[$key]] = $shared[$type][$c['filter']];

                //
                // sync our local webcamsCategories/Tags table to the shared mysql
                //

                #\Avos\Database\query("INSERT INTO $table (name, webcamsId) SELECT DISTINCT '$c[$key]', webcamsId FROM $db.$table WHERE name = '$c[filter]'");
                $rkey = "$sharedPrefix/webcams/tag/$c[filter]";
                $models = $redis->get($rkey);
                $matched = 0;

                $catGender = $c['gender'];
                $catGenderEmpty = empty($catGender);

                if (is_array($models)) {
                    foreach ($models as $m) {
                        $m['name'] = $c[$key];
                        if ((empty($sitesFilter) || strpos($sitesFilter, $m['s']) !== false) && ($catGenderEmpty || strpos($catGender, $m['g']) !== false )) {
                            \Avos\Database\query("INSERT INTO $table (name, webcamsId) VALUES (:name, :i) ON DUPLICATE KEY UPDATE keep=1", $m);
                            $matched++;
                        }
                    }
                }

                \Avos\Cron\debug("$key $c[filter]... $rkey matched $matched models");
            }
            else {

                //
                // no matching category/tag in the shared redis install
                //

                \Avos\Cron\debug("$key " . $c['filter'] . "... ERROR - SKIPPED");
            }

            $counts[$type] = $stats;
        }

        //
        // clean out any stale entries
        //

        \Avos\Database\query("DELETE FROM $table WHERE keep=0");
    }

    refreshCategoriesCounts($counts);

    /*
	//
    // save online counts for cats+tags if:
    //   the [urls] contains webcamsCategoryTag rule
    //   [webcams] contains getCatsTags=1 (will override)
	//

    $urls = getConfig('urls');
    $getCatsTags = 0;

    foreach ($urls as $name => $u) {
        if ($name == 'webcamsCategoryTag') {
            $getCatsTags = 1;
        }
    }

	if (\getConfig('webcams', 'getCatsTags', $getCatsTags)) {
		\Avos\Cron\debug("counting cats/tags intersections");
		$cats = \listWebcamsCategories('categories');
		$tags = \listWebcamsCategories('tags');
        $catsTags = array();

		foreach ($cats as $c) {
            foreach ($tags as $t) {

                $category = $c['category'];
                $tag = $t['tag'];

                $sharedCategory = $c['filter'];
                $sharedTag = $t['filter'];

                if (isset($shared['catstags'][$sharedCategory][$sharedTag])) {
                    $counts['catstags'][$category][$tag] = $shared['catstags'][$sharedCategory][$sharedTag];
                }
			}
		}
	}

	//
	// save the data in redis
	//

	$key = \Avos\Redis\getPrefix('webcams/counts');
	\Avos\Cron\debug("saving to [$key]");
    $redis->set($key, $counts);
     */
}

function refreshVideosTags()
{
	global $_AVOS;
	$videos = \Avos\Database\queryAssoc('SELECT id, tags FROM videos');
	\Avos\Database\query("UPDATE videosTags SET keep=0");

	foreach ($videos as $v) {
		$tags = explode(',', $v['tags']);

		foreach ($tags as $t) {
			if (!empty($t)) {
				\Avos\Database\query("INSERT INTO videosTags (name, videosId) VALUES (:name, :id)",
					array(
						'name' => $t,
						'id' => $v['id']
					)
				);
			}
		}
	}

	\Avos\Database\query("DELETE FROM videosTags WHERE keep=0");
}

function refreshPhotosTags()
{
	global $_AVOS;
	$videos = \Avos\Database\queryAssoc('SELECT id, tags FROM photos');
	\Avos\Database\query("UPDATE photosTags SET keep=0");

	foreach ($videos as $v) {
		$tags = explode(',', $v['tags']);

		foreach ($tags as $t) {
			if (!empty($t)) {
				\Avos\Database\query("INSERT INTO photosTags (name, photosId) VALUES (:name, :id)",
					array(
						'name' => $t,
						'id' => $v['id']
					)
				);
			}
		}
	}

	\Avos\Database\query("DELETE FROM photosTags WHERE keep=0");
}


/* pete site maps 
function refreshSitemap($createIndex = true)
{
	global $_AVOS;

	$webDir = $_AVOS['config']['site']['webDir'];
	$url = rtrim($_AVOS['config']['site']['url'], '/');
	$max = 50000;
	$indexes = 0;
	$gzip = '';
	$gz = '';
    $timestamp = getConfig('webcams', 'sitemapsTimestamp', 'updatedOn');
    \Avos\Cron\debug("web: $webDir");

	if (isset($_AVOS['config']['site']['gzipSitemaps']) && $_AVOS['config']['site']['gzipSitemaps'] == true) {
		$gzip = $_AVOS['config']['tools']['gzip'];
		$gz = '.gz';
	}

	$urlsetHeader = '<' . '?xml version="1.0" encoding="UTF-8"?' . ">\n" . '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
	$urlsetFooter = '</urlset>' . "\n";

	//
	// home + webcams categories
	//

    if (getConfig('webcams', 'sitemapsCategories', 1)) {
        $counts = getWebcamsCounts();
        $urlset = "<url><loc>$url/</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";

        foreach ($counts['categories'] as $c) {
            $curl = $url .encodeUrl('webcamsCategory', $c);
            $urlset .= "<url><loc>$curl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";
        }

        foreach ($counts['tags'] as $c) {
            $curl = $url . encodeUrl('webcamsTag', $c);
            $urlset .= "<url><loc>$curl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";
        }

        \Avos\Cron\debug('writing categories sitemap.' . $indexes . '.xml');
        file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
        if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
        $indexes++;
    }


	//
	// videos
	//

	$done = 0;
	$offset = 0;
	$max = 50000;
    $videosUrl = getConfig('videos', 'sitemapsUrlName');

	while (!$done) {
		$items = \Avos\Database\queryAssoc("SELECT id, urlTitle, publishedOn FROM videos WHERE publishedOn>0 ORDER BY publishedOn DESC LIMIT $offset, $max");

		if (count($items) > 0) {

			$urlset = '';

			foreach ($items as $i) {
				$i['title'] = $i['urlTitle'];
				$murl = $url . encodeUrl($videosUrl, $i);
				$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $i['publishedOn']) . "</lastmod></url>\n";
			}

	 		\Avos\Cron\debug('writing videos sitemap.' . $indexes . '.xml');
			file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
			if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
			$offset += $max;
			$indexes++;
		}
		else {
			$done = 1;
		}
	}

    //
    // photos
    //

	$done = 0;
	$offset = 0;
    $photosUrl = getConfig('photos', 'sitemapsUrlName');

	while (!$done) {
		$items = \Avos\Database\queryAssoc("SELECT id, urlTitle, publishedOn FROM photos WHERE publishedOn>0 ORDER BY publishedOn DESC LIMIT $offset, $max");

		if (count($items) > 0) {

			$urlset = '';

			foreach ($items as $i) {
				$i['title'] = $i['urlTitle'];
				$murl = $url . encodeUrl($photosUrl, $i);
				$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $i['publishedOn']) . "</lastmod></url>\n";
			}

	 		\Avos\Cron\debug('writing photos sitemap.' . $indexes . '.xml');
			file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
			if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
			$offset += $max;
			$indexes++;
		}
		else {
			$done = 1;
		}
	}

	//
	// webcams
	//

    if (getConfig('webcams', 'sitemapsWebcams', 1)) {

        $done = 0;
        $offset = 0;
        $webcamsProfile = getConfig('webcams', 'sitemapsUrlName');
        $webcamsProfileFn = '';

        if (preg_match('!(.*)\(\)$!', $webcamsProfile, $c) && function_exists('\\Avos\\User\\' . $c[1])) {
            $webcamsProfileFn = '\\Avos\\User\\' . $c[1];
        }
        else if (function_exists('\\Avos\\User\\webcamsProfileFn')) {
            $webcamsProfileFn = '\\Avos\\User\\webcamsProfileFn';
        }

        $sitesSql = '';
        $sitesFilter = str_replace(',', '|', \getConfig('webcams', 'sitesFilter', ''));
        if (!empty($sitesFilter)) $sitesSql = "WHERE " . \Avos\Utils\query2sql("site=$sitesFilter");

        while (!$done) {
            $models = \Avos\Database\queryAssoc("SELECT nickname, site, addedOn, updatedOn FROM webcams $sitesSql ORDER BY updatedOn DESC LIMIT $offset, $max");

            if (count($models) > 0) {

                $urlset = '';

                foreach ($models as $m) {
                    $m['site'] = \Avos\Scaffold\longSiteName($m['site']);

                    if (getConfig('site', 'lowerCaseUrls')) {
                        $m['site'] = strtolower($m['site']);
                    }

                    if (!empty($webcamsProfileFn)) {
                        $murl = $url . call_user_func($webcamsProfileFn, $m);
                    }
                    else {
                        $murl = $url . encodeUrl($webcamsProfile, $m);
                    }

                    $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $m[$timestamp]) . "</lastmod></url>\n";
                }

                \Avos\Cron\debug('writing webcams sitemap.' . $indexes . '.xml');
                file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
                if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
                $offset += $max;
                $indexes++;
            }
            else {
                $done = 1;
            }
        }

        //
        // webcamsArchive
        //

        if (getConfig('webcams', 'sitemapsArchive', 1)) {

            $lastOnline = getConfig('webcams', 'sitemapsLastOnline', 0);
            $lastOnlineSql = '';

            if ($lastOnline > 0) {
                $lastOnlineSql = 'updatedOn > ' . (time() - $lastOnline * 60*60*24) . ' AND';
            }

            $done = 0;
            $offset = 0;
            if (!empty($sitesFilter)) $sitesSql = \Avos\Utils\query2sql("site=$sitesFilter") . " AND";

            while (!$done) {
                $models = \Avos\Database\queryAssoc("SELECT nickname, site, addedOn, updatedOn FROM webcamsArchive WHERE $sitesSql $lastOnlineSql id NOT IN (SELECT id FROM webcams)  ORDER BY updatedOn DESC LIMIT $offset, $max");

                if (count($models) > 0) {

                    $urlset = '';

                    foreach ($models as $m) {
                        $m['site'] = \Avos\Scaffold\longSiteName($m['site']);

                        if (getConfig('site', 'lowerCaseUrls')) {
                            $m['site'] = strtolower($m['site']);
                        }

                        if (!empty($webcamsProfileFn)) {
                            $murl = $url . call_user_func($webcamsProfileFn, $m);
                        }
                        else {
                            $murl = $url . encodeUrl($webcamsProfile, $m);
                        }

                        $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $m[$timestamp]) . "</lastmod></url>\n";
                    }

                    \Avos\Cron\debug('writing webcamsArchive sitemap.' . $indexes . '.xml');
                    file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
                    if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
                    $offset += $max;
                    $indexes++;
                }
                else {
                    $done = 1;
                }
            }
        }
    }

	//
	// write the index
	//

    if ($createIndex) {
        $index  = '<' . '?xml version="1.0" encoding="UTF-8"?' . '>' . "\n";
        $index .= '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
        $date = date(DATE_W3C);

        for ($i = 0; $i < $indexes; $i++) {
            $index .= " <sitemap>\n  <loc>$url/sitemap.$i.xml$gz</loc>\n  <lastmod>$date</lastmod>\n </sitemap>\n";
        }

        $index .= '</sitemapindex>' . "\n";

        \Avos\Cron\debug('writing index sitemap.xml');
        file_put_contents($webDir . 'sitemap.xml', $index);
     }

     //
     // ping search engines
     //



     $key = getConfig('site', 'indexNowKey', '');
     if (!empty($key)) {
         if (file_exists($webDir . $key . '.txt')) {
             $ping = "https://api.indexnow.org/indexnow?key=$key&url=$url";
             if (file_get_contents($ping) !== false) {
                 \Avos\Cron\debug("indexnow:submitted $ping");
             }
         }
         else {
             \Avos\Cron\debug("site.indexNowKey file does not exist as $webDir$key.txt");
         }
     }
}
*/


function refreshSitemap()
{
	global $_AVOS;

	$webDir = $_AVOS['config']['site']['webDir'];
	$url = rtrim($_AVOS['config']['site']['url'], '/');
	$max = 50000;
	$indexes = 0;
	$gzip = '';
	$gz = '';
    $timestamp = getConfig('webcams', 'sitemapsTimestamp', 'updatedOn');
    \Avos\Cron\debug("web: $webDir");

	if (isset($_AVOS['config']['site']['gzipSitemaps']) && $_AVOS['config']['site']['gzipSitemaps'] == true) {
		$gzip = $_AVOS['config']['tools']['gzip'];
		$gz = '.gz';
	}

	$urlsetHeader = '<' . '?xml version="1.0" encoding="UTF-8"?' . ">\n" . '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
	$urlsetFooter = '</urlset>' . "\n";







	//
	// home + webcams categories
	//

    if (getConfig('webcams', 'sitemapsCategories', 1)) {
        $counts = getWebcamsCounts();
        $urlset = "<url><loc>$url/</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";


		//print_r($counts['id']);exit;


        foreach ($counts['categories'] as $c) {
            $curl = $url .encodeUrl('webcamsCategory', $c);
            $urlset .= "<url><loc>$curl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";
        }

        foreach ($counts['tags'] as $c) {
            $curl = $url . encodeUrl('webcamsTag', $c);
            $urlset .= "<url><loc>$curl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n";
        }

        \Avos\Cron\debug('writing categories sitemap.' . $indexes . '.xml');
        file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
        if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
        $indexes++;
    }




//
// blogs sitemap
//

$done = 0;
$offset = 0;
$blogsUrl = getConfig('blog', 'sitemapsUrlName');

while (!$done) {
    // Query to fetch the necessary fields from ArticleBlogPosts table
    $items = \Avos\Database\queryAssoc("
        SELECT id, Title, publishedOn 
        FROM ArticleBlogPosts 
        WHERE publishedOn IS NOT NULL 
        ORDER BY publishedOn DESC 
        LIMIT $offset, $max
    ");

    if (count($items) > 0) {
        $urlset = '';

        foreach ($items as $i) {
            // Process the title to replace spaces with hyphens but keep colons
            $i['Title'] = strtolower($i['Title']); // Convert to lowercase
            $i['Title'] = preg_replace('/\s+/', '-', $i['Title']); // Replace spaces with hyphens
            // Keep colons and other allowed special characters
            $i['Title'] = preg_replace('/[^a-z0-9:-]+/', '-', $i['Title']); // Replace other special chars with hyphen
            $i['Title'] = trim($i['Title'], '-'); // Remove trailing hyphens
            
            // Generate the final URL
            $murl = $url . encodeUrl($blogsUrl, $i);

            // Format the `publishedOn` datetime to `Y-m-d`
            $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', strtotime($i['publishedOn'])) . "</lastmod></url>\n";
        }

        \Avos\Cron\debug('writing blogs sitemap.' . $indexes . '.xml');
        
        // Write the sitemap XML file
        file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
        
        // Compress the file if gzip is available
        if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");

        // Increment the offset and index for pagination
        $offset += $max;
        $indexes++;
    } else {
        // No more items to process, exit loop
        $done = 1;
    }
}




//
// promote sitemap
//

$done = 0;
$offset = 0;
$promoteUrl = getConfig('promote', 'sitemapsUrlName');
while (!$done) {
    // Query to fetch the necessary fields from the promote table
    $items = \Avos\Database\queryAssoc("
        SELECT siteId, SiteLink, siteNames, publishedOn, title 
        FROM promote 
        WHERE publishedOn IS NOT NULL 
        ORDER BY publishedOn DESC  
        LIMIT $offset, $max
    ");
    if (count($items) > 0) {
        $urlset = '';
        foreach ($items as $i) {
            // Ensure that the correct URL format is used
            $i['SiteLink'] = trim($i['SiteLink']); // Remove any extra spaces
            // Use the correct title or SiteLink as needed
			
            $murl = $url . encodeUrl($promoteUrl, $i);
            // Format the `publishedOn` datetime to `Y-m-d`
            $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', strtotime($i['publishedOn'])) . "</lastmod></url>\n";
        }
        \Avos\Cron\debug('writing promote sitemap.' . $indexes . '.xml');
        // Write the sitemap XML file
        file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
        // Compress the file if gzip is available
        if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");

        // Increment the offset and index for pagination
        $offset += $max;
        $indexes++;
    } else {
        // No more items to process, exit loop
        $done = 1;
    }
}


    //
    // photos
    //

	$done = 0;
	$offset = 0;
    $photosUrl = getConfig('photos', 'sitemapsUrlName');

	while (!$done) {
		$items = \Avos\Database\queryAssoc("SELECT id, urlTitle, publishedOn FROM photos WHERE publishedOn>0 ORDER BY publishedOn DESC LIMIT $offset, $max");

		if (count($items) > 0) {

			$urlset = '';

			foreach ($items as $i) {
				$i['title'] = $i['urlTitle'];
				$murl = $url . encodeUrl($photosUrl, $i);
				$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $i['publishedOn']) . "</lastmod></url>\n";
			}

	 		\Avos\Cron\debug('writing photos sitemap.' . $indexes . '.xml');
			file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
			if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
			$offset += $max;
			$indexes++;
		}
		else {
			$done = 1;
		}
	}

	//
	// webcams
	//

    if (getConfig('webcams', 'sitemapsWebcams', 1)) {

        $done = 0;
        $offset = 0;
        $webcamsProfile = getConfig('webcams', 'sitemapsUrlName');
        $webcamsProfileFn = '';

        if (preg_match('!(.*)\(\)$!', $webcamsProfile, $c) && function_exists('\\Avos\\User\\' . $c[1])) {
            $webcamsProfileFn = '\\Avos\\User\\' . $c[1];
        }
        else if (function_exists('\\Avos\\User\\webcamsProfileFn')) {
            $webcamsProfileFn = '\\Avos\\User\\webcamsProfileFn';
        }

        $sitesSql = '';
        $sitesFilter = str_replace(',', '|', \getConfig('webcams', 'sitesFilter', ''));
        if (!empty($sitesFilter)) $sitesSql = "WHERE " . \Avos\Utils\query2sql("site=$sitesFilter");

        while (!$done) {
            $models = \Avos\Database\queryAssoc("SELECT nickname, site, addedOn, updatedOn FROM webcams $sitesSql ORDER BY updatedOn DESC LIMIT $offset, $max");

            if (count($models) > 0) {

                $urlset = '';

                foreach ($models as $m) {
                    $m['site'] = \Avos\Scaffold\longSiteName($m['site']);

                    if (getConfig('site', 'lowerCaseUrls')) {
                        $m['site'] = strtolower($m['site']);
                    }

                    if (!empty($webcamsProfileFn)) {
                        $murl = $url . call_user_func($webcamsProfileFn, $m);
                    }
                    else {
                        $murl = $url . encodeUrl($webcamsProfile, $m);
                    }

                    $urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $m[$timestamp]) . "</lastmod></url>\n";
                }

                \Avos\Cron\debug('writing webcams sitemap.' . $indexes . '.xml');
                file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
                if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
                $offset += $max;
                $indexes++;
            }
            else {
                $done = 1;
            }
        }

    

	//
	// videos
	//

	$done = 0; 
	$offset = 0;
	$max = 50000;
    $videosUrl = getConfig('videos', 'sitemapsUrlName');

	while (!$done) {
		$items = \Avos\Database\queryAssoc("SELECT id, urlTitle, publishedOn FROM videos WHERE publishedOn>0 ORDER BY publishedOn DESC LIMIT $offset, $max");

		if (count($items) > 0) {

			$urlset = '';

			foreach ($items as $i) {
				$i['title'] = $i['urlTitle'];
				$murl = $url . encodeUrl($videosUrl, $i);
				$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $i['publishedOn']) . "</lastmod></url>\n";
			}

	 		\Avos\Cron\debug('writing videos sitemap.' . $indexes . '.xml');
			file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
			if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
			$offset += $max;
			$indexes++;
		}
		else {
			$done = 1;
		}
	}


/* sitemap version for webcams razvan ideea  only the live ones -*/
	if (getConfig('webcams', 'sitemapsArchive', 1)) {

		$lastOnline = getConfig('webcams', 'sitemapsLastOnline', 0);
		$lastOnlineSql = '';

		if ($lastOnline > 0) {
			$lastOnlineSql = 'updatedOn > ' . (time() - $lastOnline * 60*60*24) . ' AND';
		}

		$done = 0;
		$offset = 0;
		if (!empty($sitesFilter)) $sitesSql = \Avos\Utils\query2sql("site=$sitesFilter") . " AND";

		while (!$done) {
			$models = \Avos\Database\queryAssoc("SELECT nickname, site, addedOn, updatedOn FROM webcamsArchive WHERE $sitesSql $lastOnlineSql id NOT IN (SELECT id FROM webcams)  ORDER BY updatedOn DESC LIMIT $offset, $max");

			if (count($models) > 0) {

				$urlset = '';

				foreach ($models as $m) {
					$m['site'] = \Avos\Scaffold\longSiteName($m['site']);

					if (getConfig('site', 'lowerCaseUrls')) {
						$m['site'] = strtolower($m['site']);
					}

					if (!empty($webcamsProfileFn)) {
						$murl = $url . call_user_func($webcamsProfileFn, $m);
					}
					else {
						$murl = $url . encodeUrl($webcamsProfile, $m);
					}

					$urlset .= "<url><loc>$murl</loc><lastmod>" . date('Y-m-d', $m[$timestamp]) . "</lastmod></url>\n";
				}

				\Avos\Cron\debug('writing webcamsArchive sitemap.' . $indexes . '.xml');
				file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
				if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
				$offset += $max;
				$indexes++;
			}
			else {
				$done = 1;
			}
		}
	}
}

//end of raz ideea

//menu sitemaps hope will be the last one this will make a sitemap with all the urls from [urls] --ok is skiping some for now 
$offset = 0;
$urls = \Avos\Scaffold\getUrlConfig()['urls']; // Fetch your URL configuration
$tmp = \Avos\Database\queryList("select count(*) from videos");
$counts['videos'] = $tmp[0];
$tmp = \Avos\Database\queryList("select count(*) from photos");
$counts['photos'] = $tmp[0];
$tmp = \Avos\Database\queryList("select count(*) from ArticleBlogPosts;");
$counts['blogs'] = $tmp[0];
$tmp = \Avos\Database\queryList("select count(*) from promote;");
$counts['promote'] = $tmp[0];
$urlsetHeader = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">\n";
$urlsetFooter = "</urlset>\n";
$urlset = ''; // Initialize empty string to accumulate URLs


foreach ($urls as $u) {
	$tmpl = $u[0];
	$type = $u[1];
	$path = $u[2];

	// Skip URLs that do not match required types
	if (preg_match('!\..*!', $path)) continue;
	 // Skip URLs that contain "hideFloatingChat"
	if (strpos($path, 'hideFloatingChat') !== false) continue;
	if (!preg_match('!(none|webcams?|videos?|photos?|blogs?|users?|casino?)!', $type)) continue;
	if ($type == 'redirect') continue;
	if ($type == 'users') continue;
	if (preg_match('!(test|fish|foo|game)!', $path)) continue; // Skip the test files

	// Skip URLs containing placeholders like :site or :nickname
	if (preg_match('!:(site|nickname|id|title|_month|_year|siteId|username|urlTitle|tag|hideFloatingChat|category)!', $path)) continue;

	// Skip .css or similar static file extensions
	if (preg_match('!\.(css|js|json|xml)$!', $path)) continue;

	// Replace known placeholders except :page
	$path = str_replace(
		[':id', ':siteId', ':title', ':urlTitle', ':nickname', ':kw', ':Title', ':siteNames', ':username', ':_month', ':_year'],
		'*',
		$path
	);

	// Check if the URL uses pagination (:page placeholder)
if (strpos($path, ':page') !== false) {
    // Determine items per page based on type
    $perPage = 36; // Default value
    if ($type === 'photos') {
        $perPage = getConfig('photos', 'perPage');
    } elseif ($type === 'blogs') {
        $perPage = getConfig('blogs', 'perPage');
    } elseif ($type === 'videos') {
        $perPage = getConfig('videos', 'perPage');
    } elseif ($type === 'promote') {
        $perPage = getConfig('promote', 'perPage');
    }

    // Get total items count from $counts array
    $totalItems = 0;
    if (isset($counts['photos']) && $type === 'photos') {
        $totalItems = $counts['photos'];
    } elseif (isset($counts['blogs']) && $type === 'blogs') {
        $totalItems = $counts['blogs'];
    } elseif (isset($counts['videos']) && $type === 'videos') {
        $totalItems = $counts['videos'];
    } elseif (isset($counts['promote']) && $type === 'promote') {
        $totalItems = $counts['promote'];
    }

    // Calculate total pages
    $totalPages = ceil($totalItems / $perPage);

    // First, output the base URL (without pagination)
    $finalUrl = $url . str_replace(':page', '', $path); // Remove the :page placeholder for the base URL
    $urlset .= "<url><loc>$finalUrl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n"; // Add the base URL to the sitemap

    // Now, generate paginated URLs (starting from page 1)
    if ($totalPages > 0) {  // Ensure totalPages is set to a valid number
        for ($i = 1; $i <= $totalPages; $i++) {
            // Replace :page with the actual page number to generate paginated URLs
            $finalUrl = $url . str_replace(':page', $i, $path);
            $urlset .= "<url><loc>$finalUrl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n"; // Add the paginated URL to the sitemap
        }
    }
} else {
    // If no pagination is needed, just output the URL without :page placeholder
    $finalUrl = $url . $path; // Prepend the base URL
    $urlset .= "<url><loc>$finalUrl</loc><lastmod>" . date('Y-m-d') . "</lastmod></url>\n"; // Add the non-paginated URL to the sitemap
}



	
}

// Write sitemap to file
\Avos\Cron\debug('writing main menu url sitemap.' . $indexes . '.xml');
file_put_contents($webDir . 'sitemap.' . $indexes . '.xml', $urlsetHeader . $urlset . $urlsetFooter);
if ($gzip != '') system("$gzip -f ${webDir}sitemap.$indexes.xml");
$offset += $max;
$indexes++;




/* sitemap version for webcams razvan ideea */



	//
	// write the index
	//

	$index  = '<' . '?xml version="1.0" encoding="UTF-8"?' . '>' . "\n";
	$index .= '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
	$date = date(DATE_W3C);

	for ($i = 0; $i < $indexes; $i++) {
		$index .= " <sitemap>\n  <loc>$url/sitemap.$i.xml$gz</loc>\n  <lastmod>$date</lastmod>\n </sitemap>\n";
	 }

	 $index .= '</sitemapindex>' . "\n";

	 \Avos\Cron\debug('writing index sitemap.xml');
     file_put_contents($webDir . 'sitemap.xml', $index);

   











}



function purgeHistory()
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	//
	// on redis deadlock a closed history record will be left in mysql
	// if older than 12 hours delete else move to redis
	// also if statusTime > 4hrs delete
	//

	$h = \Avos\Database\queryAssoc("
					SELECT * FROM history 
					WHERE (statusTime > 0 AND UNIX_TIMESTAMP() - epochTime > 3600*12)
					OR statusTime > 3600*4
				");

	foreach ($h as $s) {
		$year = date('y', $s['epochTime']);
		$month = date('m', $s['epochTime']);

		$key = "$s[webcamsId]/h/$year$month";
		$mins = (int)(($s['epochTime'] - EPOCHSTART) / 60);
		$e = array('e' => $mins, 's' => $s['status'], 't' => $s['statusTime']);

		echo "$key - "; print_r($e);

		/*
		if ($redis->rPush($key, $e) !== FALSE) {
			if (DEBUG) echo "stored in redis\n";
			\Avos\Database\query("DELETE FROM history WHERE id=:id", $s);
			if (DEBUG) echo "deleted from mysq\n";
		}
		 */
	}
}

function purge($mode='daily')
{
	global $_AVOS;
	$redis = $_AVOS['redis'];
	
	list($maxw) = \Avos\Database\queryList("SELECT max(id) FROM webcams");
	list($maxa) = \Avos\Database\queryList("SELECT max(id) FROM webcamsArchive");
	
	if ($maxa > $maxw) {
		$maxw = $maxa;
	}
	
	\Avos\Cron\debug("max webcams.id=$maxw webcamsArchive.id=$maxa");
	
	//
	// moving dailyStats
	//
	
	if ($mode == 'daily') {
        $moveDailyStats = getConfig('webcams', 'moveDailyStats', 60);
        $models = [];
		
		if ((int)$moveDailyStats > 0) {

            //
            // search all keys in redis and skip any from the last $moveDailyStats days ago
            // with a massive dataset it can take several seconds to search keys for each date range
            // or model id so just search them all to minimise overhead
            //

			$today = date('ymd');
            $keys = $redis->keys('*/d/*');
            $numkeys = count($keys);

            \Avos\Cron\debug("moving dailyStats from $moveDailyStats days ago, today is $today. found $numkeys records");

            //
            // build list of keys from the last $moveDailyStats days ago that we will skip
            //

			$alldays = \Avos\Utils\yearsMonthsKeys((int)($moveDailyStats/30)+2, true);

            //
            // the key list contain every day from this month plus the previous $moveDailyStats
            // we need to remove keys before todays date $moveDailyStats months ago
            //
            // first cut off from today and forwards
            //

			$days = array();
		
			for ($i=0; $i<count($alldays) && $alldays[$i] != $today; $i++) {
				$days[] = $alldays[$i];
            }

            //
            // add today back in
            //

            $days[] = $today;
	
            //
            // if today is the 20th this will keep everything from and including the 20th of last month if $moveDailyStats == 30
            //

            $days = array_splice($days, -$moveDailyStats-2);

            //
            // create a hash so we can quickly check if the record should be skipped
            //

            $keep = [];
            foreach ($days as $d) {
                $keep[$d] = 1;
            }

            //
            // iterate round all keys and purge the data to sqlite unless it falls within $moveDailyStats
            //
		
            $count = 0;

            for ($i=0; $i<$numkeys; $i++) {
                $k = $keys[$i];

                //
                // split into model id and date
                //

                if (preg_match('!(\d+)\/d\/(\d+)!', $k, $c)) {

                    $id = $c[1];
                    $date = $c[2];
                    $models[$id] = 1;

                    //
                    // not sure why happens but sometimes are entries with id=0
                    //

                    if ($id == 0) {
                        $redis->del($k);
                        continue;
                    }

                    //
                    // skip the ones we want to keep
                    //

                    if (isset($keep[$date])) {
                        continue;
                    }

                    //
                    // purge the rest
                    // 

                    #echo "$i: $k \td$date \tm$id\n";

                    if ($redis->type($k) == \Redis::REDIS_STRING) {
                        $stat = $redis->get($k);

                        if (\Avos\SQLite\set($k, $stat) != false) {
                            $redis->del($k);
							if ($count++ % 1000 == 0) {
                                \Avos\Cron\debug("purged $count records");
								sleep(1);
							}
                        }
                    }
                }
            }
		}
		
		\Avos\Cron\debug("purged $count dailyStats records from " . count($models) . " models");
		return;
	}
	
	//
	// monthlyStats and ranks, $mode='monthly'
	//

    $conf = [
	    'h' => getConfig('webcams', 'moveStatusHistory', 2),
	    'm' => getConfig('webcams', 'moveMonthlyStats', 2),
        'r' => getConfig('webcams', 'moveRanks', 2)
    ];

    $name = [
        'h' => 'status',
        'm' => 'monthly',
        'r' => 'ranks'
    ];

    $type = [
        'h' => \Redis::REDIS_LIST,
        'r' => \Redis::REDIS_LIST,
        'm' => \Redis::REDIS_STRING
    ];

    $month = date('ym');
    $total = 0;
	\Avos\Cron\debug("month is $month");

    //
    // keys formatted like:
    //
    // 1000113/h/2502 -> Redis::REDIS_LIST
    // 1000113/m/2502 -> Redis::REDIS_STRING
    // 1000113/r/2502 -> Redis::REDIS_LIST
    //

    foreach (['h', 'm', 'r'] as $t) {
        if ((int)$conf[$t] == 0) {
            continue;
        }

        \Avos\Cron\debug("moving $name[$t] from $conf[$t] months ago");

        //
        // create a hash of months to skip
        //

        $months = \Avos\Utils\yearsMonthsKeys($conf[$t]);

        $keep = [];
        foreach ($months as $m) {
            $keep[$m] = 1;
        }

        //
        // iterate round all keys skipping the ones we want to keep
        //

        $keys = $redis->keys("*/$t/*");
        $numkeys = count($keys);
        $count = 0;

        for ($i=0; $i<$numkeys; $i++) {
            $k = $keys[$i];

            //
            // split into model id and date
            //

            if (preg_match("!(\d+)\/$t\/(\d+)!", $k, $c)) {

                $id = $c[1];
                $date = $c[2];

                if (!isset($keep[$date])) {
                    #echo "$k: id=$id date=$date\n";
                    $v = false;

                    switch ($type[$t]) {
                        case \Redis::REDIS_STRING:
                            $v = $redis->get($k);
                            break;
                        case \Redis::REDIS_LIST:
                            $v = $redis->lRange($k, 0, -1);
                            break;
                    }

                    if ($v !== false) {
                        if (\Avos\SQLite\set($k, $v) != false) {
                            $redis->del($k);
							if (($count++ + 1) % 1000 == 0) {
                                \Avos\Cron\debug("purged $count $name[$t] records");
								sleep(1);
							}
                        }
                    }
                }
            }
        }

        \Avos\Cron\debug("purged $count $name[$t] records");
        $total += $count;
    }

    \Avos\Cron\debug("purged $total records");

    /*

	$day = date('j');
	$count = 1;

	\Avos\Cron\debug("moving statusHistory from $moveStatusHistory months ago");
	\Avos\Cron\debug("moving monthlyStats from $moveMonthlyStats months ago");
    \Avos\Cron\debug("moving ranks from $moveRanks months ago");


	// cut off moveStatusHistory months ago
	if ((int)$moveStatusHistory > 0) {
		$months = \Avos\Utils\yearsMonthsKeys($moveStatusHistory+1);
		$monthsStatusHistory = array_splice($months, 0, count($months) - $moveStatusHistory);
	}

	// cut off moveMonthlyStats months ago
	if ((int)$moveMonthlyStats > 0) {
		$months = \Avos\Utils\yearsMonthsKeys($moveMonthlyStats+1);
		$monthsStats = array_splice($months, 0, count($months) - $moveMonthlyStats);
	}

	// cut off moveMonthlyStats months ago
	if ((int)$moveRanks > 0) {
		$months = \Avos\Utils\yearsMonthsKeys($moveRanks+1);
		$monthsRanks = array_splice($months, 0, count($months) - $moveRanks);
	}

	for ($i=1; $i<=$maxw; $i++) {

		//
		// statusHistory
		//

		if ((int)$moveStatusHistory > 0) {
			foreach ($monthsStatusHistory as $month) {
				$key = "$i/h/$month";
				if ($redis->type($key) == \Redis::REDIS_LIST) {
					$stat = $redis->lRange($key, 0, -1);

					if (\Avos\SQLite\set($key, $stat) != false) {
						$redis->del($key);
						\Avos\Cron\debug($key);
					}

					if ($count++ % 500 == 0) {
						sleep(1);
					}
				}
			}
		}
		#else {
		#	\Avos\Cron\debug("never moving statusHistory");
		#}

		//
		// monthlyStats
		//

		if ((int)$moveMonthlyStats > 0) {
			foreach ($monthsStats as $month) {
				$key = "$i/m/$month";
				if ($redis->type($key) == \Redis::REDIS_STRING) {
					$stat = $redis->get($key);

					if (\Avos\SQLite\set($key, $stat) != false) {
						$redis->del($key);
						\Avos\Cron\debug($key);
					}

					if ($count++ % 500 == 0) {
						sleep(1);
					}
				}
			}
		}
		#else {
		#	\Avos\Cron\debug("never moving monthlyStats");
		#}

		//
		// ranks
		//

		if ((int)$moveRanks > 0) {
			foreach ($monthsRanks as $month) {
				$key = "$i/r/$month";
				if ($redis->type($key) == \Redis::REDIS_LIST) {
					$stat = $redis->lRange($key, 0, -1);

					if (\Avos\SQLite\set($key, $stat) != false) {
						$redis->del($key);
						\Avos\Cron\debug($key);
					}

					if ($count++ % 500 == 0) {
						sleep(1);
					}
				}
			}
		}
		#else {
		#	\Avos\Cron\debug("never moving ranks");
		#}
	}

    \Avos\Cron\debug("purged $count monthly records");
    */
} 

function purgeModels()
{
    global $_AVOS;

    $redis = \Avos\Redis\handle();
    $moveAllAfterDays = getConfig('webcams', 'moveAllAfterDays', 90);
    $oldestTimestamp = time() - $moveAllAfterDays * 60*60*24;
    $chunkSize = 100000;
    $chunk = 0;

    list($maxModelsId) = \Avos\Database\queryList("SELECT max(id) FROM webcamsArchive");

    \Avos\Cron\debug("maxModelsId=$maxModelsId oldest=" . date("Y-m-d H:i:s", $oldestTimestamp));

    //
    // go thru the archive in chunks to reduce memory usage
    //

    while ($chunk < $maxModelsId) {
        $models = \Avos\Database\queryAssoc("SELECT wa.id AS id, w.updatedOn AS updatedOn, wa.updatedOn AS archiveUpdatedOn FROM webcamsArchive wa LEFT JOIN webcams w ON wa.id=w.id WHERE wa.purged=0 AND wa.id > $chunk AND wa.updatedOn < $oldestTimestamp LIMIT $chunkSize");

        if (count($models) > 0) {
            \Avos\Cron\debug("chunk $chunk found " . count($models) . " models to purge from " . $models[0]['id']);

            foreach ($models as $m) {

                //
                // check the updatedOn value if exists in the webcams table as signifies a probable
                // screwup somehow in updating the archive (need to investigate what is causing the sql error)
                // sync updatedOn and skip
                //

                if ($m['updatedOn'] > $m['archiveUpdatedOn']) {
                    \Avos\Database\query("UPDATE webcamsArchive SET updatedOn=:updatedOn WHERE id=:id", $m);
                }
                else {
                    $type = $redis->type("$m[id]/p");

                    if ($type == \Redis::REDIS_HASH) {
                        #\Avos\Cron\debug("purging $m[id]");
                        $redis->del("$m[id]/p");
                    }
                    else {
                        \Avos\Database\query("UPDATE webcamsArchive SET purged=1 WHERE id=:id", $m);
                    }
                }
            }
        }

        $chunk += $chunkSize;
    }
}

function backups($backupnow = false)
{
	global $_AVOS;
	$conf = $_AVOS['config']['backups'];
    $redis = $_AVOS['redis'];

	$backups = \Avos\Redis\getPrefix('backups');
	$backupFolders = \Avos\Redis\getPrefix('backupFolders');
	$info = $redis->hGetAll($backups);
	$avosDir = $_AVOS['config']['site']['avosDir'];
	$tar = $_AVOS['config']['tools']['tar'];
	$bzip2 = $_AVOS['config']['tools']['bzip2'];
	$now = time();
	$keep = getConfig('backups', 'keepBackupsCount', 3);

	//
	// clean out old backups
	//

    /*
    $folders = $redis->lRange($backupFolders, 0, -1);
	$have = count($folders);
	$delete = array_slice($folders, 0, $have - $keep);

	foreach ($delete as $f) {
		$folder = $conf['backupsFolder'] . $f;
		\Avos\Cron\debug("rm -rf $folder");
		system("rm -rf $folder");
		$redis->blPop($backupFolders, 10);
    }
     */

    $dirs = \Avos\Utils\list_dirs($conf['backupsFolder']);
    print_r($info);

    for ($i=0; $i <= count($dirs) - $keep - 1; $i++) {
        $folder = "$conf[backupsFolder]/$dirs[$i]";
        // sanity check
        if (strlen($folder) > 10) {
            \Avos\Cron\debug("removing $folder");
		    system("rm -rf $folder");
        }
    }

	//
	// backup
	//

	if ($backupnow == true || !isset($info['lastEpoch']) || $info['lastEpoch'] < $now - $conf['backupIntervalDays'] * 86400 - 360) {

		//
		// create folder
		//

		$folder = $conf['backupsFolder'] . date('ymd');
		\Avos\Cron\debug("backing up to $folder");

		if (file_exists($folder)) {
            \Avos\Cron\debug("backup folder already exists");
            if ($backupnow == false) {
                return;
            }
		}
        elseif (@mkdir($folder, 0755) == false) {
			\Avos\Cron\debug("unable to create backup folder");
			return;
		}

		//
		// files
		//

		if ($conf['backupFiles'] == 1) {
			\Avos\Cron\debug("archiving files");
            system("cd '$avosDir' && cd .. && $tar czf $folder/files.tgz --exclude=avos/db/* --exclude=avos/incoming/* --exclude=avos/backups/* avos");

            // web folder
            $webDir = getConfig('site', 'webDir');
            $imagesUrl = getConfig('webcams', 'imagesUrl');
            $videosUrl = getConfig('videos', 'videosUrl');
            $photosUrl = getConfig('photos', 'photosUrl');

            system("cd '$webDir' && cd .. && $tar czf $folder/web.tgz --exclude=web$imagesUrl* --exclude=web$videosUrl --exclude=web$photosUrl web");
        }

		//
		// mysql
		//

		if ($conf['backupMysql'] == 1) {

			$mysql = $_AVOS['config']['tools']['mysqldump'];
			$hostname = $_AVOS['config']['mysql']['hostname'];
			$database = $_AVOS['config']['mysql']['database'];
			$username = $_AVOS['config']['mysql']['username'];
			$password = $_AVOS['config']['mysql']['password'];

			\Avos\Cron\debug("dumping mysql");
			system("$mysql -h $hostname -u $username -p$password $database > $folder/mysql.sql");

			\Avos\Cron\debug("compressing mysql");
			system("$bzip2 $folder/mysql.sql");
		}

		//
		// redis
		//

		if ($conf['backupRedis'] == 1) {
			\Avos\Cron\debug("copying redis snapshot");

			if (isset($_AVOS['config']['redis']['dumpsDir'])) {
				$dir = $_AVOS['config']['redis']['dumpsDir'];
			}
			else {
				$dir = "/var/lib/redis/";
			}

			system("cp ${dir}dump.rdb $folder/redis.rdb");

			\Avos\Cron\debug("compressing redis");
			system("$bzip2 $folder/redis.rdb");
		}

		//
		// sqlite
		//

		if ($conf['backupSQLite'] == 1) {
			\Avos\Cron\debug("achiving sqlite");
			system("cd '$avosDir' && $tar cf $folder/sqlite.tar db");

			\Avos\Cron\debug("compressing sqlite");
			system("$bzip2 $folder/sqlite.tar");
		}
		
		//$redis->rPush($backupFolders, date('ymd'));
		$redis->hSet($backups, 'lastEpoch', $now);
	}
	else {
		$date = date('ymd', $info['lastEpoch']);
		\Avos\Cron\debug("too soon to backup, last backup was created on $date");
	}
}

function refreshSiteTags()
{
	global $_AVOS;

	$tags = getSiteTags();
	$_AVOS['redis']->set('siteTags', $tags);
}

function getSiteTags()
{
	// analise tags usage per site
	\Avos\Database\disableCache();
	
	$alltags = array();
	
	$Archive = '';
	$Archive = 'Archive';
	$q = \Avos\Database\queryAssoc("SELECT site,tags FROM webcams$Archive WHERE tags!=''");
	$tags = array();
	
	foreach ($q as $i => $m) {
	
		$tags = explode(',', strtolower(trim($m['tags'], ',')));
	
		foreach ($tags as $t) {
			if (!isset($alltags[$t])) {
				$alltags[$t] = array('cb' => 0, 'bc' => 0, 'sc' => 0, 'cs' => 0, 'ff' => 0, 'sm' => 0, 'sr' => 0, 'lj' => 0, 'so' => 0);
			}
	
			$alltags[$t][$m['site']]++;
			#echo "$i $m[site] $t\n";
		}
	}
	
	$keys = array_keys($alltags);
	sort($keys);
	$min = 200;
	$ret = array();
	
	foreach ($keys as $k) {
		$count = 
			$alltags[$k]['cb'] +
			$alltags[$k]['bc'] +
			$alltags[$k]['sc'] +
			$alltags[$k]['cs'] +
			$alltags[$k]['ff'] +
			$alltags[$k]['sm'] +
			$alltags[$k]['sr'] +
			$alltags[$k]['lj'];
	
		if ($count > $min) {
			$ret[$k] = $alltags[$k];
		}
	}
	
	return $ret;
}
