<?
namespace Avos\Photos;

function cron($mode='import')
{
	global $_AVOS;

	\Avos\Cron\debugModule('photos');

	switch ($mode) {

		case 'import':
			\Avos\Photos\import();
			\Avos\Cron\refreshPhotosTags();
			break;

		case 'actions':
			\Avos\Photos\actions();
			\Avos\Cron\refreshPhotosTags();
			break;

		case 'publish':
			\Avos\Photos\publish();
			break;
	}
}

function import()
{
	global $_AVOS;

	$incoming = $_AVOS['config']['photos']['incomingDir'];
	$trash = $_AVOS['config']['photos']['trashDir'];

	$top = $_AVOS['config']['site']['avosDir'];
	$web = $_AVOS['config']['site']['webDir'];
	$unzip = $_AVOS['config']['tools']['unzip'];
	$convert = $_AVOS['config']['tools']['convert'];
	$thumbsSize = $_AVOS['config']['photos']['thumbsSize'];

	foreach (\Avos\Utils\list_files($top . $incoming) as $input) {
		if (preg_match('!(.*)\.zip$!', $input, $c)) {

			//
			// folder name to use
			//

			$folder = \Avos\Utils\urlsafe(strtolower($c[1]));
			$info = array('folder' => $folder);
			$input_esc = escapeshellarg($input);

			//
			// check its not already been imported
			//

			$chk = \Avos\Database\queryList('
								SELECT id 
								FROM photos 
								WHERE folder=:folder', 
								$info
								);

			if(count($chk) == 0) {

				//
				// fetch photo info
				//

				#$input = $top . $incoming . $input;
				$path = photoPath($folder, true);
				\Avos\Cron\debug("importing $top$incoming$input");

				$zip = new \ZipArchive; 
				if ($zip->open($top . $incoming . $input) == TRUE) { 

					//
					// unzip to temp folder***
					//	

					$temp = "/tmp/avos.photos/";

					if (file_exists($temp)) {
						system("rm -rf $temp");
					}

					mkdir($temp);
					$cmd = "$unzip -oq $top$incoming$input_esc -d $temp";
					system($cmd);

					//
					// parse the zip file
					//

					$files = array();

					for($i=0; $i<$zip->numFiles; $i++) {   

						// can be in a subfolder
						if (preg_match('!(.*/)?(.*\.jpe?g)!i', $zip->getNameIndex($i), $c)) {

							// skip dotfiles
							if ($c[2][0] != '.') {
								$files[] = array(
									'full' => $c[1] . $c[2],
									'file' => $c[2]
								);
							}
						}
					} 

					//
					// create target folder
					//

					$target = $web . photoPath($folder, true);
					\Avos\Cron\debug("target is $target");

					if (!file_exists($target)) {
						mkdir($target, 0755, true);
					}

					//
					// move files to target
					//

					$info['numPhotos'] = count($files);
					foreach ($files as $i => $f) {
						$from = $temp . $f['full'];
						$to = $target . ($i+1) . '.jpg';
						rename($from, $to);
					}

					//
					// create thumbs
					//

					\Avos\Cron\debug("converting $info[numPhotos] thumbnails");
					foreach ($files as $i => $f) {
						$jpg = escapeshellarg($target . ($i+1) . '.jpg');
						$thm = escapeshellarg($target . 't' . ($i+1) . '.jpg');
						$cmd = "$convert -resize $thumbsSize $jpg $thm";
						system($cmd);
					}

					//
					// move zip to trash
					//

					$from = $top . $incoming . $input;
					$to = $top . $trash . $input;
					rename($from, $to);

					//
					// remove temp folder***
					//

					if (file_exists($temp)) {
						system("rm -rf $temp");
					}

					//
					// merge in original zip filename then add mysql entry
					//

					$info['origFilename'] = $input;

					\Avos\Database\query('
						INSERT INTO photos (title, urlTitle, folder, origFilename, numPhotos, addedOn)
						VALUES (:folder, :folder, :folder, :origFilename, :numPhotos, UNIX_TIMESTAMP())
						',
						$info);

				} 
				else {
					\Avos\Cron\debug("error reading $top$incoming$input");
				}



			}
		}
	}
}

function publish()
{
	global $_AVOS;
	$redis = $_AVOS['redis'];

	$stats = $redis->get("photos");

	if (!is_array($stats)) {
		$stats = array(
			'lastPublished' => time() - 86400
		);
	}

	$publishPerDay = $_AVOS['config']['photos']['publishPerDay'];
	$lastPublished = $stats['lastPublished'];
	$elapsed = time() - $lastPublished;
	$publishInterval = 86400 / $publishPerDay;

	\Avos\Cron\debug("lastPublished = $lastPublished, publishInterval = $publishInterval, elapsed = $elapsed");

	if ($elapsed > $publishInterval) {
		
		//
		// need to publish a photo and set publishedOn randonly between
		// time() and time() - $publishTime
		//

		$publishedOnPast = rand(0, $publishInterval);
		$publishedOnEpoch = time() - $publishedOnPast;
		\Avos\Cron\debug("want to publish a photo $publishedOnPast seconds ago");

		$chk = \Avos\Database\queryList("SELECT id FROM photos WHERE publishedOn=0 AND description!='' ORDER BY RAND() LIMIT 1");

		if (count($chk) == 1) {

			\Avos\Cron\debug("publishing photos $chk[0]");
			\Avos\Database\query("UPDATE photos SET publishedOn=$publishedOnEpoch WHERE id=$chk[0]");

			$stats['lastPublished'] = time();
			$redis->set("photos", $stats);
		}
	}
}


function actions()
{
	global$_AVOS;

	$web = $_AVOS['config']['site']['webDir'];
	$actions = \Avos\Database\queryAssoc("SELECT pa.*, p.folder AS ofolder FROM photosActions pa LEFT JOIN photos p ON pa.photosId=p.id");

	foreach ($actions as $a) {
		switch ($a['action']) {
			case 'delete':
				if (strlen($a['info']) > 8) {
					$folder = rtrim($web, '/') . photoPath($a['info'], true); 

					if (file_exists($folder)) {

						\Avos\Cron\debug("deleting photo gallery id=$a[photosId] folder=$a[info]");
						$files = \Avos\Utils\list_files($folder);

						foreach ($files as $f) {
							unlink($folder . $f);
						}

						rmdir($folder);
					}

					\Avos\Database\query("DELETE FROM photosActions WHERE id=:id", $a);
				}
				break;

			case 'rename':

				$otop = rtrim($web, '/') . photoPath($a['ofolder'], false); 
				$ofolder = $a['ofolder'];
				$nfolder = \Avos\Utils\urlsafe(strtolower($a['info']));
				$ntop = rtrim($web, '/') . photoPath($nfolder, false); 

				echo "$otop$ofolder\n";
				echo "$ntop$nfolder\n";

				if ($ofolder != $nfolder) {

					//
					// need to check the folder doesnt already exist
					//

					if (file_exists($ntop . $nfolder)) {
						$i = 1;
						while (file_exists($ntop . $nfolder . '-' . $i) && $i < 100) {
							$i++;
						}

						if ($i == 100) {
							\Avos\Cron\debug("unable to rename photo id=$a[photosId] folder=$nfolder");
							return;
						}

						$nfolder .= "-$i";
					}

					\Avos\Cron\debug("renaming photo gallery id=$a[photosId] folder=$nfolder");

					//
					// rename the folder
					//

					if (!file_exists($ntop)) {
						mkdir($ntop, 0755, true);
					}

					$old = $otop . $ofolder;
					$new = $ntop . $nfolder;
					rename($old, $new);

					//
					// update the photos entry
					//
				// for some reason here was set urlTitle=:url (that bug the image path if change the title)
					\Avos\Database\query("UPDATE photos SET urlTitle=:url, folder=:folder WHERE id=:id",
						array('url' => $nfolder, 'folder' => $nfolder, 'id' => $a['photosId'])
					);
				}

				\Avos\Database\query("DELETE FROM photosActions WHERE id=:id", $a);

				break;
		}
	}
}

function photoPath($v, $full=false)
{
	global $_AVOS;

	
	if (preg_match('!^((.)(.).*)!', $v, $c)) {
		$top = $_AVOS['config']['photos']['photosUrl'];
		$path = "$top$c[2]/$c[3]/";

		if ($full) {
			return strtolower($path . "$c[1]/");
		}
		else {
			return strtolower($path);
		}
	}
	else {
		return '';
	}
}


